@@ -9,7 +9,7 @@
 #define GENSIO_BASE_H
 
 #include <gensio/gensio_dllvisibility.h>
-#include <gensio/gensio.h>
+#include <gensio/gensio_types.h>
 
 struct gensio_filter;
 
@@ -43,6 +43,18 @@ typedef int (*gensio_ll_filter_data_handler)(void *cb_data,
  */
 #define GENSIO_FILTER_CB_STOP_TIMER	3
 
+/*
+ * Run a control on the child.
+ */
+struct gensio_filter_cb_control_data {
+    int depth;
+    bool get;
+    unsigned int option;
+    char *data;
+    gensiods *datalen;
+};
+#define GENSIO_FILTER_CB_CONTROL	4
+
 typedef int (*gensio_filter_cb)(void *cb_data, int func, void *data);
 
 
@@ -58,7 +70,7 @@ void gensio_filter_set_callback(struct gensio_filter *filter,
 				gensio_filter_cb cb, void *cb_data);
 
 /*
- * Is there data ready to be read from the top of the filter? 
+ * Is there data ready to be read from the top of the filter?
  */
 #define GENSIO_FILTER_FUNC_UL_READ_PENDING	2
 GENSIO_DLL_PUBLIC
