@@ -15,25 +15,236 @@
 #include <assert.h>
 
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
+#include <gensio/gensio_addr.h>
+#include <gensio/argvutils.h>
 #include <gensio/gensio_builtins.h>
 #include <gensio/gensio_class.h>
+#include <gensio/gensio_list.h>
+#include <gensio/gensio_time.h>
 
-#include "utils.h"
 #include "gensio_net.h"
 
 static unsigned int gensio_log_mask =
     (1 << GENSIO_LOG_FATAL) | (1 << GENSIO_LOG_ERR);
 
+static void check_flush_sync_io(struct gensio *io);
+
 struct gensio_classobj {
     const char *name;
     void *classdata;
+    struct gensio_classops *ops;
     struct gensio_classobj *next;
 };
 
+void
+gensio_os_funcs_set_vlog(struct gensio_os_funcs *o, gensio_vlog_func func)
+{
+    o->vlog = func;
+}
+
+gensio_vlog_func *
+gensio_os_funcs_get_vlog(struct gensio_os_funcs *o)
+{
+    return o->vlog;
+}
+
+void
+gensio_os_funcs_free(struct gensio_os_funcs *o)
+{
+    o->free_funcs(o);
+}
+
+void *
+gensio_os_funcs_zalloc(struct gensio_os_funcs *o, gensiods size)
+{
+    return o->zalloc(o, size);
+}
+
+void
+gensio_os_funcs_zfree(struct gensio_os_funcs *o, void *data)
+{
+    o->free(o, data);
+}
+
+struct gensio_waiter *
+gensio_os_funcs_alloc_waiter(struct gensio_os_funcs *o)
+{
+    return o->alloc_waiter(o);
+}
+
+void
+gensio_os_funcs_free_waiter(struct gensio_os_funcs *o,
+			    struct gensio_waiter *waiter)
+{
+    o->free_waiter(waiter);
+}
+
+int
+gensio_os_funcs_wait(struct gensio_os_funcs *o,
+		     struct gensio_waiter *waiter, unsigned int count,
+		     gensio_time *timeout)
+{
+    return o->wait(waiter, count, timeout);
+}
+
+int
+gensio_os_funcs_wait_intr(struct gensio_os_funcs *o,
+			  struct gensio_waiter *waiter, unsigned int count,
+			  gensio_time *timeout)
+{
+    return o->wait_intr(waiter, count, timeout);
+}
+
+int
+gensio_os_funcs_wait_intr_sigmask(struct gensio_os_funcs *o,
+				  struct gensio_waiter *waiter,
+				  unsigned int count,
+				  gensio_time *timeout,
+				  struct gensio_os_proc_data *proc_data)
+{
+    return o->wait_intr_sigmask(waiter, count, timeout, proc_data);
+}
+
+void
+gensio_os_funcs_wake(struct gensio_os_funcs *o,
+		     struct gensio_waiter *waiter)
+{
+    o->wake(waiter);
+}
+
+struct gensio_lock *
+gensio_os_funcs_alloc_lock(struct gensio_os_funcs *o)
+{
+    return o->alloc_lock(o);
+}
+
+void
+gensio_os_funcs_free_lock(struct gensio_os_funcs *o,
+			  struct gensio_lock *lock)
+{
+    o->free_lock(lock);
+}
+
+void
+gensio_os_funcs_lock(struct gensio_os_funcs *o,
+		     struct gensio_lock *lock)
+{
+    o->lock(lock);
+}
+
+void
+gensio_os_funcs_unlock(struct gensio_os_funcs *o,
+		       struct gensio_lock *lock)
+{
+    o->unlock(lock);
+}
+
+void
+gensio_os_funcs_get_monotonic_time(struct gensio_os_funcs *o,
+				   gensio_time *time)
+{
+    o->get_monotonic_time(o, time);
+}
+
+struct gensio_timer *
+gensio_os_funcs_alloc_timer(struct gensio_os_funcs *o,
+			    void (*handler)(struct gensio_timer *t,
+					    void *cb_data),
+			    void *cb_data)
+{
+    return o->alloc_timer(o, handler, cb_data);
+}
+
+void
+gensio_os_funcs_free_timer(struct gensio_os_funcs *o,
+			   struct gensio_timer *timer)
+{
+    o->free_timer(timer);
+}
+
+int
+gensio_os_funcs_start_timer(struct gensio_os_funcs *o,
+			    struct gensio_timer *timer,
+			    gensio_time *timeout)
+{
+    return o->start_timer(timer, timeout);
+}
+
+int gensio_os_funcs_start_timer_abs(struct gensio_os_funcs *o,
+				    struct gensio_timer *timer,
+				    gensio_time *timeout)
+{
+    return o->start_timer_abs(timer, timeout);
+}
+
+int
+gensio_os_funcs_stop_timer(struct gensio_os_funcs *o,
+			   struct gensio_timer *timer)
+{
+    return o->stop_timer(timer);
+}
+
+int
+gensio_os_funcs_stop_timer_with_done(struct gensio_os_funcs *o,
+				 struct gensio_timer *timer,
+				 void (*done_handler)(struct gensio_timer *t,
+						      void *cb_data),
+				 void *cb_data)
+{
+    return o->stop_timer_with_done(timer, done_handler, cb_data);
+}
+
+struct gensio_runner *
+gensio_os_funcs_alloc_runner(struct gensio_os_funcs *o,
+			     void (*handler)(struct gensio_runner *r,
+					     void *cb_data),
+			     void *cb_data)
+{
+    return o->alloc_runner(o, handler, cb_data);
+}
+
+void
+gensio_os_funcs_free_runner(struct gensio_os_funcs *o,
+			    struct gensio_runner *runner)
+{
+    o->free_runner(runner);
+}
+
+int
+gensio_os_funcs_run(struct gensio_os_funcs *o,
+		    struct gensio_runner *runner)
+{
+    return o->run(runner);
+}
+
+int
+gensio_os_funcs_service(struct gensio_os_funcs *o, gensio_time *timeout)
+{
+    return o->service(o, timeout);
+}
+
+int
+gensio_os_funcs_handle_fork(struct gensio_os_funcs *o)
+{
+    return o->handle_fork(o);
+}
+
+void
+gensio_os_funcs_set_data(struct gensio_os_funcs *o, void *data)
+{
+    o->other_data = data;
+}
+
+void *
+gensio_os_funcs_get_data(struct gensio_os_funcs *o)
+{
+    return o->other_data;
+}
+
 static int
-gen_addclass(struct gensio_os_funcs *o,
-	     struct gensio_classobj **classes,
-	     const char *name, void *classdata)
+gen_addclass(struct gensio_os_funcs *o, struct gensio_classobj **classes,
+	     const char *name, struct gensio_classops *ops, void *classdata)
 {
     struct gensio_classobj *c;
 
@@ -41,6 +252,7 @@ gen_addclass(struct gensio_os_funcs *o,
     if (!c)
 	return GE_NOMEM;
     c->name = name;
+    c->ops = ops;
     c->classdata = classdata;
     c->next = *classes;
     *classes = c;
@@ -65,14 +277,13 @@ struct gensio_nocbwait {
     struct gensio_link link;
 };
 
-struct gensio_sync_io;
-
 struct gensio {
     struct gensio_os_funcs *o;
     void *user_data;
     gensio_event cb;
     unsigned int cb_count;
     struct gensio_list waiters;
+    unsigned int refcount;
     struct gensio_lock *lock;
 
     struct gensio_classobj *classes;
@@ -96,8 +307,6 @@ struct gensio {
     struct gensio_sync_io *sync_io;
 
     struct gensio_link link;
-
-    struct gensio_link glink;
 };
 
 static struct gensio_os_funcs *o_base;
@@ -147,6 +356,7 @@ gensio_data_alloc(struct gensio_os_funcs *o,
     io = o->zalloc(o, sizeof(*io));
     if (!io)
 	return NULL;
+    io->refcount = 1;
 
     io->lock = o->alloc_lock(o);
     if (!io->lock) {
@@ -162,6 +372,21 @@ gensio_data_alloc(struct gensio_os_funcs *o,
     io->gensio_data = gensio_data;
     io->child = child;
 
+    if (child) {
+	struct gensio_classobj *c = child->classes;
+
+	while (c) {
+	    if (c->ops && c->ops->propagate_to_parent) {
+		int rv = c->ops->propagate_to_parent(io, child, c->classdata);
+		if (rv) {
+		    gensio_data_free(io);
+		    return NULL;
+		}
+	    }
+	    c = c->next;
+	}
+    }
+
     o_base->lock(gensio_base_lock);
     num_alloced_gensios++;
     o_base->unlock(gensio_base_lock);
@@ -182,6 +407,8 @@ gensio_data_free(struct gensio *io)
     while (io->classes) {
 	struct gensio_classobj *c = io->classes;
 
+	if (c->ops && c->ops->cleanup)
+	    c->ops->cleanup(io, c->classdata);
 	io->classes = c->next;
 	io->o->free(io->o, c);
     }
@@ -240,9 +467,10 @@ gensio_cb(struct gensio *io, int event, int err,
 }
 
 int
-gensio_addclass(struct gensio *io, const char *name, void *classdata)
+gensio_addclass(struct gensio *io, const char *name, int classops_ver,
+		struct gensio_classops *ops, void *classdata)
 {
-    return gen_addclass(io->o, &io->classes, name, classdata);
+    return gen_addclass(io->o, &io->classes, name, ops, classdata);
 }
 
 void *
@@ -261,6 +489,44 @@ gensio_getclass(struct gensio *io, const char *name)
     return rv;
 }
 
+struct gensio_acc_classobj {
+    const char *name;
+    void *classdata;
+    struct gensio_acc_classops *ops;
+    struct gensio_acc_classobj *next;
+};
+
+static int
+gen_acc_addclass(struct gensio_os_funcs *o,
+		 struct gensio_acc_classobj **classes,
+		 const char *name, struct gensio_acc_classops *ops,
+		 void *classdata)
+{
+    struct gensio_acc_classobj *c;
+
+    c = o->zalloc(o, sizeof(*c));
+    if (!c)
+	return GE_NOMEM;
+    c->name = name;
+    c->ops = ops;
+    c->classdata = classdata;
+    c->next = *classes;
+    *classes = c;
+    return 0;
+}
+
+static void *
+gen_acc_getclass(struct gensio_acc_classobj *classes, const char *name)
+{
+    struct gensio_acc_classobj *c;
+
+    for (c = classes; c; c = c->next) {
+	if (strcmp(c->name, name) == 0)
+	    return c->classdata;
+    }
+    return NULL;
+}
+
 struct gensio_accepter {
     struct gensio_os_funcs *o;
 
@@ -268,7 +534,7 @@ struct gensio_accepter {
     gensio_accepter_event cb;
     struct gensio_lock *lock;
 
-    struct gensio_classobj *classes;
+    struct gensio_acc_classobj *classes;
 
     const struct gensio_accepter_functions *funcs;
     gensio_acc_func func;
@@ -325,6 +591,21 @@ gensio_acc_data_alloc(struct gensio_os_funcs *o,
     gensio_list_init(&acc->waiting_ios);
     gensio_list_init(&acc->waiting_accepts);
 
+    if (child) {
+	struct gensio_acc_classobj *c = child->classes;
+
+	while (c) {
+	    if (c->ops && c->ops->propagate_to_parent) {
+		int rv = c->ops->propagate_to_parent(acc, child, c->classdata);
+		if (rv) {
+		    gensio_acc_data_free(acc);
+		    return NULL;
+		}
+	    }
+	    c = c->next;
+	}
+    }
+
     return acc;
 }
 
@@ -335,8 +616,10 @@ gensio_acc_data_free(struct gensio_accepter *acc)
 	acc->frdata->freed(acc, acc->frdata);
 
     while (acc->classes) {
-	struct gensio_classobj *c = acc->classes;
+	struct gensio_acc_classobj *c = acc->classes;
 
+	if (c->ops && c->ops->cleanup)
+	    c->ops->cleanup(acc, c->classdata);
 	acc->classes = c->next;
 	acc->o->free(acc->o, c);
     }
@@ -386,15 +669,17 @@ gensio_acc_cb(struct gensio_accepter *acc, int event, void *data)
 
 int
 gensio_acc_addclass(struct gensio_accepter *acc,
-		    const char *name, void *classdata)
+		    const char *name, int classops_ver,
+		    struct gensio_acc_classops *ops,
+		    void *classdata)
 {
-    return gen_addclass(acc->o, &acc->classes, name, classdata);
+    return gen_acc_addclass(acc->o, &acc->classes, name, ops, classdata);
 }
 
 void *
 gensio_acc_getclass(struct gensio_accepter *acc, const char *name)
 {
-    return gen_getclass(acc->classes, name);
+    return gen_acc_getclass(acc->classes, name);
 }
 
 const char *
@@ -729,8 +1014,13 @@ gensio_get_child(struct gensio *io, unsigned int depth)
 int
 gensio_close(struct gensio *io, gensio_done close_done, void *close_data)
 {
-    return io->func(io, GENSIO_FUNC_CLOSE, NULL, close_done, 0, close_data,
-		    NULL);
+    int rv;
+
+    rv = io->func(io, GENSIO_FUNC_CLOSE, NULL, close_done, 0, close_data,
+		  NULL);
+    if (!rv)
+	check_flush_sync_io(io);
+    return rv;
 }
 
 struct gensio_close_s_data {
@@ -778,7 +1068,16 @@ gensio_disable(struct gensio *io)
 void
 gensio_free(struct gensio *io)
 {
-    io->func(io, GENSIO_FUNC_FREE, NULL, NULL, 0, NULL, NULL);
+    struct gensio_os_funcs *o = io->o;
+    unsigned int count;
+
+    o->lock(io->lock);
+    count = --io->refcount;
+    o->unlock(io->lock);
+    if (count == 0) {
+	check_flush_sync_io(io);
+	io->func(io, GENSIO_FUNC_FREE, NULL, NULL, 0, NULL, NULL);
+    }
 }
 
 void
@@ -798,7 +1097,11 @@ gensio_set_write_callback_enable(struct gensio *io, bool enabled)
 void
 gensio_ref(struct gensio *io)
 {
-    io->func(io, GENSIO_FUNC_REF, NULL, NULL, 0, NULL, NULL);
+    struct gensio_os_funcs *o = io->o;
+
+    o->lock(io->lock);
+    io->refcount++;
+    o->unlock(io->lock);
 }
 
 bool
@@ -1243,6 +1546,12 @@ add_default_gensio_accepters(void *cb_data)
     REG_FILT_GENSIO_ACC(o, "perf", str_to_perf_gensio_accepter,
 			perf_gensio_accepter_alloc);
     REG_GENSIO_ACC(o, "conacc", str_to_conacc_gensio_accepter);
+    REG_FILT_GENSIO_ACC(o, "kiss", str_to_kiss_gensio_accepter,
+			kiss_gensio_accepter_alloc);
+    REG_FILT_GENSIO_ACC(o, "ax25", str_to_ax25_gensio_accepter,
+			ax25_gensio_accepter_alloc);
+    REG_FILT_GENSIO_ACC(o, "xlt", str_to_xlt_gensio_accepter,
+			xlt_gensio_accepter_alloc);
 }
 
 int
@@ -1452,6 +1761,14 @@ add_default_gensios(void *cb_data)
 		    trace_gensio_alloc);
     REG_FILT_GENSIO(o, "perf", str_to_perf_gensio,
 		    perf_gensio_alloc);
+    REG_FILT_GENSIO(o, "kiss", str_to_kiss_gensio,
+		    kiss_gensio_alloc);
+    REG_FILT_GENSIO(o, "ax25", str_to_ax25_gensio,
+		    ax25_gensio_alloc);
+    REG_FILT_GENSIO(o, "xlt", str_to_xlt_gensio,
+		    xlt_gensio_alloc);
+    REG_FILT_GENSIO(o, "keepopen", str_to_keepopen_gensio,
+		    keepopen_gensio_alloc);
 }
 
 int
@@ -1897,7 +2214,8 @@ gensio_vlog(struct gensio_os_funcs *o, enum gensio_log_levels level,
     if (!(gensio_log_mask & (1 << level)))
 	return;
 
-    o->vlog(o, level, str, args);
+    if (o->vlog)
+	o->vlog(o, level, str, args);
 }
 
 void
@@ -2513,7 +2831,7 @@ gensio_get_default(struct gensio_os_funcs *o,
 		err = GE_NOMEM;
 		goto out_unlock;
 	    }
-	    memcpy(str, val->strval, val->intval + 1L); /* copy emding \0 */
+	    memcpy(str, val->strval, (size_t) val->intval + 1); /* copy emding \0 */
 	    *strval = str;
 	    *intval = val->intval;
 	} else {
@@ -2562,7 +2880,7 @@ gensio_del_default(struct gensio_os_funcs *o,
 	    prevc->next = c->next;
 	else
 	    d->classvals = c->next;
-	
+
 	if (c->val.strval)
 	    o->free(o, c->val.strval);
 	o->free(o, c->class);
@@ -2803,6 +3121,21 @@ gensio_sync_flush_waiters(struct gensio_sync_io *sync_io,
     }
 }
 
+static void
+check_flush_sync_io(struct gensio *io)
+{
+    struct gensio_sync_io *sync_io = io->sync_io;
+    struct gensio_os_funcs *o = io->o;
+
+    if (sync_io) {
+	o->lock(sync_io->lock);
+	if (!sync_io->err)
+	    sync_io->err = GE_LOCALCLOSED;
+	gensio_sync_flush_waiters(sync_io, io->o);
+	o->unlock(sync_io->lock);
+    }
+}
+
 static int
 gensio_syncio_event(struct gensio *io, void *user_data,
 		    int event, int err,
@@ -3302,7 +3635,7 @@ gensio_alloc_vsprintf(struct gensio_os_funcs *o, const char *fmt, va_list va)
     char c[1], *str;
 
     va_copy(va2, va);
-    len = vsnprintf(c, 0, fmt, va) + 1L;
+    len = (size_t) vsnprintf(c, 0, fmt, va) + 1L;
     str = o->zalloc(o, len);
     if (str)
 	vsnprintf(str, len, fmt, va2);
@@ -3392,7 +3725,12 @@ static const char *gensio_errs[] = {
     /*  34 */    "Operation on shutdown fd",
     /*  35 */    "Local end closed connection",
     /*  36 */    "Permission denied",
-    /*  37 */    "Application error"
+    /*  37 */    "Application error",
+    /*  38 */	 "Unknown name server lookup failure",
+    /*  39 */	 "Unable to find a valid name on the name server",
+    /*  40 */	 "Serious name server failure",
+    /*  41 */	 "Invalid name server information",
+    /*  42 */	 "Network address for the given name is not available"
 };
 const unsigned int errno_len = sizeof(gensio_errs) / sizeof(char *);
 
@@ -3417,34 +3755,42 @@ gensio_addr_equal(const struct gensio_addr *a1,
 		  const struct gensio_addr *a2,
 		  bool compare_ports, bool compare_all)
 {
-    return a1->o->addr_equal(a1, a2, compare_ports, compare_all);
+    if (a1->funcs != a2->funcs)
+	return false;
+    return a1->funcs->addr_equal(a1, a2, compare_ports, compare_all);
 }
 
 int
 gensio_addr_to_str(const struct gensio_addr *addr,
 		   char *buf, gensiods *pos, gensiods buflen)
 {
-    return addr->o->addr_to_str(addr, buf, pos, buflen);
+    gensiods dummypos = 0;
+
+    if (!pos)
+	pos = &dummypos;
+    return addr->funcs->addr_to_str(addr, buf, pos, buflen);
 }
 
 int
 gensio_addr_to_str_all(const struct gensio_addr *addr,
 		       char *buf, gensiods *pos, gensiods buflen)
 {
-    return addr->o->addr_to_str_all(addr, buf, pos, buflen);
+    return addr->funcs->addr_to_str_all(addr, buf, pos, buflen);
 }
 
 struct gensio_addr *
 gensio_addr_dup(const struct gensio_addr *iaddr)
 {
-    return iaddr->o->addr_dup(iaddr);
+    return iaddr->funcs->addr_dup(iaddr);
 }
 
 struct gensio_addr *
 gensio_addr_cat(const struct gensio_addr *addr1,
 		const struct gensio_addr *addr2)
 {
-    return addr1->o->addr_cat(addr1, addr2);
+    if (addr1->funcs != addr2->funcs)
+	return NULL;
+    return addr1->funcs->addr_cat(addr1, addr2);
 }
 
 bool
@@ -3452,43 +3798,155 @@ gensio_addr_addr_present(const struct gensio_addr *gai,
 			 const void *addr, gensiods addrlen,
 			 bool compare_ports)
 {
-    return gai->o->addr_addr_present(gai, addr, addrlen, compare_ports);
+    return gai->funcs->addr_addr_present(gai, addr, addrlen, compare_ports);
 }
 
 void
 gensio_addr_free(struct gensio_addr *addr)
 {
-    addr->o->addr_free(addr);
+    addr->funcs->addr_free(addr);
 }
 
 bool
 gensio_addr_next(struct gensio_addr *addr)
 {
-    return addr->o->addr_next(addr);
+    return addr->funcs->addr_next(addr);
 }
 
 void
 gensio_addr_rewind(struct gensio_addr *addr)
 {
-    addr->o->addr_rewind(addr);
+    addr->funcs->addr_rewind(addr);
 }
 
 int
 gensio_addr_get_nettype(const struct gensio_addr *addr)
 {
-    return addr->o->addr_get_nettype(addr);
+    return addr->funcs->addr_get_nettype(addr);
 }
 
 bool
 gensio_addr_family_supports(const struct gensio_addr *addr,
 			    int family, int flags)
 {
-    return addr->o->addr_family_supports(addr, family, flags);
+    return addr->funcs->addr_family_supports(addr, family, flags);
 }
 
 void
 gensio_addr_getaddr(const struct gensio_addr *addr,
 		    void *oaddr, gensiods *rlen)
 {
-    addr->o->addr_getaddr(addr, oaddr, rlen);
+    addr->funcs->addr_getaddr(addr, oaddr, rlen);
+}
+
+void
+gensio_fdump_init(struct gensio_fdump *h, unsigned int indent)
+{
+    h->column = 0;
+    h->pos = 0;
+    h->indent = indent;
+}
+
+void
+gensio_fdump_buf(FILE *f, const unsigned char *buf, gensiods len,
+		 struct gensio_fdump *h)
+{
+    gensiods i, j;
+
+    for (i = 0; i < len; i++) {
+	if (h->column == 0)
+	    fprintf(f, "%*s%4.4x:", h->indent, "", h->pos);
+	fprintf(f, " %2.2x", buf[i]);
+	h->data[h->column++] = buf[i];
+	h->pos++;
+	if (h->column == 16) {
+	    fputs("  ", f);
+	    for (j = 0; j < 16; j++) {
+		if (isprint(h->data[j]))
+		    fputc(h->data[j], f);
+		else
+		    fputc('.', f);
+	    }
+	    fputc('\n', f);
+	    h->column = 0;
+	}
+    }
+}
+
+void
+gensio_fdump_buf_finish(FILE *f, struct gensio_fdump *h)
+{
+    gensiods i;
+
+    if (h->column == 0)
+	return;
+    for (i = h->column; i < 16; i++)
+	fputs("   ", f);
+    fputs("  ", f);
+    for (i = 0; i < h->column; i++) {
+	if (isprint(h->data[i]))
+	    fputc(h->data[i], f);
+	else
+	    fputc('.', f);
+    }
+    fputc('\n', f);
+}
+
+void
+gensio_time_add_nsecs(gensio_time *t, int64_t v)
+{
+    t->secs += v / GENSIO_NSECS_IN_SEC;
+    t->nsecs += v % GENSIO_NSECS_IN_SEC;
+    while (t->nsecs > GENSIO_NSECS_IN_SEC) {
+	t->secs++;
+	t->nsecs -= GENSIO_NSECS_IN_SEC;
+    }
+    while (t->nsecs < 0) {
+	t->secs--;
+	t->nsecs += GENSIO_NSECS_IN_SEC;
+    }
+}
+
+int64_t
+gensio_time_to_msecs(gensio_time *t1)
+{
+    int64_t v;
+
+    v = t1->secs * 1000;
+    v += GENSIO_NSECS_TO_MSECS(t1->nsecs);
+    return v;
+}
+
+int64_t
+gensio_time_to_usecs(gensio_time *t1)
+{
+    int64_t v;
+
+    v = t1->secs * 1000000;
+    v += GENSIO_NSECS_TO_USECS(t1->nsecs);
+    return v;
+}
+
+void
+gensio_msecs_to_time(gensio_time *t1, int64_t v)
+{
+    t1->secs = v / 1000;
+    t1->nsecs = GENSIO_MSECS_TO_NSECS(v % 1000);
+}
+
+void
+gensio_usecs_to_time(gensio_time *t1, int64_t v)
+{
+    t1->secs = v / 1000000;
+    t1->nsecs = GENSIO_USECS_TO_NSECS(v % 1000000);
+}
+
+int64_t
+gensio_time_diff_nsecs(gensio_time *t1, gensio_time *t2)
+{
+    int64_t v;
+
+    v = t1->secs - t2->secs;
+    v += (int64_t) t1->nsecs - (int64_t) t2->nsecs;
+    return v;
 }
