@@ -5,6 +5,7 @@
  *  SPDX-License-Identifier: LGPL-2.1-only
  */
 
+#define _GNU_SOURCE /* Get extended getaddrinfo errors. */
 #include "config.h"
 #include <stdbool.h>
 
@@ -12,6 +13,7 @@
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #else
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -26,7 +28,11 @@
 #include <sys/un.h>
 #endif
 
+
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
+#include <gensio/gensio_addr.h>
+#include <gensio/gensio_err.h>
 #include <gensio/gensio_osops_addrinfo.h>
 
 /* For older systems that don't have this. */
@@ -36,6 +42,7 @@
 
 struct gensio_addr_addrinfo {
     struct gensio_addr r;
+    struct gensio_os_funcs *o;
     struct addrinfo *a;
     struct addrinfo *curr;
 #if HAVE_GCC_ATOMICS
@@ -44,6 +51,8 @@ struct gensio_addr_addrinfo {
     bool is_getaddrinfo; /* Allocated with getaddrinfo()? */
 };
 
+static struct gensio_addr_funcs addrinfo_funcs;
+
 static void gensio_addr_addrinfo_free(struct gensio_addr *addr);
 
 #define a_to_info(a) gensio_container_of(a, struct gensio_addr_addrinfo, r);
@@ -77,7 +86,7 @@ gensio_addrinfo_make(struct gensio_os_funcs *o, unsigned int size,
 		     bool is_recvfrom)
 {
     struct gensio_addr_addrinfo *addr = o->zalloc(o, sizeof(*addr));
-    struct addrinfo *ai = NULL;
+    struct addrinfo *ai = NULL, *nai;
 
     if (!addr)
 	return NULL;
@@ -85,42 +94,57 @@ gensio_addrinfo_make(struct gensio_os_funcs *o, unsigned int size,
 #if HAVE_GCC_ATOMICS
     if (!is_recvfrom) {
 	addr->refcount = o->zalloc(o, sizeof(*addr->refcount));
-	if (!addr->refcount) {
-	    o->free(o, addr);
-	    return NULL;
-	}
+	if (!addr->refcount)
+	    goto out_err;
 	*addr->refcount = 1;
     }
 #endif
 
     if (size > 0) {
 	ai = o->zalloc(o, sizeof(*ai));
-	if (!ai) {
-#if HAVE_GCC_ATOMICS
-	    if (addr->refcount)
-		o->free(o, addr->refcount);
-#endif
-	    o->free(o, addr);
-	    return NULL;
-	}
+	if (!ai)
+	    goto out_err;
 
 	ai->ai_addr = o->zalloc(o, size);
-	if (!ai->ai_addr) {
-#if HAVE_GCC_ATOMICS
-	    if (addr->refcount)
-		o->free(o, addr->refcount);
-#endif
-	    o->free(o, addr);
-	    o->free(o, ai);
-	    return NULL;
-	}
+	if (!ai->ai_addr)
+	    goto out_err;
 	ai->ai_addrlen = size;
     }
-    addr->r.o = o;
+    if (is_recvfrom && ai) {
+	/* Tack on two more for room for ifindex and dest addr. */
+	unsigned int i;
+
+	nai = ai;
+	for (i = 0; i < 2; i++) {
+	    nai->ai_next = o->zalloc(o, sizeof(*ai));
+	    if (!nai->ai_next)
+		goto out_err;
+	    nai->ai_next->ai_addr = o->zalloc(o, size);
+	    if (!nai->ai_next->ai_addr)
+		goto out_err;
+	    nai = nai->ai_next;
+	}
+    }
+    addr->o = o;
+    addr->r.funcs = &addrinfo_funcs;
     addr->a = ai;
     addr->curr = ai;
 
     return addr;
+ out_err:
+#if HAVE_GCC_ATOMICS
+    if (addr->refcount)
+	o->free(o, addr->refcount);
+#endif
+    while (ai) {
+	nai = ai->ai_next;
+	if (ai->ai_addr)
+	    o->free(o, ai->ai_addr);
+	o->free(o, ai);
+	ai = nai;
+    }
+    o->free(o, addr);
+    return NULL;
 }
 
 struct gensio_addr *
@@ -387,6 +411,11 @@ gensio_sockaddr_to_str(const struct sockaddr *addr, int flags,
 
 	gensio_pos_snprintf(buf, buflen, pos, "unix,%s", au->sun_path);
 #endif
+    } else if (addr->sa_family == GENSIO_AF_IFINDEX) {
+	struct sockaddr *as = (struct sockaddr *) addr;
+	unsigned int *iptr = (unsigned int *) as->sa_data;
+
+	gensio_pos_snprintf(buf, buflen, pos, "ifidx:%u", *iptr);
     } else {
 	if (*pos < buflen)
 	    buf[*pos] = '\0';
@@ -756,7 +785,7 @@ gensio_addr_addrinfo_scan_ips(struct gensio_os_funcs *o, const char *str,
 	/*
 	 * If the user specified something like "tcp,0", ip will be
 	 * NULL and getaddrinfo will return IPv4 and IPv6 addresses if
-	 * they are available.  AF_V4MAPPED will be set, so we really
+	 * they are available.  AI_V4MAPPED will be set, so we really
 	 * only want IPv6 addresses (if any are available) as once you
 	 * open the IPv6 address you can't open the IPv4 address.
 	 */
@@ -784,7 +813,46 @@ gensio_addr_addrinfo_scan_ips(struct gensio_os_funcs *o, const char *str,
 		rv = 0;
 		goto ignore_getaddr_error;
 	    }
-	    rv = GE_INVAL;
+	    switch (rv) {
+#ifdef EAI_INTR
+	    case EAI_INTR:
+		goto redo_getaddrinfo;
+#endif
+	    case EAI_AGAIN:
+		rv = GE_RETRY;
+		break;
+#ifdef EAI_ADDRFAMILY
+	    case EAI_ADDRFAMILY:
+		rv = GE_NAME_NET_NOT_UP;
+		break;
+#endif
+	    case EAI_BADFLAGS:
+	    case EAI_FAMILY:
+	    case EAI_SOCKTYPE:
+		/* These mean the code here did something wrong. */
+		rv = GE_NAME_INVALID;
+		break;
+	    case EAI_FAIL:
+		rv = GE_NAME_SERVER_FAILURE;
+		break;
+	    case EAI_MEMORY:
+		rv = GE_NOMEM;
+		break;
+#ifdef EAI_ADDRFAMILY
+	    case EAI_NODATA:
+#endif
+	    case EAI_NONAME:
+		rv = GE_NAME_ERROR;
+		break;
+#ifdef EAI_SYSTEM
+	    case EAI_SYSTEM:
+		rv = gensio_os_err_to_err(o, errno);
+		break;
+#endif
+	    default:
+		rv = GE_UNKNOWN_NAME_ERROR;
+		break;
+	    }
 	    goto out_err;
 	}
 	gotaddr = true;
@@ -874,11 +942,12 @@ gensio_addr_addrinfo_dup(const struct gensio_addr *iaaddr)
     if (!iaaddr)
 	return NULL;
     iaddr = a_to_info(iaaddr);
-    o = iaddr->r.o;
+    o = iaddr->o;
     addr = o->zalloc(o, sizeof(*addr));
     if (!addr)
 	return NULL;
-    addr->r.o = o;
+    addr->o = o;
+    addr->r.funcs = &addrinfo_funcs;
 
 #if HAVE_GCC_ATOMICS
     if (iaddr->refcount) {
@@ -921,7 +990,7 @@ gensio_addr_addrinfo_cat(const struct gensio_addr *aaddr1,
 {
     struct gensio_addr_addrinfo *addr1 = a_to_info(aaddr1);
     struct gensio_addr_addrinfo *addr2 = a_to_info(aaddr2);
-    struct gensio_os_funcs *o = addr1->r.o;
+    struct gensio_os_funcs *o = addr1->o;
     struct gensio_addr_addrinfo *addr;
     struct addrinfo *aip = NULL;
     int rv;
@@ -973,7 +1042,7 @@ static void
 gensio_addr_addrinfo_free(struct gensio_addr *aaddr)
 {
     struct gensio_addr_addrinfo *addr = a_to_info(aaddr);
-    struct gensio_os_funcs *o = addr->r.o;
+    struct gensio_os_funcs *o = addr->o;
 
 #if HAVE_GCC_ATOMICS
     if (addr->refcount) {
@@ -1050,21 +1119,24 @@ gensio_addr_addrinfo_getaddr(const struct gensio_addr *aaddr,
     *rlen = addr->curr->ai_addrlen;
 }
 
+static struct gensio_addr_funcs addrinfo_funcs = {
+    .addr_equal = gensio_addr_addrinfo_equal,
+    .addr_to_str = gensio_addr_addrinfo_to_str,
+    .addr_to_str_all = gensio_addr_addrinfo_to_str_all,
+    .addr_dup = gensio_addr_addrinfo_dup,
+    .addr_cat = gensio_addr_addrinfo_cat,
+    .addr_addr_present = gensio_addr_addrinfo_addr_present,
+    .addr_free = gensio_addr_addrinfo_free,
+    .addr_next = gensio_addr_addrinfo_next,
+    .addr_rewind = gensio_addr_addrinfo_rewind,
+    .addr_get_nettype = gensio_addr_addrinfo_get_nettype,
+    .addr_family_supports = gensio_addr_addrinfo_family_supports,
+    .addr_getaddr = gensio_addr_addrinfo_getaddr
+};
+
 void
 gensio_addr_addrinfo_set_os_funcs(struct gensio_os_funcs *o)
 {
     o->addr_create = gensio_addr_addrinfo_create;
-    o->addr_equal = gensio_addr_addrinfo_equal;
-    o->addr_to_str = gensio_addr_addrinfo_to_str;
-    o->addr_to_str_all = gensio_addr_addrinfo_to_str_all;
-    o->addr_dup = gensio_addr_addrinfo_dup;
-    o->addr_cat = gensio_addr_addrinfo_cat;
-    o->addr_addr_present = gensio_addr_addrinfo_addr_present;
-    o->addr_free = gensio_addr_addrinfo_free;
-    o->addr_next = gensio_addr_addrinfo_next;
-    o->addr_rewind = gensio_addr_addrinfo_rewind;
-    o->addr_get_nettype = gensio_addr_addrinfo_get_nettype;
-    o->addr_family_supports = gensio_addr_addrinfo_family_supports;
-    o->addr_getaddr = gensio_addr_addrinfo_getaddr;
     o->addr_scan_ips = gensio_addr_addrinfo_scan_ips;
 }
