@@ -12,8 +12,10 @@
 #include <assert.h>
 #include <stdio.h>
 
+#include <gensio/gensio.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_base.h>
+#include <gensio/gensio_os_funcs.h>
 
 #ifdef DEBUG_DATA
 #define ENABLE_PRBUF 1
@@ -157,8 +159,6 @@ struct basen_data {
 
     unsigned int refcount;
 
-    unsigned int freeref;
-
     enum basen_state state;
 
     gensio_done_err open_done;
@@ -249,6 +249,8 @@ i_basen_unlock(struct basen_data *ndata)
 static void
 basen_finish_free(struct basen_data *ndata)
 {
+    if (ndata->io)
+	gensio_data_free(ndata->io);
     if (ndata->lock)
 	ndata->o->free_lock(ndata->lock);
     if (ndata->timer)
@@ -259,8 +261,6 @@ basen_finish_free(struct basen_data *ndata)
 	gensio_filter_free(ndata->filter);
     if (ndata->ll)
 	gensio_ll_free(ndata->ll);
-    if (ndata->io)
-	gensio_data_free(ndata->io);
     ndata->o->free(ndata->o, ndata);
 }
 
@@ -438,7 +438,7 @@ filter_ul_write(struct basen_data *ndata, gensio_ul_filter_data_handler handler,
 	return gensio_filter_ul_write(ndata->filter, handler,
 				      ndata, rcount, sg, sglen, auxdata);
     return handler(ndata, rcount, sg, sglen, auxdata);
-}	     
+}
 
 static int
 filter_ll_write(struct basen_data *ndata, gensio_ll_filter_data_handler handler,
@@ -452,7 +452,7 @@ filter_ll_write(struct basen_data *ndata, gensio_ll_filter_data_handler handler,
     if (buflen)
 	return handler(ndata, rcount, buf, buflen, auxdata);
     return 0;
-}	     
+}
 
 static int
 filter_setup(struct basen_data *ndata)
@@ -737,6 +737,7 @@ basen_read_data_handler(void *cb_data,
 {
     struct basen_data *ndata = cb_data;
     gensiods count = 0, rval;
+    int err = 0;
 
     basen_lock(ndata);
     if (!basen_can_deliver_ul_data(ndata)) {
@@ -754,14 +755,14 @@ basen_read_data_handler(void *cb_data,
 	   (count < buflen || ndata->ll_err)) {
 	if (ndata->ll_err && !filter_ul_read_pending(ndata)) {
 	    basen_unlock(ndata);
-	    gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->ll_err,
-		      NULL, NULL, NULL);
+	    err = gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->ll_err,
+			    NULL, NULL, NULL);
 	    basen_lock(ndata);
 	} else {
 	    basen_unlock(ndata);
 	    rval = buflen - count;
-	    gensio_cb(ndata->io, GENSIO_EVENT_READ, 0,
-		      buf + count, &rval, auxdata);
+	    err = gensio_cb(ndata->io, GENSIO_EVENT_READ, 0,
+			    buf + count, &rval, auxdata);
 #ifdef ENABLE_INTERNAL_TRACE
 	    /* Only for testing. */
 	    assert(rval <= buflen - count);
@@ -780,7 +781,7 @@ basen_read_data_handler(void *cb_data,
 
  out:
     *rcount = count;
-    return 0;
+    return err;
 }
 
 static void
@@ -994,18 +995,18 @@ basen_deferred_op(struct gensio_runner *runner, void *cbdata)
 		/* Automatically disable read on an error. */
 		ndata->read_enabled = false;
 		basen_unlock(ndata);
-		gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->ll_err,
-			  NULL, NULL, NULL);
+		err = gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->ll_err,
+				NULL, NULL, NULL);
 		basen_lock(ndata);
 	    } else {
 		basen_unlock(ndata);
 		err = filter_ll_write(ndata, basen_read_data_handler,
 				      NULL, NULL, 0, NULL);
 		basen_lock(ndata);
-		if (err) {
-		    handle_ioerr(ndata, err);
-		    break;
-		}
+	    }
+	    if (err) {
+		handle_ioerr(ndata, err);
+		break;
 	    }
 	} while (ndata->read_enabled &&
 		 (ndata->ll_err || filter_ul_read_pending(ndata)));
@@ -1023,8 +1024,13 @@ basen_deferred_op(struct gensio_runner *runner, void *cbdata)
 	       (filter_ll_can_write(ndata) || ndata->ll_err)
 	       && ndata->xmit_enabled) {
 	    basen_unlock(ndata);
-	    gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL, 0, NULL);
+	    err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL,
+			    0, NULL);
 	    basen_lock(ndata);
+	    if (err) {
+		handle_ioerr(ndata, err);
+		break;
+	    }
 	}
 	ndata->in_xmit_ready = false;
     }
@@ -1349,13 +1355,6 @@ static void
 basen_free(struct basen_data *ndata)
 {
     basen_lock(ndata);
-    i_basen_add_trace(ndata, 103, __LINE__);
-    assert(ndata->freeref > 0);
-    if (--ndata->freeref > 0) {
-	basen_unlock(ndata);
-	return;
-    }
-
     i_basen_add_trace(ndata, 103, __LINE__);
     switch (ndata->state) {
     case BASEN_CLOSED:
@@ -1390,14 +1389,6 @@ basen_free(struct basen_data *ndata)
     basen_deref_and_unlock(ndata);
 }
 
-static void
-basen_do_ref(struct basen_data *ndata)
-{
-    basen_lock(ndata);
-    ndata->freeref++;
-    basen_unlock(ndata);
-}
-
 static void
 basen_timeout(struct gensio_timer *timer, void *cb_data)
 {
@@ -1508,10 +1499,6 @@ gensio_base_func(struct gensio *io, int func, gensiods *count,
 	basen_free(ndata);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	basen_do_ref(ndata);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	basen_set_read_callback_enable(ndata, buflen);
 	return 0;
@@ -1581,6 +1568,7 @@ basen_ll_read(void *cb_data, int readerr,
 {
     struct basen_data *ndata = cb_data;
     unsigned char *buf = ibuf;
+    int err;
 
 #ifdef DEBUG_DATA
     printf("LL read:");
@@ -1649,8 +1637,13 @@ basen_ll_read(void *cb_data, int readerr,
 	       (filter_ll_can_write(ndata) || ndata->ll_err)
 	       && ndata->xmit_enabled) {
 	    basen_unlock(ndata);
-	    gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL, 0, NULL);
+	    err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL,
+			    0, NULL);
 	    basen_lock(ndata);
+	    if (err) {
+		handle_ioerr(ndata, err);
+		break;
+	    }
 	}
     }
 
@@ -1707,8 +1700,12 @@ basen_ll_write_ready(void *cb_data)
 	   (filter_ll_can_write(ndata) || ndata->ll_err)
 	   && ndata->xmit_enabled) {
 	basen_unlock(ndata);
-	gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL, 0, NULL);
+	err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0, NULL, 0, NULL);
 	basen_lock(ndata);
+	if (err) {
+	    handle_ioerr(ndata, err);
+	    goto out_setnotready;
+	}
     }
 
     if (ndata->redo_xmit_ready) {
@@ -1777,6 +1774,14 @@ basen_stop_timer_op(void *cb_data)
     }
 }
 
+static int
+basen_call_child_control_op(struct basen_data *ndata,
+			    struct gensio_filter_cb_control_data *ctrl)
+{
+    return gensio_control(ndata->child, ctrl->depth, ctrl->get, ctrl->option,
+			  ctrl->data, ctrl->datalen);
+}
+
 static int
 gensio_base_filter_cb(void *cb_data, int op, void *data)
 {
@@ -1793,6 +1798,10 @@ gensio_base_filter_cb(void *cb_data, int op, void *data)
 	basen_stop_timer_op(cb_data);
 	return 0;
 
+    case GENSIO_FILTER_CB_CONTROL:
+	basen_call_child_control_op(cb_data, data);
+	return 0;
+
     default:
 	return GE_NOTSUP;
     }
@@ -1815,7 +1824,6 @@ gensio_i_alloc(struct gensio_os_funcs *o,
 
     ndata->o = o;
     ndata->refcount = 1;
-    ndata->freeref = 1;
 
     ndata->lock = o->alloc_lock(o);
     if (!ndata->lock)
@@ -1830,10 +1838,6 @@ gensio_i_alloc(struct gensio_os_funcs *o,
 	goto out_nomem;
 
     ll->ndata = ndata;
-    if (filter) {
-	filter->ndata = ndata;
-	gensio_filter_set_callback(filter, gensio_base_filter_cb, ndata);
-    }
     ndata->io = gensio_data_alloc(o, cb, user_data, gensio_base_func,
 				  child, typename, ndata);
     if (!ndata->io)
@@ -1841,6 +1845,10 @@ gensio_i_alloc(struct gensio_os_funcs *o,
     ndata->child = child;
     gensio_set_is_client(ndata->io, is_client);
     gensio_ll_set_callback(ll, gensio_ll_base_cb, ndata);
+    if (filter) {
+	filter->ndata = ndata;
+	gensio_filter_set_callback(filter, gensio_base_filter_cb, ndata);
+    }
 
     /*
      * Save this until we succeed, otherwise basen_finish_free will
