@@ -12,6 +12,7 @@
 #include <stdlib.h>
 #include "config.h"
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
 #include <gensio/gensio_builtins.h>
@@ -30,7 +31,6 @@ struct echon_data {
     struct gensio_lock *lock;
 
     unsigned int refcount;
-    unsigned int freeref;
     enum echon_state state;
 
     struct gensio *io;
@@ -139,11 +139,10 @@ static void
 echon_deferred_op(struct gensio_runner *runner, void *cb_data)
 {
     struct echon_data *ndata = cb_data;
+    int err = 0;
 
     echon_lock(ndata);
     if (ndata->state == ECHON_IN_OPEN || ndata->state == ECHON_IN_OPEN_CLOSE) {
-	int err = 0;
-
 	if (ndata->state == ECHON_IN_OPEN_CLOSE) {
 	    ndata->state = ECHON_IN_CLOSE;
 	    err = GE_LOCALCLOSED;
@@ -165,8 +164,10 @@ echon_deferred_op(struct gensio_runner *runner, void *cb_data)
 
 	gensio_circbuf_next_read_area(ndata->buf, &data, &count);
 	echon_unlock(ndata);
-	gensio_cb(ndata->io, GENSIO_EVENT_READ, 0, data, &count, NULL);
+	err = gensio_cb(ndata->io, GENSIO_EVENT_READ, 0, data, &count, NULL);
 	echon_lock(ndata);
+	if (err)
+	    break;
 	gensio_circbuf_data_removed(ndata->buf, count);
     }
 
@@ -174,11 +175,13 @@ echon_deferred_op(struct gensio_runner *runner, void *cb_data)
 	   gensio_circbuf_room_left(ndata->buf) > 0 &&
 	   ndata->xmit_enabled) {
 	echon_unlock(ndata);
-	gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0,
-		  NULL, NULL, NULL);
+	err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0,
+			NULL, NULL, NULL);
 	echon_lock(ndata);
+	if (err)
+	    break;
     }
-    if (ndata->state == ECHON_OPEN &&
+    if (!err && ndata->state == ECHON_OPEN &&
 		gensio_circbuf_datalen(ndata->buf) > 0 && ndata->read_enabled)
 	goto more_read;
 
@@ -278,29 +281,14 @@ echon_close(struct gensio *io, gensio_done close_done, void *close_data)
     return err;
 }
 
-static void
-echon_func_ref(struct gensio *io)
-{
-    struct echon_data *ndata = gensio_get_gensio_data(io);
-
-    echon_lock(ndata);
-    ndata->freeref++;
-    echon_unlock(ndata);
-}
-
 static void
 echon_free(struct gensio *io)
 {
     struct echon_data *ndata = gensio_get_gensio_data(io);
 
     echon_lock(ndata);
-    assert(ndata->freeref > 0);
-    if (--ndata->freeref == 0) {
-	ndata->state = ECHON_CLOSED;
-	echon_unlock_and_deref(ndata);
-    } else {
-	echon_unlock(ndata);
-    }
+    ndata->state = ECHON_CLOSED;
+    echon_unlock_and_deref(ndata);
 }
 
 static int
@@ -348,10 +336,6 @@ gensio_echo_func(struct gensio *io, int func, gensiods *count,
 	echon_free(io);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	echon_func_ref(io);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	echon_set_read_callback_enable(io, buflen);
 	return 0;
@@ -382,7 +366,6 @@ echo_ndata_setup(struct gensio_os_funcs *o, gensiods max_read_size,
 	return GE_NOMEM;
     ndata->o = o;
     ndata->refcount = 1;
-    ndata->freeref = 1;
 
     ndata->buf = gensio_circbuf_alloc(o, max_read_size);
     if (!ndata->buf)
