@@ -13,6 +13,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
 #include <gensio/gensio_builtins.h>
@@ -215,12 +216,12 @@ filen_finish_free(struct filen_data *ndata)
 {
     struct gensio_os_funcs *o = ndata->o;
 
+    if (ndata->io)
+	gensio_data_free(ndata->io);
     if (ndata->infile)
 	o->free(ndata->o, ndata->infile);
     if (ndata->outfile)
 	o->free(ndata->o, ndata->outfile);
-    if (ndata->io)
-	gensio_data_free(ndata->io);
     if (ndata->read_data)
 	o->free(o, ndata->read_data);
     if (ndata->deferred_op_runner)
@@ -293,13 +294,12 @@ static void
 filen_deferred_op(struct gensio_runner *runner, void *cb_data)
 {
     struct filen_data *ndata = cb_data;
+    int err = 0;
 
     filen_lock(ndata);
     ndata->deferred_op_pending = false;
 
     if (ndata->state == FILEN_IN_OPEN || ndata->state == FILEN_IN_OPEN_CLOSE) {
-	int err = 0;
-
 	if (ndata->state == FILEN_IN_OPEN_CLOSE) {
 	    ndata->state = FILEN_IN_CLOSE;
 	    err = GE_LOCALCLOSED;
@@ -318,12 +318,12 @@ filen_deferred_op(struct gensio_runner *runner, void *cb_data)
 	gensiods count = 0;
 
 	if (ndata->data_pending_len == 0 && !ndata->read_err) {
-	    int rv = f_read(ndata->o, ndata->inf, ndata->read_data,
-			    ndata->max_read_size, &count);
+	    err = f_read(ndata->o, ndata->inf, ndata->read_data,
+			 ndata->max_read_size, &count);
 
-	    if (rv) {
+	    if (err) {
 		ndata->read_enabled = false;
-		ndata->read_err = rv;
+		ndata->read_err = err;
 	    } else {
 		ndata->data_pending_len = count;
 	    }
@@ -334,9 +334,14 @@ filen_deferred_op(struct gensio_runner *runner, void *cb_data)
 	    ndata->read_enabled = false;
 	} else {
 	    filen_unlock(ndata);
-	    gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->read_err,
-		      ndata->read_data, &count, NULL);
+	    err = gensio_cb(ndata->io, GENSIO_EVENT_READ, ndata->read_err,
+			    ndata->read_data, &count, NULL);
 	    filen_lock(ndata);
+	    if (err) {
+		ndata->read_enabled = false;
+		ndata->read_err = err;
+		break;
+	    }
 	}
 	if (count > 0) {
 	    if (count >= ndata->data_pending_len) {
@@ -351,9 +356,14 @@ filen_deferred_op(struct gensio_runner *runner, void *cb_data)
 
     while (ndata->state == FILEN_OPEN && ndata->xmit_enabled) {
 	filen_unlock(ndata);
-	gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0,
-		  NULL, NULL, NULL);
+	err = gensio_cb(ndata->io, GENSIO_EVENT_WRITE_READY, 0,
+			NULL, NULL, NULL);
 	filen_lock(ndata);
+	if (err) {
+	    ndata->read_enabled = false;
+	    ndata->read_err = err;
+	    break;
+	}
     }
 
     if (ndata->state == FILEN_IN_CLOSE) {
@@ -475,16 +485,6 @@ filen_close(struct gensio *io, gensio_done close_done, void *close_data)
     return err;
 }
 
-static void
-filen_func_ref(struct gensio *io)
-{
-    struct filen_data *ndata = gensio_get_gensio_data(io);
-
-    filen_lock(ndata);
-    filen_ref(ndata);
-    filen_unlock(ndata);
-}
-
 static void
 filen_free(struct gensio *io)
 {
@@ -552,10 +552,6 @@ gensio_file_func(struct gensio *io, int func, gensiods *count,
 	filen_free(io);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	filen_func_ref(io);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	filen_set_read_callback_enable(io, buflen);
 	return 0;
