@@ -7,10 +7,8 @@
 
 #include "config.h"
 
-#include <gensio/gensio_class.h>
-
 #include "gensio_filter_certauth.h"
-#include "gensio_filter_ssl.h"
+#include <gensio/gensio_err.h>
 
 #ifdef _WIN32
 #define DIRSEP '\\'
@@ -53,6 +51,10 @@ struct gensio_certauth_filter_data {
 #include <openssl/rand.h>
 #include <openssl/err.h>
 
+#include <gensio/gensio.h>
+#include <gensio/gensio_class.h>
+#include <gensio/gensio_os_funcs.h>
+
 /* In gensio_filter_ssl.c, semi-private. */
 int gensio_cert_get_name(X509 *cert, char *data, gensiods *datalen);
 int gensio_cert_to_buf(X509 *cert, char *buf, gensiods *datalen);
@@ -1032,7 +1034,7 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
 
     case CERTAUTH_CHALLENGE_RESPONSE:
 	if (!sfilter->cert || !sfilter->response_result) {
-	    /* 
+	    /*
 	     * Remote end didn't send certificate and/or challenge
 	     * response, try password.
 	     */
