@@ -8,6 +8,7 @@
 #include "config.h"
 
 #include <gensio/gensio_class.h>
+#include <gensio/gensio_err.h>
 
 #include "gensio_filter_ssl.h"
 
@@ -20,6 +21,9 @@
 #include <openssl/bio.h>
 #include <openssl/err.h>
 
+#include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
+
 #ifdef _WIN32
 #define DIRSEP '\\'
 #else
@@ -1104,7 +1108,7 @@ gensio_ssl_filter_raw_alloc(struct gensio_os_funcs *o,
     sfilter = o->zalloc(o, sizeof(*sfilter));
     if (!sfilter)
 	return NULL;
-    
+
     sfilter->o = o;
     sfilter->is_client = is_client;
     sfilter->max_write_size = max_write_size;
