@@ -9,7 +9,9 @@
 #include <assert.h>
 #include <stdio.h>
 
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
+#include <gensio/gensio_err.h>
 #include <gensio/gensio_ll_fd.h>
 #include <gensio/gensio_osops.h>
 
