@@ -5,10 +5,14 @@
  *  SPDX-License-Identifier: LGPL-2.1-only
  */
 
-/* This code is for a gensio that echos all writes back to read. */
+/*
+ * This code is for discovering gensio parameters via MDNS then
+ * creating a gensio based upon those..
+ */
 
 #include "config.h"
 #include <gensio/gensio_builtins.h>
+#include <gensio/gensio_err.h>
 
 #ifdef HAVE_AVAHI
 #include <assert.h>
@@ -17,6 +21,7 @@
 #include <stdlib.h>
 #include "config.h"
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_mdns.h>
 #include <gensio/argvutils.h>
@@ -35,7 +40,6 @@ struct mdnsn_data {
     struct gensio_lock *lock;
 
     unsigned int refcount;
-    unsigned int freeref;
     enum mdnsn_state state;
 
     struct gensio *io;
@@ -305,7 +309,7 @@ addarg(char **args, gensiods *len, struct gensio_os_funcs *o,
 }
 
 static int
-get_mdns_gensiostack(struct mdnsn_data *ndata, const char *txt[],
+get_mdns_gensiostack(struct mdnsn_data *ndata, const char * const *txt,
 		     const struct gensio_addr *addr, char **rstack)
 {
     struct gensio_os_funcs *o = ndata->o;
@@ -389,7 +393,7 @@ mdns_cb(struct gensio_mdns_watch *w,
 	int interface, int ipdomain,
 	const char *name, const char *type,
 	const char *domain, const char *host,
-	const struct gensio_addr *addr, const char *txt[],
+	const struct gensio_addr *addr, const char * const *txt,
 	void *userdata)
 {
     struct mdnsn_data *ndata = userdata;
@@ -604,30 +608,15 @@ mdnsn_close(struct gensio *io, gensio_done close_done, void *close_data)
     return err;
 }
 
-static void
-mdnsn_func_ref(struct gensio *io)
-{
-    struct mdnsn_data *ndata = gensio_get_gensio_data(io);
-
-    mdnsn_lock(ndata);
-    ndata->freeref++;
-    mdnsn_unlock(ndata);
-}
-
 static void
 mdnsn_free(struct gensio *io)
 {
     struct mdnsn_data *ndata = gensio_get_gensio_data(io);
 
     mdnsn_lock(ndata);
-    assert(ndata->freeref > 0);
-    if (--ndata->freeref == 0) {
-	if (ndata->state != MDNSN_CLOSED)
-	    mdnsn_start_close(ndata);
-	mdnsn_deref_and_unlock(ndata);
-    } else {
-	mdnsn_unlock(ndata);
-    }
+    if (ndata->state != MDNSN_CLOSED)
+	mdnsn_start_close(ndata);
+    mdnsn_deref_and_unlock(ndata);
 }
 
 static int
@@ -666,10 +655,6 @@ gensio_mdns_func(struct gensio *io, int func, gensiods *count,
 	mdnsn_free(io);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	mdnsn_func_ref(io);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	if (ndata->child)
 	    gensio_set_read_callback_enable(ndata->child, buflen);
@@ -707,7 +692,6 @@ mdns_ndata_setup(struct gensio_os_funcs *o, gensiods max_read_size,
 
     ndata->o = o;
     ndata->refcount = 1;
-    ndata->freeref = 1;
     ndata->max_read_size = max_read_size;
     ndata->nodelay = nodelay;
     ndata->interface = interface;
