@@ -11,9 +11,12 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_acc_gensio.h>
 #include <gensio/gensio_builtins.h>
+#include <gensio/gensio_list.h>
 
 /*
  * The protocol consists of messages.  The first byte of a message
@@ -191,7 +194,6 @@ struct mux_inst {
     struct gensio *io;
     struct mux_data *mux;
     unsigned int refcount;
-    unsigned int freeref; /* Used to count muxc_ref() and muxc_free() */
     unsigned int id;
     unsigned int remote_id;
     enum mux_inst_state state;
@@ -200,6 +202,7 @@ struct mux_inst {
     bool send_close;
     bool is_client;
     bool close_sent;
+    bool do_oob;
 
     /*
      * The service, either the local one or the remote one.
@@ -213,6 +216,7 @@ struct mux_inst {
     gensiods max_read_size;
     bool read_enabled;
     bool in_read_report;
+    int in_newchannel;
 
     /* Number of bytes we need to send an ack for. */
     gensiods received_unacked;
@@ -663,14 +667,14 @@ chan_free(struct mux_inst *chan)
 {
     struct gensio_os_funcs *o = chan->o;
 
+    if (chan->io)
+	gensio_data_free(chan->io);
     if (chan->read_data)
 	o->free(o, chan->read_data);
     if (chan->write_data)
 	o->free(o, chan->write_data);
     if (chan->service)
 	o->free(o, chan->service);
-    if (chan->io)
-	gensio_data_free(chan->io);
     if (chan->deferred_op_runner)
 	o->free_runner(chan->deferred_op_runner);
     o->free(o, chan);
@@ -788,7 +792,6 @@ mux_channel_set_closed(struct mux_inst *chan, gensio_done close_done,
     int err;
 
     muxc_set_state(chan, MUX_INST_CLOSED);
-    chan_deref(chan);
     assert(muxdata->nr_not_closed > 0);
     muxdata->nr_not_closed--;
     mux_add_trace(muxdata, chan, 150, muxdata->nr_not_closed);
@@ -824,6 +827,7 @@ finish_close(struct mux_inst *chan)
 	close_done(chan->io, chan->close_data);
 	mux_lock(muxdata);
     }
+    chan_deref(chan);
 }
 
 static void
@@ -884,6 +888,8 @@ mux_send_init(struct mux_data *muxdata)
 static void
 chan_check_send_more(struct mux_inst *chan)
 {
+    int err;
+
     if (chan->in_write_ready)
 	/* Another caller is already handling, just let it retry. */
 	return;
@@ -894,10 +900,15 @@ chan_check_send_more(struct mux_inst *chan)
 	   chan->write_ready_enabled && chan->state == MUX_INST_OPEN) {
 	chan_ref(chan);
 	mux_unlock(chan->mux);
-	gensio_cb(chan->io, GENSIO_EVENT_WRITE_READY, 0, NULL, NULL, NULL);
+	err = gensio_cb(chan->io, GENSIO_EVENT_WRITE_READY, 0, NULL,
+			NULL, NULL);
 	mux_lock(chan->mux);
 	if (chan_deref(chan))
 	    return; /* chan was freed. */
+	if (err) {
+	    chan->errcode = err;
+	    break;
+	}
     }
     chan->in_write_ready = false;
 }
@@ -933,6 +944,7 @@ chan_check_read(struct mux_inst *chan)
     const char *flstr[3];
     unsigned int i;
     bool fullmsg;
+    int err;
 
     while (((fullmsg = full_msg_ready(chan, &len)) || chan->errcode) &&
 	   chan->read_enabled && !chan->in_read_report) {
@@ -941,10 +953,12 @@ chan_check_read(struct mux_inst *chan)
 	    chan->in_read_report = true;
 	    chan->read_enabled = false;
 	    mux_unlock(muxdata);
-	    gensio_cb(chan->io, GENSIO_EVENT_READ, chan->errcode,
-		      NULL, NULL, NULL);
+	    err = gensio_cb(chan->io, GENSIO_EVENT_READ, chan->errcode,
+			    NULL, NULL, NULL);
 	    mux_lock(muxdata);
 	    chan->in_read_report = false;
+	    if (err)
+		break;
 	    continue;
 	}
 
@@ -955,17 +969,24 @@ chan_check_read(struct mux_inst *chan)
 
 	chan->in_read_report = true;
 	i = 0;
-	if (flags & MUX_FLAG_OUT_OF_BOUND)
+	if (flags & MUX_FLAG_OUT_OF_BOUND) {
+	    if (!chan->do_oob)
+		goto after_read_done;
 	    flstr[i++] = "oob";
+	}
 	flstr[i] = NULL;
 	if (pos + len > chan->max_read_size) {
 	    /* Buffer wraps, deliver in two parts. */
 	    rcount = chan->max_read_size - pos;
 	    orcount = rcount;
 	    mux_unlock(muxdata);
-	    gensio_cb(chan->io, GENSIO_EVENT_READ,
-		      0, chan->read_data + pos, &rcount, flstr);
+	    err = gensio_cb(chan->io, GENSIO_EVENT_READ,
+			    0, chan->read_data + pos, &rcount, flstr);
 	    mux_lock(muxdata);
+	    if (err) {
+		chan->errcode = err;
+		goto after_read_done;
+	    }
 	    if (rcount > orcount)
 		rcount = orcount;
 	    len -= rcount;
@@ -982,9 +1003,13 @@ chan_check_read(struct mux_inst *chan)
 	if (flags & MUX_FLAG_END_OF_MESSAGE)
 	    flstr[i++] = "eom";
 	flstr[i] = NULL;
-	gensio_cb(chan->io, GENSIO_EVENT_READ,
-		  0, chan->read_data + pos, &rcount, flstr);
+	err = gensio_cb(chan->io, GENSIO_EVENT_READ,
+			0, chan->read_data + pos, &rcount, flstr);
 	mux_lock(muxdata);
+	if (err) {
+	    chan->errcode = err;
+	    goto after_read_done;
+	}
 	if (rcount > orcount)
 	    rcount = orcount;
 	len -= rcount;
@@ -1218,14 +1243,21 @@ muxc_close_nolock(struct mux_inst *chan,
 
     case MUX_INST_OPEN:
 	muxc_set_state(chan, MUX_INST_IN_CLOSE);
-	chan->send_close = true;
-	muxc_add_to_wrlist(chan);
+	if (chan->in_newchannel) {
+	    chan->in_newchannel = 2;
+	    muxc_set_state(chan, MUX_INST_IN_CLOSE_FINAL);
+	    chan_sched_deferred_op(chan);
+	} else {
+	    chan->send_close = true;
+	    muxc_add_to_wrlist(chan);
+	}
 	break;
 
     default:
 	return GE_NOTREADY;
     }
 
+    chan_ref(chan);
     chan->close_done = close_done;
     chan->close_data = close_data;
 
@@ -1247,33 +1279,19 @@ muxc_close(struct mux_inst *chan, gensio_done close_done, void *close_data)
     return err;
 }
 
-static void
-muxc_func_ref(struct mux_inst *chan)
-{
-    struct mux_data *muxdata = chan->mux;
-
-    mux_lock(muxdata);
-    chan->freeref++;
-    mux_unlock(muxdata);
-}
-
 static void
 muxc_free(struct mux_inst *chan)
 {
     struct mux_data *muxdata = chan->mux;
 
     mux_lock_and_ref(muxdata);
-    assert(chan->freeref > 0);
-    if (--chan->freeref > 0)
-	goto out_unlock;
+    if (chan->in_newchannel) {
+	chan->in_newchannel = 2;
+	goto out_deref;
+    }
+
     switch (chan->state) {
     case MUX_INST_IN_REM_CLOSE:
-	/*
-	 * We don't call finish_close() in this case, so we need to do
-	 * the deref here instead.
-	 */
-	chan_deref(chan);
-	/* Fallthrough */
     case MUX_INST_IN_CLOSE:
     case MUX_INST_IN_CLOSE_FINAL:
     case MUX_INST_IN_OPEN_CLOSE:
@@ -1295,8 +1313,8 @@ muxc_free(struct mux_inst *chan)
 	/* Shouldn't be possible. */
 	abort();
     }
+ out_deref:
     chan_deref(chan);
- out_unlock:
     mux_deref_and_unlock(muxdata);
 }
 
@@ -1351,7 +1369,6 @@ mux_new_channel(struct mux_data *muxdata, gensio_event cb, void *user_data,
 	gensio_set_is_encrypted(chan->io, true);
     chan->mux = muxdata;
     chan->refcount = 1;
-    chan->freeref = 1;
     chan->is_client = is_client;
     chan->max_read_size = muxdata->max_read_size;
     chan->max_write_size = muxdata->max_write_size;
@@ -1394,6 +1411,9 @@ mux_new_channel(struct mux_data *muxdata, gensio_event cb, void *user_data,
 	    p = l;
 	    l = gensio_list_next_wrap(&muxdata->chans, p);
 	} while (f != l);
+	tchan = gensio_container_of(l, struct mux_inst, link);
+	if (id != tchan->id)
+	    goto found;
 
 	err = GE_INUSE;
 
@@ -1584,8 +1604,7 @@ mux_call_open_done(struct mux_data *muxdata, struct mux_inst *chan, int err)
 	open_done(chan->io, err, open_data);
 	mux_lock(muxdata);
     }
-    if (err)
-	chan_deref(chan);
+    chan_deref(chan);
 }
 
 static void
@@ -1600,9 +1619,9 @@ mux_child_open_done(struct gensio *child, int err, void *open_data)
 	mux_shutdown_channels(muxdata, err);
 	muxdata->nr_not_closed = 0;
     } else if (chan->state != MUX_INST_IN_OPEN) {
-	/* A close was requested, handle it. */	
+	/* A close was requested, handle it. */
 	muxc_set_state(chan, MUX_INST_CLOSED);
-	mux_call_open_done(muxdata, chan, err);
+	mux_call_open_done(muxdata, chan, 0);
 	mux_channel_finish_close(chan);
     } else {
 	mux_set_state(muxdata, MUX_UNINITIALIZED);
@@ -1650,6 +1669,7 @@ muxc_open(struct mux_inst *chan, gensio_done_err open_done, void *open_data,
 	muxdata->hdr_pos = 0;
 	muxdata->hdr_size = 0;
 	muxdata->exit_err = 0;
+	muxdata->err_shutdown = 0;
 	muxdata->do_normal_close = false;
 	muxc_reinit(chan);
 	if (muxdata->is_client) {
@@ -1754,6 +1774,13 @@ muxc_control(struct mux_inst *chan, bool get, int op,
 	}
 	break;
 
+    case GENSIO_CONTROL_ENABLE_OOB:
+	if (get)
+	    *datalen = snprintf(data, *datalen, "%u", chan->do_oob);
+	else
+	    chan->do_oob = !!strtoul(data, NULL, 0);
+	break;
+
     default:
 	err = GE_NOTSUP;
 	break;
@@ -1782,10 +1809,6 @@ muxc_gensio_handler(struct gensio *io, int func, gensiods *count,
 	muxc_free(chan);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	muxc_func_ref(chan);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	muxc_set_read_callback_enable(chan, buflen);
 	return 0;
@@ -1880,7 +1903,6 @@ mux_shutdown_channels(struct mux_data *muxdata, int err)
 	    /* Just close it. */
 	    muxc_set_state(chan, MUX_INST_CLOSED);
 	    finish_close(chan);
-	    chan_deref(chan);
 	    break;
 
 	case MUX_INST_OPEN:
@@ -2332,8 +2354,13 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 		muxdata->opencount--;
 
 		if (chan->errcode) {
-		    muxc_set_state(chan, MUX_INST_IN_CLOSE_FINAL);
-		    chan_sched_deferred_op(chan);
+		    enum mux_inst_state old_state = chan->state;
+
+		    muxc_set_state(chan, MUX_INST_CLOSED);
+		    mux_call_open_done(muxdata, chan, chan->errcode);
+		    if (old_state == MUX_INST_IN_OPEN_CLOSE)
+			i_finish_close_close_done(chan, muxdata);
+		    chan = NULL;
 		} else if (chan->state == MUX_INST_IN_OPEN_CLOSE) {
 		    muxc_set_state(chan, MUX_INST_IN_CLOSE);
 		    mux_call_open_done(muxdata, chan, GE_LOCALCLOSED);
@@ -2513,7 +2540,6 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 		}
 	    new_chan_no_service:
 		muxc_set_state(chan, MUX_INST_OPEN);
-		chan_ref(chan);
 		if (muxdata->state == MUX_WAITING_OPEN) {
 		    /*
 		     * This is the first channel, deliver to the
@@ -2530,34 +2556,39 @@ mux_child_read(struct mux_data *muxdata, int ierr,
 			mux_lock(muxdata);
 		    } else {
 			mux_call_open_done(muxdata, chan, 0);
-			/*
-			 * In this case we got a new channel after we
-			 * sent a new channel on initialization.  We
-			 * will have already claimed a ref for the
-			 * open, undo the ref above since it is not
-			 * required.
-			 */
-			chan_deref(chan);
 		    }
 		} else {
 		    if (muxdata->xmit_data_len) {
 			proto_err_str = "New channel while in progress";
 			goto protocol_err;
 		    }
-		    mux_send_new_channel_rsp(muxdata, chan->remote_id,
-					     chan->max_read_size,
-					     chan->id, 0);
-		    if (chan->service_len) {
-			/* Ack the service data. */
-			chan->received_unacked = chan->service_len;
-			muxc_add_to_wrlist(chan);
-		    }
 		    if (chan->service)
 			auxdata[0] = chan->service;
 		    else
 			auxdata[0] = "";
-		    mux_firstchan_event(muxdata, GENSIO_EVENT_NEW_CHANNEL,
-					0, (void *) chan->io, 0, auxdata);
+		    chan->in_newchannel = 1;
+		    chan_ref(chan);
+		    err = mux_firstchan_event(muxdata, GENSIO_EVENT_NEW_CHANNEL,
+					      0, (void *) chan->io, 0, auxdata);
+		    if (!err && chan->in_newchannel == 2)
+			err = GE_REMCLOSE;
+		    chan->in_newchannel = 0;
+		    mux_send_new_channel_rsp(muxdata, chan->remote_id,
+					     chan->max_read_size,
+					     chan->id, err);
+		    if (err) {
+			if (chan->state == MUX_INST_OPEN)
+			    /*
+			     * We might have gone to in_close, let the
+			     * close call happen in that case.
+			     */
+			    muxc_set_state(chan, MUX_INST_CLOSED);
+		    } else if (chan->service_len) {
+			/* Ack the service data. */
+			chan->received_unacked = chan->service_len;
+			muxc_add_to_wrlist(chan);
+		    }
+		    chan_deref(chan);
 		}
 	    finish_new_chan:
 		muxdata->in_hdr = true;
