@@ -15,10 +15,12 @@
 #include <tcpd.h>
 #endif /* HAVE_TCPD_H */
 
-#include <gensio/gensio_osops.h>
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
+#include <gensio/gensio_osops.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
+#include <gensio/gensio_list.h>
 
 #include "errtrig.h"
 
@@ -33,6 +35,7 @@ bool gensio_set_progname(const char *iprogname)
 
 #ifdef _WIN32
 #include <winsock2.h> /* For AF_UNSPEC */
+#include <iphlpapi.h>
 #else
 #include <sys/socket.h>
 #include <arpa/inet.h> /* For AF_UNSPEC */
@@ -41,6 +44,9 @@ bool gensio_set_progname(const char *iprogname)
 #include <grp.h>
 #include <pwd.h>
 #include <errno.h>
+#include <ifaddrs.h>
+#include <netinet/in.h>
+#include <net/if.h>
 
 int
 gensio_unix_os_setupnewprog(void)
@@ -270,6 +276,302 @@ gensio_os_scan_netaddr(struct gensio_os_funcs *o, const char *str, bool listen,
     return rv;
 }
 
+
+void
+gensio_os_free_net_ifs(struct gensio_os_funcs *o,
+		       struct gensio_net_if **ifs, unsigned int nifs)
+{
+    unsigned int i, j;
+
+    for (i = 0; i < nifs; i++) {
+	if (!ifs)
+	    continue;
+	if (ifs[i]->name)
+	    gensio_os_funcs_zfree(o, ifs[i]->name);
+	if (ifs[i]->addrs) {
+	    for (j = 0; j < ifs[i]->naddrs; j++) {
+		if (ifs[i]->addrs[j].addrstr)
+		    gensio_os_funcs_zfree(o, ifs[i]->addrs[j].addrstr);
+	    }
+	    gensio_os_funcs_zfree(o, ifs[i]->addrs);
+	}
+	gensio_os_funcs_zfree(o, ifs[i]);
+    }
+    gensio_os_funcs_zfree(o, ifs);
+}
+
+static bool
+is_inet_family(struct sockaddr *s)
+{
+    if (!s) /* This can happen on down interfaces. */
+	return false;
+    return (s->sa_family == AF_INET || s->sa_family == AF_INET6);
+}
+
+int
+gensio_os_get_net_ifs(struct gensio_os_funcs *o,
+		      struct gensio_net_if ***rifs, unsigned int *rnifs)
+{
+    struct gensio_net_if **ifs = NULL;
+    char buf[100], *addrtype;
+#ifdef _WIN32
+    IP_ADAPTER_ADDRESSES *t, *c, *p;
+    ULONG err;
+    unsigned int i, j, nifs;
+    ULONG buflen = 15 * 1024;
+
+    while (true) {
+	t = o->zalloc(o, buflen);
+	if (!t)
+	    return GE_NOMEM;
+	err = GetAdaptersAddresses(AF_UNSPEC,
+				   (GAA_FLAG_SKIP_ANYCAST |
+				    GAA_FLAG_SKIP_MULTICAST |
+				    GAA_FLAG_SKIP_DNS_SERVER |
+				    GAA_FLAG_SKIP_FRIENDLY_NAME),
+				   NULL, t, &buflen);
+	if (err == NO_ERROR)
+	    break;
+	o->free(o, t);
+	if (err == ERROR_BUFFER_OVERFLOW) {
+	    /* Just retry */
+	} else if (err == ERROR_NOT_ENOUGH_MEMORY) {
+	    return GE_NOMEM;
+	} else {
+	    return GE_OSERR;
+	}
+    }
+
+    i = 0;
+    for (c = t; c; c = c->Next) {
+	if (c->IfIndex != c->Ipv6IfIndex && c->Ipv6IfIndex != 0)
+	    continue; /* FIXME - Not sure what to do with these. */
+	i++;
+    }
+    nifs = i;
+
+    ifs = gensio_os_funcs_zalloc(o, sizeof(*ifs) * (nifs + 1));
+    if (!ifs)
+	goto out_err;
+
+    i = 0;
+    for (c = t; c; c = c->Next) {
+	IP_ADAPTER_UNICAST_ADDRESS *al;
+	unsigned int slen;
+	size_t rlen;
+
+	if (c->IfIndex != c->Ipv6IfIndex && c->Ipv6IfIndex != 0)
+	    continue; /* FIXME - Not sure what to do with these. */
+
+	ifs[i] = gensio_os_funcs_zalloc(o, sizeof(**ifs));
+	if (!ifs[i])
+	    goto out_err;
+
+	ifs[i]->ifindex = c->IfIndex;
+	if (c->IfType == IF_TYPE_SOFTWARE_LOOPBACK)
+	    ifs[i]->flags |= GENSIO_NET_IF_LOOPBACK;
+	if (c->OperStatus == IfOperStatusUp)
+	    ifs[i]->flags |= GENSIO_NET_IF_UP;
+	if (!c->NoMulticast)
+	    ifs[i]->flags |= GENSIO_NET_IF_MULTICAST;
+	slen = wcslen(c->FriendlyName) * 2;
+	ifs[i]->name = o->zalloc(o, slen + 1);
+	if (!ifs[i]->name)
+	    goto out_err;
+	wcstombs_s(&rlen, ifs[i]->name, slen + 1, c->FriendlyName, slen);
+
+	for (j = 0, al = c->FirstUnicastAddress; al; j++, al = al->Next)
+	    ;
+	ifs[i]->addrs = o->zalloc(o, sizeof(struct gensio_net_addr) * j);
+	if (!ifs[i]->addrs)
+	    goto out_err;
+	for (j = 0, al = c->FirstUnicastAddress; al; al = al->Next) {
+	    struct sockaddr *a = (void *) al->Address.lpSockaddr;
+
+	    if (a->sa_family == AF_INET) {
+		struct sockaddr_in *ia = (void *) a;
+
+		ifs[i]->addrs[j].family = GENSIO_NETTYPE_IPV4;
+		ifs[i]->addrs[j].netbits = al->OnLinkPrefixLength;
+		ifs[i]->addrs[j].addrlen = 4;
+		memcpy(ifs[i]->addrs[j].addr, &ia->sin_addr, 4);
+		addrtype = "ipv4:";
+	    } else if (a->sa_family == AF_INET6) {
+		struct sockaddr_in6 *ia = (void *) a;
+
+		ifs[i]->addrs[j].family = GENSIO_NETTYPE_IPV6;
+		ifs[i]->addrs[j].netbits = al->OnLinkPrefixLength;
+		ifs[i]->addrs[j].addrlen = 16;
+		memcpy(ifs[i]->addrs[j].addr, &ia->sin6_addr, 16);
+		addrtype = "ipv6:";
+	    } else {
+		continue;
+	    }
+	    memcpy(buf, addrtype, 5);
+	    inet_ntop(a->sa_family, ifs[i]->addrs[j].addr,
+		      buf + 5, sizeof(buf) - 5);
+	    ifs[i]->addrs[j].addrstr = gensio_strdup(o, buf);
+	    if (!ifs[i]->addrs[j].addrstr)
+		goto out_err;
+	    j++;
+	}
+	ifs[i]->naddrs = j;
+
+	i++;
+	assert(i <= nifs);
+    }
+
+    o->free(o, t);
+    *rifs = ifs;
+    *rnifs = i;
+
+    return 0;
+ out_err:
+    o->free(o, t);
+    if (ifs)
+	gensio_os_free_net_ifs(o, ifs, nifs);
+    return GE_NOMEM;
+#else
+    struct ifaddrs *ifap, *ifp, *ifp2;
+    unsigned int i, j, k, nifs = 0, naddrs, addrlen, nbits;
+    unsigned char *addr, *netmask;
+    int rv;
+    bool found;
+
+    rv = getifaddrs(&ifap);
+    if (rv) {
+	rv = gensio_os_err_to_err(o, errno);
+	return rv;
+    }
+
+    /* Count the number of unique interfaces by name. */
+    for (ifp = ifap; ifp; ifp = ifp->ifa_next) {
+	if (!is_inet_family(ifp->ifa_addr))
+	    continue;
+
+	found = false;
+	for (ifp2 = ifap; ifp2 != ifp; ifp2 = ifp2->ifa_next) {
+	    if (!is_inet_family(ifp2->ifa_addr))
+		continue;
+	    if (strcmp(ifp2->ifa_name, ifp->ifa_name) == 0) {
+		found = true;
+		break;
+	    }
+	}
+	if (found)
+	    continue;
+	nifs++;
+    }
+    rv = GE_NOMEM;
+    ifs = gensio_os_funcs_zalloc(o, sizeof(*ifs) * (nifs + 1));
+    if (!ifs)
+	goto out_err;
+    for (ifp = ifap; ifp; ifp = ifp->ifa_next) {
+	if (!is_inet_family(ifp->ifa_addr))
+	    continue;
+
+	for (i = 0; i < nifs && ifs[i]; i++) {
+	    if (strcmp(ifs[i]->name, ifp->ifa_name) == 0)
+		break;
+	}
+	if (!ifs[i]) {
+	    /*
+	     * First occurence of this name, allocate info and the
+	     * if address array.
+	     */
+	    ifs[i] = gensio_os_funcs_zalloc(o, sizeof(**ifs));
+	    if (!ifs[i])
+		goto out_err;
+	    ifs[i]->name = gensio_strdup(o, ifp->ifa_name);
+	    if (!ifs[i]->name)
+		goto out_err;
+	    ifs[i]->ifindex = if_nametoindex(ifp->ifa_name);
+	    if (!ifs[i]->ifindex) {
+		rv = gensio_os_err_to_err(o, errno);
+		goto out_err;
+	    }
+
+	    /* Count the number of addresses for this interface. */
+	    naddrs = 1;
+	    for (ifp2 = ifp->ifa_next; ifp2; ifp2 = ifp2->ifa_next) {
+		if (!is_inet_family(ifp2->ifa_addr))
+		    continue;
+
+		if (strcmp(ifp2->ifa_name, ifp->ifa_name) == 0)
+		    naddrs++;
+	    }
+	    ifs[i]->addrs = gensio_os_funcs_zalloc(o,
+				  naddrs * sizeof(struct gensio_net_addr));
+	    if (!ifs[i]->addrs)
+		goto out_err;
+	}
+	if (ifp->ifa_flags & IFF_UP)
+	    ifs[i]->flags |= GENSIO_NET_IF_UP;
+	if (ifp->ifa_flags & IFF_LOOPBACK)
+	    ifs[i]->flags |= GENSIO_NET_IF_LOOPBACK;
+	if (ifp->ifa_flags & IFF_MULTICAST)
+	    ifs[i]->flags |= GENSIO_NET_IF_MULTICAST;
+	if (ifp->ifa_addr->sa_family == AF_INET) {
+	    struct sockaddr_in *s;
+
+	    s = (struct sockaddr_in *) ifp->ifa_addr;
+	    addr = (unsigned char *) &s->sin_addr;
+	    s = (struct sockaddr_in *) ifp->ifa_netmask;
+	    netmask = (unsigned char *) &s->sin_addr;
+	    addrlen = 4;
+	    addrtype = "ipv4:";
+	} else {
+	    struct sockaddr_in6 *s;
+
+	    s = (struct sockaddr_in6 *) ifp->ifa_addr;
+	    addr = s->sin6_addr.s6_addr;
+	    s = (struct sockaddr_in6 *) ifp->ifa_netmask;
+	    netmask = s->sin6_addr.s6_addr;
+	    addrlen = 16;
+	    addrtype = "ipv6:";
+	}
+
+	j = (ifs[i]->naddrs)++;
+	memcpy(ifs[i]->addrs[j].addr, addr, addrlen);
+	ifs[i]->addrs[j].addrlen = addrlen;
+	if (ifp->ifa_addr->sa_family == AF_INET)
+	    ifs[i]->addrs[j].family = GENSIO_NETTYPE_IPV4;
+	else
+	    ifs[i]->addrs[j].family = GENSIO_NETTYPE_IPV6;
+	for (nbits = 0, k = 0; k < addrlen && netmask[k] == 0xff; k++)
+	    nbits += 8;
+	if (k < addrlen) {
+	    unsigned char v = netmask[k];
+
+	    while (v & 0xff) {
+		if (v & 0x80)
+		    nbits++;
+		else
+		    break;
+		v <<= 1;
+	    }
+	}
+	ifs[i]->addrs[j].netbits = nbits;
+	memcpy(buf, addrtype, 5);
+	inet_ntop(ifp->ifa_addr->sa_family, addr, buf + 5, sizeof(buf) - 5);
+	ifs[i]->addrs[j].addrstr = gensio_strdup(o, buf);
+	if (!ifs[i]->addrs[j].addrstr)
+	    goto out_err;
+    }
+    freeifaddrs(ifap);
+    *rifs = ifs;
+    *rnifs = nifs;
+
+    return 0;
+
+ out_err:
+    freeifaddrs(ifap);
+    if (ifs)
+	gensio_os_free_net_ifs(o, ifs, nifs);
+    return rv;
+#endif
+}
+
 const char *
 gensio_os_check_tcpd_ok(struct gensio_iod *iod, const char *iprogname)
 {
@@ -301,7 +603,7 @@ struct stdio_mode {
 };
 
 int
-gensio_win_stdio_makeraw(struct gensio_os_funcs *o, HANDLE h,
+gensio_win_stdin_makeraw(struct gensio_os_funcs *o, HANDLE h,
 			 struct stdio_mode **rm)
 {
     DWORD mode, omode;
@@ -321,6 +623,40 @@ gensio_win_stdio_makeraw(struct gensio_os_funcs *o, HANDLE h,
 		     ENABLE_INSERT_MODE |
 		     ENABLE_ECHO_INPUT |
 		     ENABLE_PROCESSED_INPUT);
+    mode |= ENABLE_WINDOW_INPUT | ENABLE_VIRTUAL_TERMINAL_INPUT;
+
+    if (!SetConsoleMode(h, mode)) {
+	if (m)
+	    o->free(o, m);
+	return gensio_os_err_to_err(o, GetLastError());
+    }
+
+    if (m)
+	*rm = m;
+
+    return 0;
+}
+
+int
+gensio_win_stdout_makeraw(struct gensio_os_funcs *o, HANDLE h,
+			  struct stdio_mode **rm)
+{
+    DWORD mode, omode;
+    struct stdio_mode *m = NULL;
+
+    if (!GetConsoleMode(h, &omode))
+	return GE_NOTSUP;
+
+    if (!*rm) {
+	m = o->zalloc(o, sizeof(*m));
+	if (!m)
+	    return GE_NOMEM;
+	m->old_mode_flags = omode;
+    }
+
+    mode = omode;
+    mode |= (ENABLE_PROCESSED_OUTPUT | ENABLE_VIRTUAL_TERMINAL_PROCESSING
+	     | ENABLE_WRAP_AT_EOL_OUTPUT);
 
     if (!SetConsoleMode(h, mode)) {
 	if (m)
