@@ -8,9 +8,8 @@
 /* This code handles SCTP network I/O. */
 
 #include "config.h"
-#include <errno.h>
-#include <gensio/gensio.h>
 #include <gensio/gensio_builtins.h>
+#include <gensio/gensio_err.h>
 
 #if HAVE_LIBSCTP
 
@@ -21,7 +20,10 @@
 #include <netinet/sctp.h>
 #include <string.h>
 #include <assert.h>
+#include <errno.h>
 
+#include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/gensio_ll_fd.h>
 #include <gensio/argvutils.h>
@@ -41,6 +43,7 @@ struct sctp_data {
     struct sctp_sack_info sackinfo;
 
     bool nodelay;
+    bool do_oob;
     unsigned int instreams;
     unsigned int ostreams;
 
@@ -288,6 +291,13 @@ sctp_control(void *handler_data, struct gensio_iod *iod, bool get, unsigned int
 	*datalen = pos;
 	return 0;
 
+    case GENSIO_CONTROL_ENABLE_OOB:
+	if (get)
+	    *datalen = snprintf(data, *datalen, "%u", tdata->do_oob);
+	else
+	    tdata->do_oob = !!strtoul(data, NULL, 0);
+	return 0;
+
     default:
 	return GE_NOTSUP;
     }
@@ -329,8 +339,9 @@ sctp_do_read(struct gensio_iod *iod, void *data, gensiods count, gensiods *rcoun
     struct sctp_sndrcvinfo sinfo;
     int flags = 0;
     unsigned int stream;
-    unsigned int i = 0;
+    unsigned int i;
 
+ restart:
     rv = tdata->o->sctp_recvmsg(iod, data, count, rcount, &sinfo, &flags);
     /* If the data length is zero, we won't have any info. */
     if (rv || *rcount == 0)
@@ -340,11 +351,15 @@ sctp_do_read(struct gensio_iod *iod, void *data, gensiods count, gensiods *rcoun
     /* Shouldn't happen, but just in case. */
     assert(stream < tdata->instreams);
 
+    i = 0;
     if (tdata->strind[stream])
 	(*auxdata)[i++] = tdata->strind[stream];
 
-    if (sinfo.sinfo_flags && SCTP_UNORDERED)
+    if (sinfo.sinfo_flags && SCTP_UNORDERED) {
+	if (!tdata->do_oob)
+	    goto restart;
 	(*auxdata)[i++] = "oob";
+    }
 
     (*auxdata)[i] = NULL;
 
@@ -728,8 +743,10 @@ static void
 sctpna_cb_en_done(struct gensio_runner *runner, void *cb_data)
 {
     struct sctpna_data *nadata = cb_data;
+    gensio_acc_done done = nadata->cb_en_done;
 
-    nadata->cb_en_done(nadata->acc, NULL);
+    nadata->cb_en_done = NULL;
+    done(nadata->acc, NULL);
 }
 
 static int
@@ -740,6 +757,9 @@ sctpna_set_accept_callback_enable(struct gensio_accepter *accepter,
 {
     unsigned int i;
 
+    if (nadata->cb_en_done)
+	return GE_INUSE;
+
     nadata->cb_en_done = done;
     for (i = 0; i < nadata->nr_acceptfds; i++)
 	sctpna_set_fd_enables(nadata, enabled);
@@ -1041,7 +1061,7 @@ sctp_gensio_accepter_alloc(struct gensio_addr *iai,
 				    o, "sctp", cb, user_data, accepter);
     if (err)
 	goto out_err;
-	
+
     nadata->acc = *accepter;
     gensio_acc_set_is_reliable(nadata->acc, true);
     /* See comment on gensio_set_is_packet() above. */
