@@ -40,8 +40,6 @@ struct stdion_channel {
     bool in_handler_set;
     bool out_handler_set;
 
-    unsigned int refcount;
-
     struct gensio *io;
 
     gensiods max_read_size;
@@ -98,6 +96,10 @@ struct stdiona_data {
     struct gensio_runner *connect_runner;
     bool in_connect_runner;
 
+    struct gensio_runner *enable_done_runner;
+    gensio_acc_done enable_done;
+    void *enable_done_data;
+
     struct gensio_timer *waitpid_timer;
 
     /* For the accepter only. */
@@ -145,6 +147,10 @@ stdiona_finish_free(struct stdiona_data *nadata)
 {
     struct gensio_os_funcs *o = nadata->o;
 
+    if (nadata->io.io)
+	gensio_data_free(nadata->io.io);
+    if (nadata->err.io)
+	gensio_data_free(nadata->err.io);
     if (nadata->io.out_iod)
 	o->release_iod(nadata->io.out_iod);
     if (nadata->io.in_iod)
@@ -159,6 +165,8 @@ stdiona_finish_free(struct stdiona_data *nadata)
 	nadata->o->free_runner(nadata->err.deferred_op_runner);
     if (nadata->connect_runner)
 	nadata->o->free_runner(nadata->connect_runner);
+    if (nadata->enable_done_runner)
+	nadata->o->free_runner(nadata->enable_done_runner);
     if (nadata->io.read_data)
 	nadata->o->free(nadata->o, nadata->io.read_data);
     if (nadata->waitpid_timer)
@@ -167,10 +175,6 @@ stdiona_finish_free(struct stdiona_data *nadata)
 	nadata->o->free(nadata->o, nadata->err.read_data);
     if (nadata->lock)
 	nadata->o->free_lock(nadata->lock);
-    if (nadata->io.io)
-	gensio_data_free(nadata->io.io);
-    if (nadata->err.io)
-	gensio_data_free(nadata->err.io);
     if (nadata->acc)
 	gensio_acc_data_free(nadata->acc);
     nadata->o->free(nadata->o, nadata);
@@ -199,7 +203,7 @@ stdiona_deref_and_unlock(struct stdiona_data *nadata)
 	stdiona_finish_free(nadata);
     } else {
 	stdiona_unlock(nadata);
-    }	
+    }
 }
 
 static int
@@ -243,21 +247,36 @@ stdion_finish_read(struct stdion_channel *schan, int err)
 	if (schan->ll_err && schan->data_pending_len == 0) {
 	    schan->read_enabled = false;
 	    stdiona_unlock(nadata);
-	    gensio_cb(io, GENSIO_EVENT_READ, schan->ll_err, NULL, NULL, NULL);
+	    err = gensio_cb(io, GENSIO_EVENT_READ, schan->ll_err, NULL,
+			    NULL, NULL);
 	    stdiona_lock(nadata);
+	    if (err) {
+		schan->ll_err = err;
+		nadata->o->set_read_handler(schan->out_iod, false);
+		nadata->o->set_except_handler(schan->out_iod, false);
+		break;
+	    }
 	} else {
 	    stdiona_unlock(nadata);
-	    gensio_cb(io, GENSIO_EVENT_READ, 0,
-		      schan->read_data + schan->data_pos, &count, NULL);
+	    err = gensio_cb(io, GENSIO_EVENT_READ, 0,
+			    schan->read_data + schan->data_pos, &count, NULL);
 	    stdiona_lock(nadata);
-	    if (count < schan->data_pending_len) {
-		/* The user didn't consume all the data. */
-		schan->data_pending_len -= count;
-		schan->data_pos += count;
-	    } else {
-		schan->data_pending_len = 0;
+	    if (!err) {
+		if (count < schan->data_pending_len) {
+		    /* The user didn't consume all the data. */
+		    schan->data_pending_len -= count;
+		    schan->data_pos += count;
+		} else {
+		    schan->data_pending_len = 0;
+		}
 	    }
 	}
+	if (err) {
+	    schan->ll_err = err;
+	    nadata->o->set_read_handler(schan->out_iod, false);
+	    nadata->o->set_except_handler(schan->out_iod, false);
+	    break;
+	}
     }
 
     schan->in_read = false;
@@ -568,6 +587,7 @@ stdion_write_ready(struct gensio_iod *iod, void *cbdata)
 {
     struct stdion_channel *schan = cbdata;
     struct stdiona_data *nadata = schan->nadata;
+    int err;
 
     stdiona_lock(nadata);
     if (schan->in_write_ready) {
@@ -577,9 +597,13 @@ stdion_write_ready(struct gensio_iod *iod, void *cbdata)
     schan->in_write_ready = true;
  retry:
     stdiona_unlock(nadata);
-    gensio_cb(schan->io, GENSIO_EVENT_WRITE_READY, 0, NULL, NULL, NULL);
+    err = gensio_cb(schan->io, GENSIO_EVENT_WRITE_READY, 0, NULL, NULL, NULL);
     stdiona_lock(nadata);
-    if (schan->write_pending) {
+    if (err) {
+	schan->ll_err = err;
+	nadata->o->set_read_handler(schan->out_iod, false);
+	nadata->o->set_except_handler(schan->out_iod, false);
+    } else if (schan->write_pending) {
 	schan->write_pending = false;
 	if (schan->xmit_enabled)
 	    goto retry;
@@ -818,11 +842,6 @@ stdion_free(struct gensio *io)
     struct stdiona_data *nadata = schan->nadata;
 
     stdiona_lock(nadata);
-    assert(schan->refcount > 0);
-    if (--schan->refcount > 0) {
-	stdiona_unlock(nadata);
-	return;
-    }
     schan->in_free = true;
     if (schan->in_close) {
 	schan->close_done = NULL;
@@ -837,18 +856,6 @@ stdion_free(struct gensio *io)
     }
 }
 
-static void
-stdion_ref(struct gensio *io)
-{
-    struct stdion_channel *schan = gensio_get_gensio_data(io);
-    struct stdiona_data *nadata = schan->nadata;
-
-    stdiona_lock(nadata);
-    assert(schan->refcount > 0);
-    schan->refcount++;
-    stdiona_unlock(nadata);
-}
-
 static int
 stdion_disable(struct gensio *io)
 {
@@ -990,6 +997,20 @@ stdion_control(struct gensio *io, bool get, unsigned int option,
 	*datalen = snprintf(data, *datalen, "%llu",
 			    (unsigned long long) nadata->opid);
 	return 0;
+
+    case GENSIO_CONTROL_IOD:
+	if (!get)
+	    return GE_NOTSUP;
+	if (*datalen != sizeof(void *))
+	    return GE_INVAL;
+	val = strtoul(data, NULL, 0);
+	if (val == 0)
+	    memcpy(data, &schan->in_iod, sizeof(void *));
+	else if (val == 1)
+	    memcpy(data, &schan->out_iod, sizeof(void *));
+	else
+	    return GE_INVAL;
+	return 0;
     }
 
     return GE_NOTSUP;
@@ -1014,10 +1035,6 @@ gensio_stdio_func(struct gensio *io, int func, gensiods *count,
 	stdion_free(io);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	stdion_ref(io);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	stdion_set_read_callback_enable(io, buflen);
 	return 0;
@@ -1055,8 +1072,6 @@ stdio_nadata_setup(struct gensio_os_funcs *o, gensiods max_read_size,
 	return GE_NOMEM;
     nadata->o = o;
     nadata->refcount = 1;
-    nadata->io.refcount = 1;
-    nadata->err.refcount = 1;
     nadata->io.closed = true;
     nadata->err.closed = true;
     nadata->io.nadata = nadata;
@@ -1359,26 +1374,22 @@ stdiona_shutdown(struct gensio_accepter *accepter,
     return rv;
 }
 
-struct stdiona_waiters {
-    struct gensio_os_funcs *o;
-    struct stdiona_data *nadata;
-    gensio_acc_done done;
-    void *done_data;
-    struct gensio_runner *runner;
-};
-
 static void
-waiter_runner_cb(struct gensio_runner *runner, void *cb_data)
+enable_done_op(struct gensio_runner *runner, void *cb_data)
 {
-    struct stdiona_waiters *w = cb_data;
+    struct stdiona_data *nadata = cb_data;
 
-    w->done(w->nadata->acc, w->done_data);
-    w->o->free_runner(w->runner);
+    stdiona_lock(nadata);
+    if (nadata->enable_done) {
+	gensio_acc_done done = nadata->enable_done;
+	void *done_data = nadata->enable_done_data;
 
-    stdiona_lock(w->nadata);
-    stdiona_deref_and_unlock(w->nadata);
-
-    w->o->free(w->o, w);
+	nadata->enable_done = NULL;
+	stdiona_unlock(nadata);
+	done(nadata->acc, done_data);
+	stdiona_lock(nadata);
+    }
+    stdiona_deref_and_unlock(nadata);
 }
 
 static int
@@ -1389,26 +1400,14 @@ stdiona_set_accept_callback_enable(struct gensio_accepter *accepter,
     struct stdiona_data *nadata = gensio_acc_get_gensio_data(accepter);
     int rv = 0;
 
-    if (done) {
-	struct gensio_os_funcs *o = nadata->o;
-	struct stdiona_waiters *w = o->zalloc(o, sizeof(*w));
-
-	if (!w)
-	    rv = GE_NOMEM;
-	else {
-	    w->o = o;
-	    w->done = done;
-	    w->done_data = done_data;
-	    w->nadata = nadata;
-	    w->runner = o->alloc_runner(o, waiter_runner_cb, w);
-	    if (!w->runner) {
-		o->free(o, w);
-		rv = GE_NOMEM;
-	    } else {
-		stdiona_ref(nadata);
-		o->run(w->runner);
-	    }
-	}
+    /* FIXME - there's no real enable for this, maybe there should be? */
+    if (nadata->enable_done) {
+	rv = GE_INUSE;
+    } else {
+	nadata->enable_done = done;
+	nadata->enable_done_data = done_data;
+	stdiona_ref(nadata);
+	nadata->o->run(nadata->enable_done_runner);
     }
 
     return rv;
@@ -1479,6 +1478,14 @@ stdio_gensio_accepter_alloc(const char * const args[],
 	return GE_NOMEM;
     }
 
+    nadata->enable_done_runner = nadata->o->alloc_runner(nadata->o,
+							 enable_done_op,
+							 nadata);
+    if (!nadata->enable_done_runner) {
+	stdiona_finish_free(nadata);
+	return err;
+    }
+
     err = setup_self(nadata, false);
     if (err) {
 	stdiona_finish_free(nadata);
