@@ -14,10 +14,12 @@
 #include <assert.h>
 
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_class.h>
 #include <gensio/argvutils.h>
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_builtins.h>
+#include <gensio/gensio_list.h>
 
 #ifdef ENABLE_INTERNAL_TRACE
 #define DEBUG_STATE
@@ -44,8 +46,6 @@ struct udpn_data {
 
     struct gensio_os_funcs *o;
 
-    unsigned int refcount;
-
     /* iod the original request came in on, for sending. */
     struct gensio_iod *myiod;
 
@@ -57,6 +57,7 @@ struct udpn_data {
     bool write_pending; /* Need to redo the write callback. */
     bool in_open_cb;	/* Currently in an open callback. */
     bool in_close_cb;	/* Currently in a close callback. */
+    bool extrainfo;	/* Deliver extrainfo to user? */
 
     enum udpn_state state;
     bool freed;		/* Freed during the close process. */
@@ -80,15 +81,6 @@ struct udpn_data {
 
 struct udpna_data;
 
-struct udpna_waiters {
-    struct gensio_os_funcs *o;
-    struct udpna_data *nadata;
-    gensio_acc_done done;
-    void *done_data;
-    struct gensio_runner *runner;
-    struct udpna_waiters *next;
-};
-
 #ifdef DEBUG_STATE
 struct udp_state_trace {
     enum udpn_state old_state;
@@ -130,7 +122,10 @@ struct udpna_data {
     struct gensio_runner *deferred_op_runner;
 
     bool in_new_connection;
-    struct udpna_waiters *acc_disable_waiters;
+
+    struct gensio_runner *enable_done_runner;
+    gensio_acc_done enable_done;
+    void *enable_done_data;
 
     bool is_dummy; /* Am I a dummy udpna? */
 
@@ -149,6 +144,8 @@ struct udpna_data {
     unsigned int   nr_fds;
     unsigned int opensock_flags;
 
+    unsigned int extrainfo; /* Is extrainfo enabled or disabled in the iod? */
+
     bool nocon;		/* Disable connection-oriented handling. */
     struct gensio_addr *curr_recvaddr;	/* Address of current received packet */
 
@@ -413,6 +410,8 @@ udpna_do_free(struct udpna_data *nadata)
 
     if (nadata->deferred_op_runner)
 	nadata->o->free_runner(nadata->deferred_op_runner);
+    if (nadata->enable_done_runner)
+	nadata->o->free_runner(nadata->enable_done_runner);
     if (nadata->ai)
 	gensio_addr_free(nadata->ai);
     if (nadata->fds)
@@ -459,10 +458,10 @@ udpna_check_finish_free(struct udpna_data *nadata)
 static void
 udpn_do_free(struct udpn_data *ndata)
 {
-    if (ndata->deferred_op_runner)
-	ndata->o->free_runner(ndata->deferred_op_runner);
     if (ndata->io)
 	gensio_data_free(ndata->io);
+    if (ndata->deferred_op_runner)
+	ndata->o->free_runner(ndata->deferred_op_runner);
     if (ndata->raddr)
 	gensio_addr_free(ndata->raddr);
     ndata->o->free(ndata->o, ndata);
@@ -551,10 +550,12 @@ udpn_finish_read(struct udpn_data *ndata)
     struct gensio *io = ndata->io;
     gensiods count;
     char raddrdata[200];
-    const char *auxmem[2] = { NULL, NULL };
+    char daddrdata[200];
+    char ifidx[20];
+    const char *auxmem[4] = { NULL, NULL, NULL, NULL };
     const char *const *auxdata;
     int err;
-    gensiods addrlen = sizeof(raddrdata), pos = 5;
+    gensiods pos;
 
  retry:
     udpna_unlock(nadata);
@@ -564,16 +565,45 @@ udpn_finish_read(struct udpn_data *ndata)
     auxdata = auxmem;
     auxmem[0] = raddrdata;
     strcpy(raddrdata, "addr:");
+    pos = 5;
     err = gensio_addr_to_str(nadata->curr_recvaddr, raddrdata, &pos,
-			     addrlen);
+			     sizeof(raddrdata));
     if (err) {
 	strcpy(raddrdata, "err:addr:");
 	strncpy(raddrdata + 9, gensio_err_to_str(err), sizeof(raddrdata) - 9);
 	raddrdata[sizeof(raddrdata) - 1] = '\0';
     }
 
-    gensio_cb(io, GENSIO_EVENT_READ, 0, nadata->read_data, &count, auxdata);
+    if (ndata->extrainfo) {
+	/* Get the ifidx */
+	if (gensio_addr_next(nadata->curr_recvaddr)) {
+	    pos = 0;
+	    err = gensio_addr_to_str(nadata->curr_recvaddr, ifidx, &pos,
+				     sizeof(ifidx));
+	    if (!err)
+		auxmem[1] = ifidx;
+	}
+	/* Get the destination address */
+	if (gensio_addr_next(nadata->curr_recvaddr)) {
+	    strncpy(daddrdata, "daddr:", sizeof(daddrdata));
+	    pos = 6;
+	    err = gensio_addr_to_str(nadata->curr_recvaddr, daddrdata, &pos,
+				     sizeof(daddrdata));
+	    if (!err) {
+		/* Chop off the ,0 at the end. */
+		pos -= 2;
+		if (daddrdata[pos] == ',' && daddrdata[pos + 1] == '0')
+		    daddrdata[pos] = '\0';
+		auxmem[2] = daddrdata;
+	    }
+	}
+    }
+
+    err = gensio_cb(io, GENSIO_EVENT_READ, 0, nadata->read_data,
+		    &count, auxdata);
     udpna_lock(nadata);
+    if (err)
+	goto out;
 
     if (ndata->state == UDPN_IN_CLOSE) {
 	udpn_finish_close(nadata, ndata);
@@ -764,10 +794,6 @@ udpn_free(struct gensio *io)
     struct udpna_data *nadata = ndata->nadata;
 
     udpna_lock_and_ref(nadata);
-    assert(ndata->refcount > 0);
-    if (--ndata->refcount > 0)
-	goto out_unlock;
-
     ndata->freed = true;
     if (ndata->state == UDPN_IN_CLOSE)
 	ndata->close_done = NULL;
@@ -775,21 +801,9 @@ udpn_free(struct gensio *io)
 	udpn_start_close(ndata, NULL, NULL);
     else if (!ndata->in_close_cb && !ndata->deferred_op_pending)
 	udpn_finish_free(ndata);
- out_unlock:
     udpna_deref_and_unlock(nadata);
 }
 
-static void
-udpn_do_ref(struct gensio *io)
-{
-    struct udpn_data *ndata = gensio_get_gensio_data(io);
-    struct udpna_data *nadata = ndata->nadata;
-
-    udpna_lock(nadata);
-    ndata->refcount++;
-    udpna_unlock(nadata);
-}
-
 static void
 udpn_set_read_callback_enable(struct gensio *io, bool enabled)
 {
@@ -878,6 +892,7 @@ static void
 udpn_handle_write_incoming(struct udpna_data *nadata, struct udpn_data *ndata)
 {
     struct gensio *io = ndata->io;
+    int err;
 
     if (ndata->in_write) {
 	/* Only one write callback at a time. */
@@ -887,14 +902,17 @@ udpn_handle_write_incoming(struct udpna_data *nadata, struct udpn_data *ndata)
     ndata->in_write = true;
  retry:
     udpna_unlock(nadata);
-    gensio_cb(io, GENSIO_EVENT_WRITE_READY, 0, NULL, NULL, NULL);
+    err = gensio_cb(io, GENSIO_EVENT_WRITE_READY, 0, NULL, NULL, NULL);
     udpna_lock(nadata);
+    if (err)
+	goto out;
     if (ndata->write_pending) {
 	/* Another write came in while we were unlocked.  Retry. */
 	ndata->write_pending = false;
 	if (ndata->write_enabled)
 	    goto retry;
     }
+ out:
     ndata->in_write = false;
 
     if (ndata->state == UDPN_IN_CLOSE)
@@ -1106,9 +1124,6 @@ udpn_control(struct gensio *io, bool get, int option,
 		return err;
 	    *datalen = snprintf(data, *datalen, "%u", ttl);
 	} else {
-	    unsigned int ttl;
-	    gensiods size = sizeof(ttl);
-
 	    ttl = strtoul(data, NULL, 0);
 	    return o->sock_control(iod, GENSIO_SOCKCTL_SET_MCAST_TTL,
 				   &ttl, &size);
@@ -1116,6 +1131,40 @@ udpn_control(struct gensio *io, bool get, int option,
 	break;
     }
 
+    case GENSIO_CONTROL_EXTRAINFO: {
+	int val;
+	gensiods size = sizeof(val);
+	struct gensio_iod *iod = nadata->fds->iod;
+
+	if (get) {
+	    err = o->sock_control(iod, GENSIO_SOCKCTL_GET_EXTRAINFO,
+				  &val, &size);
+	    if (err)
+		return err;
+	    *datalen = snprintf(data, *datalen, "%u", val);
+	} else {
+	    val = !!strtoul(data, NULL, 0);
+	    udpna_lock(nadata);
+	    if (ndata->extrainfo != val) {
+		err = 0;
+		if ((val && nadata->extrainfo == 0) ||
+			(!val && nadata->extrainfo == 1)) {
+		    err = o->sock_control(iod, GENSIO_SOCKCTL_SET_EXTRAINFO,
+					  &val, &size);
+		    if (err)
+			return err;
+		    ndata->extrainfo = val;
+		    if (val)
+			nadata->extrainfo++;
+		    else
+			nadata->extrainfo--;
+		}
+	    }
+	    udpna_unlock(nadata);
+	}
+	break;
+    }
+
     default:
 	return GE_NOTSUP;
     }
@@ -1141,10 +1190,6 @@ gensio_udp_func(struct gensio *io, int func, gensiods *count,
 	udpn_free(io);
 	return 0;
 
-    case GENSIO_FUNC_REF:
-	udpn_do_ref(io);
-	return 0;
-
     case GENSIO_FUNC_SET_READ_CALLBACK:
 	udpn_set_read_callback_enable(io, buflen);
 	return 0;
@@ -1177,7 +1222,6 @@ udp_alloc_gensio(struct udpna_data *nadata, struct gensio_iod *iod,
 	return NULL;
 
     ndata->o = nadata->o;
-    ndata->refcount = 1;
     ndata->nadata = nadata;
 
     ndata->deferred_op_runner = ndata->o->alloc_runner(ndata->o,
@@ -1218,7 +1262,6 @@ udpna_readhandler(struct gensio_iod *iod, void *cbdata)
 {
     struct udpna_data *nadata = cbdata;
     struct udpn_data *ndata;
-    struct udpna_waiters *waiters = NULL, *next;
     gensiods datalen;
     int err;
 
@@ -1284,9 +1327,16 @@ udpna_readhandler(struct gensio_iod *iod, void *cbdata)
 
     udpna_lock(nadata);
     ndata->in_read = false;
+    while (nadata->enable_done) {
+	gensio_acc_done enable_done = nadata->enable_done;
+	void *enable_done_data = nadata->enable_done_data;
+
+	nadata->enable_done = NULL;
+	udpna_unlock(nadata);
+	enable_done(nadata->acc, enable_done_data);
+	udpna_lock(nadata);
+    }
     nadata->in_new_connection = false;
-    waiters = nadata->acc_disable_waiters;
-    nadata->acc_disable_waiters = NULL;
 
     if (ndata->state == UDPN_OPEN) {
     got_ndata:
@@ -1328,15 +1378,6 @@ udpna_readhandler(struct gensio_iod *iod, void *cbdata)
     }
  out_unlock:
     udpna_deref_and_unlock(nadata);
-
-    while (waiters) {
-	next = waiters->next;
-	waiters->done(nadata->acc, waiters->done_data);
-	nadata->o->free(nadata->o, waiters);
-	waiters = next;
-    }
-
-    return;
 }
 
 static int
@@ -1389,17 +1430,21 @@ udpna_shutdown(struct gensio_accepter *accepter,
 }
 
 static void
-waiter_runner_cb(struct gensio_runner *runner, void *cb_data)
+udpna_enable_op(struct gensio_runner *runner, void *cb_data)
 {
-    struct udpna_waiters *w = cb_data;
+    struct udpna_data *nadata = cb_data;
 
-    w->done(w->nadata->acc, w->done_data);
-    w->o->free_runner(w->runner);
+    udpna_lock(nadata);
+    if (nadata->enable_done) {
+	gensio_acc_done done = nadata->enable_done;
+	void *done_data = nadata->enable_done_data;
 
-    udpna_lock(w->nadata);
-    udpna_deref_and_unlock(w->nadata);
-
-    w->o->free(w->o, w);
+	nadata->enable_done = NULL;
+	udpna_unlock(nadata);
+	done(nadata->acc, done_data);
+	udpna_lock(nadata);
+    }
+    udpna_deref_and_unlock(nadata);
 }
 
 static int
@@ -1410,31 +1455,15 @@ udpna_set_accept_callback_enable(struct gensio_accepter *accepter, bool enabled,
     int rv = 0;
 
     udpna_lock(nadata);
-    nadata->enabled = enabled;
-    if (done) {
-	struct gensio_os_funcs *o = nadata->o;
-	struct udpna_waiters *w = o->zalloc(o, sizeof(*w));
-
-	if (!w)
-	    rv = GE_NOMEM;
-	else {
-	    w->o = o;
-	    w->done = done;
-	    w->done_data = done_data;
-	    if (nadata->in_new_connection) {
-		w->next = nadata->acc_disable_waiters;
-		nadata->acc_disable_waiters = w;
-	    } else {
-		w->nadata = nadata;
-		w->runner = o->alloc_runner(o, waiter_runner_cb, w);
-		if (!w->runner) {
-		    o->free(o, w);
-		    rv = GE_NOMEM;
-		} else {
-		    udpna_ref(nadata);
-		    o->run(w->runner);
-		}
-	    }
+    if (nadata->enable_done) {
+	rv = GE_INUSE;
+    } else {
+	nadata->enabled = enabled;
+	nadata->enable_done = done;
+	nadata->enable_done_data = done_data;
+	if (!nadata->in_new_connection) {
+	    udpna_ref(nadata);
+	    nadata->o->run(nadata->enable_done_runner);
 	}
     }
     udpna_unlock(nadata);
@@ -1641,6 +1670,12 @@ i_udp_gensio_accepter_alloc(struct gensio_addr *iai, gensiods max_read_size,
     if (!nadata->deferred_op_runner)
 	goto out_nomem;
 
+    nadata->enable_done_runner = nadata->o->alloc_runner(nadata->o,
+							 udpna_enable_op,
+							 nadata);
+    if (!nadata->enable_done_runner)
+	goto out_nomem;
+
     nadata->lock = o->alloc_lock(o);
     if (!nadata->lock)
 	goto out_nomem;
