@@ -27,10 +27,9 @@
 #include <bcrypt.h>
 
 #include <gensio/gensio.h>
+#include <gensio/gensio_os_funcs.h>
 #include <gensio/sergensio.h>
 #include <gensio/gensio_list.h>
-#include <gensio/gensio_err.h>
-#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_osops_addrinfo.h>
 #include <gensio/gensio_osops_stdsock.h>
 #include <gensio/gensio_osops.h>
@@ -131,7 +130,7 @@ struct gensio_iod_win {
     void *cb_data;
 };
 
-#define iod_to_win(iod) gensio_container_of(iod, struct gensio_iod_win, iod);
+#define iod_to_win(i) gensio_container_of(i, struct gensio_iod_win, iod);
 
 enum win_timer_state {
     WIN_TIMER_STOPPED = 0,
@@ -226,7 +225,7 @@ struct gensio_data {
 #define glock_unlock(d)  LeaveCriticalSection(&(d)->glock)
 
 static void *
-win_zalloc(struct gensio_os_funcs *o, unsigned int size)
+win_zalloc(struct gensio_os_funcs *o, gensiods size)
 {
     struct gensio_data *d = o->user_data;
 
@@ -1197,6 +1196,7 @@ winsock_func(LPVOID data)
 	    /* FIXME - check if events changed? */
 	    rv = WSAEventSelect(wiod->fd, swiod->sockev, events);
 	    if (rv == SOCKET_ERROR) {
+		EnterCriticalSection(&wiod->lock);
 		if (!wiod->werr)
 		    wiod->werr = WSAGetLastError();
 		wiod->closed = TRUE;
@@ -1427,6 +1427,7 @@ struct gensio_iod_win_oneway {
 
     struct gensio_circbuf *buf;
 
+    BOOL is_console;
     BOOL readable;
 
     BOOL do_flush; /* Tell out to flush it's data. */
@@ -1436,6 +1437,8 @@ struct gensio_iod_win_oneway {
 					       struct gensio_iod_win_oneway, \
 					       wiod);
 
+static void check_winsize(struct gensio_iod_win_oneway *owiod);
+
 static DWORD WINAPI
 win_oneway_in_thread(LPVOID data)
 {
@@ -1463,6 +1466,29 @@ win_oneway_in_thread(LPVOID data)
 
 	    gensio_circbuf_next_write_area(owiod->buf, &readpos, &readsize);
 	    LeaveCriticalSection(&wiod->lock);
+	    if (owiod->is_console) {
+		while (true) {
+		    rvw = WaitForSingleObject(owiod->ioh, INFINITE);
+		    if (wiod->done)
+			break;
+		    if (rvw == WAIT_OBJECT_0) {
+			INPUT_RECORD r;
+
+			if (!PeekConsoleInput(owiod->ioh, &r, 1, &nread))
+			    goto out_err;
+			if (r.EventType == KEY_EVENT
+				&& r.Event.KeyEvent.bKeyDown)
+			    /*
+			     * Only let key down events through,
+			     * that's all we care about ReadFile handling.
+			     */
+			    break;
+			ReadConsoleInput(owiod->ioh, &r, 1, &nread);
+			if (r.EventType == WINDOW_BUFFER_SIZE_EVENT)
+			    goto handle_winsize;
+		    }
+		}
+	    }
 	    rvb = ReadFile(owiod->ioh, readpos, readsize, &nread, NULL);
 	    EnterCriticalSection(&wiod->lock);
 	    if (!rvb) {
@@ -1475,7 +1501,10 @@ win_oneway_in_thread(LPVOID data)
 		    goto continue_loop;
 		goto out_err;
 	    }
-
+	handle_winsize:
+	    if (owiod->is_console) {
+		check_winsize(owiod);
+	    }
 	    if (nread == 0) {
 		/* EOF (^Z<cr>) from windows. */
 		if (wiod->is_raw) {
@@ -1809,16 +1838,15 @@ win_console_makeraw(struct gensio_iod_win *wiod)
     struct gensio_iod_win_console *cowiod = owiod_to_winconsole(owiod);
     int rv;
 
-    if (!owiod->readable)
-	/*
-	 * Nothing to do for stdout. Disabling ENABLE_PROCESSED_OUTPUT
-	 * is not a good thing to do.
-	 */
-	return 0;
-
-    rv = gensio_win_stdio_makeraw(wiod->iod.f, owiod->ioh, &cowiod->mode);
-    if (!rv)
-	wiod->is_raw = TRUE;
+    if (owiod->readable) {
+	rv = gensio_win_stdin_makeraw(wiod->iod.f, owiod->ioh, &cowiod->mode);
+	if (!rv)
+	    wiod->is_raw = TRUE;
+    } else {
+	rv = gensio_win_stdout_makeraw(wiod->iod.f, owiod->ioh, &cowiod->mode);
+	if (!rv)
+	    wiod->is_raw = TRUE;
+    }
     return rv;
 }
 
@@ -1840,6 +1868,7 @@ win_iod_console2_init(struct gensio_iod_win *wiod, void *cb_data)
 
     owiod->readable = readable;
     owiod->ioh = (HANDLE) wiod->fd;
+    owiod->is_console = true;
     return win_iod_oneway_init(wiod, NULL);
 }
 
@@ -2386,6 +2415,7 @@ win_iod_dev_init(struct gensio_iod_win *wiod, void *cb_data)
 	rv = GE_NOTSUP;
 	goto out_err;
     }
+    memset(&props, 0, sizeof(props));
     if (!GetCommProperties(twiod->ioh, &props))
 	goto out_err_conv;
 
@@ -3271,10 +3301,24 @@ gensio_win_funcs_alloc(struct gensio_os_funcs **ro)
 struct gensio_os_proc_data {
     struct gensio_os_funcs *o;
     lock_type lock;
+
     BOOL term_handler_set;
     BOOL got_term_sig;
     void (*term_handler)(void *handler_data);
     void *term_handler_data;
+
+    BOOL winsize_handler_set;
+    HANDLE outcon;
+    int x_chrs;
+    int y_chrs;
+    int x_bits;
+    int y_bits;
+    BOOL got_winsize_sig;
+    void (*winsize_handler)(int x_chrs, int y_chrs,
+			    int x_bits, int y_bits,
+			    void *handler_data);
+    void *winsize_handler_data;
+
     HANDLE global_waiter;
 };
 static struct gensio_os_proc_data proc_data;
@@ -3301,20 +3345,26 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     return 0;
 }
 
-static BOOL
-ConCtrlHandler(DWORD dwCtrlType)
+static void
+proc_release_sem(struct gensio_os_proc_data *data)
 {
     BOOL rvb;
 
+    rvb = ReleaseSemaphore(proc_data.global_waiter, 1, NULL);
+    if (!rvb)
+	/* Too many posts is improbable, but ok. */
+	assert(GetLastError() == ERROR_TOO_MANY_POSTS);
+}
+
+static BOOL
+ConCtrlHandler(DWORD dwCtrlType)
+{
     switch (dwCtrlType) {
     case CTRL_C_EVENT:
     case CTRL_CLOSE_EVENT:
     case CTRL_SHUTDOWN_EVENT:
 	proc_data.got_term_sig = true;
-	rvb = ReleaseSemaphore(proc_data.global_waiter, 1, NULL);
-	if (!rvb)
-	    /* Too many posts is improbable, but ok. */
-	    assert(GetLastError() == ERROR_TOO_MANY_POSTS);
+	proc_release_sem(&proc_data);
 	return TRUE;
 
     default:
@@ -3322,6 +3372,42 @@ ConCtrlHandler(DWORD dwCtrlType)
     }
 }
 
+void check_winsize(struct gensio_iod_win_oneway *owiod)
+{
+    struct gensio_iod_win *wiod = &owiod->wiod;
+    struct gensio_iod *iod = &wiod->iod;
+    struct gensio_os_funcs *o = iod->f;
+    struct gensio_data *d = o->user_data;
+    struct gensio_os_proc_data *data = d->proc_data;
+    CONSOLE_SCREEN_BUFFER_INFOEX sbi;
+    int x_chrs, y_chrs;
+
+    LOCK(&data->lock);
+    if (!data->outcon) {
+	data->outcon = CreateFileA("CONOUT$", GENERIC_WRITE | GENERIC_READ, 0,
+				   NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
+				   NULL);
+	if (data->outcon == INVALID_HANDLE_VALUE) {
+	    data->outcon = NULL;
+	    goto out_unlock;
+	}
+    }
+    memset(&sbi, 0, sizeof(sbi));
+    sbi.cbSize = sizeof(sbi);
+    if (!GetConsoleScreenBufferInfoEx(data->outcon, &sbi))
+	goto out_unlock;
+    x_chrs = sbi.dwSize.X;
+    y_chrs = sbi.srWindow.Bottom - sbi.srWindow.Top + 1;
+    if (x_chrs != data->x_chrs || y_chrs != data->y_chrs) {
+	data->x_chrs = x_chrs;
+	data->y_chrs = y_chrs;
+	data->got_winsize_sig = TRUE;
+	proc_release_sem(data);
+    }
+ out_unlock:
+    UNLOCK(&data->lock);
+}
+
 void
 gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
 {
@@ -3329,6 +3415,10 @@ gensio_os_proc_cleanup(struct gensio_os_proc_data *data)
 	SetConsoleCtrlHandler(ConCtrlHandler, false);
 	data->term_handler_set = false;
     }
+    if (data->outcon) {
+	CloseHandle(data->outcon);
+	data->outcon = NULL;
+    }
     if (data->global_waiter) {
 	CloseHandle(data->global_waiter);
 	data->global_waiter = NULL;
@@ -3348,7 +3438,15 @@ gensio_os_proc_check_handlers(struct gensio_os_proc_data *data)
     LOCK(&data->lock);
     if (data->got_term_sig) {
 	data->got_term_sig = FALSE;
-	data->term_handler(data->term_handler_data);
+	if (data->term_handler)
+	    data->term_handler(data->term_handler_data);
+    }
+    if (data->got_winsize_sig) {
+	data->got_winsize_sig = FALSE;
+	if (data->winsize_handler)
+	    data->winsize_handler(data->x_chrs, data->y_chrs,
+				  data->x_bits, data->y_bits,
+				  data->winsize_handler_data);
     }
     UNLOCK(&data->lock);
 }
@@ -3358,12 +3456,18 @@ gensio_os_proc_register_term_handler(struct gensio_os_proc_data *data,
 				     void (*handler)(void *handler_data),
 				     void *handler_data)
 {
-    if (SetConsoleCtrlHandler(ConCtrlHandler, TRUE) == 0)
-	return gensio_os_err_to_err(data->o, GetLastError());
-    data->term_handler = handler;
-    data->term_handler_data = handler_data;
-    data->term_handler_set = true;
-    return 0;
+    int err = 0;
+
+    LOCK(&data->lock);
+    if (SetConsoleCtrlHandler(ConCtrlHandler, TRUE) == 0) {
+	err = gensio_os_err_to_err(data->o, GetLastError());
+    } else {
+	data->term_handler = handler;
+	data->term_handler_data = handler_data;
+	data->term_handler_set = true;
+    }
+    UNLOCK(&data->lock);
+    return err;
 }
 
 int
@@ -3374,6 +3478,28 @@ gensio_os_proc_register_reload_handler(struct gensio_os_proc_data *data,
     return GE_NOTSUP;
 }
 
+int
+gensio_os_proc_register_winsize_handler(struct gensio_os_proc_data *data,
+					struct gensio_iod *console_iod,
+					void (*handler)(int x_chrs, int y_chrs,
+							int x_bits, int y_bits,
+							void *handler_data),
+					void *handler_data)
+{
+    struct gensio_iod_win *wiod = iod_to_win(console_iod);
+    int err = 0;
+
+    if (wiod->type != GENSIO_IOD_CONSOLE)
+	return GE_INVAL;
+    LOCK(&data->lock);
+    data->winsize_handler = handler;
+    data->winsize_handler_data = handler_data;
+    data->winsize_handler_set = true;
+    UNLOCK(&data->lock);
+    proc_release_sem(data);
+    return err;
+}
+
 struct gensio_thread {
     struct gensio_os_funcs *o;
     HANDLE handle;
