@@ -304,6 +304,22 @@ sel_wake_all(struct selector_s *sel)
     sel_timer_unlock(sel);
 }
 
+static void
+i_sel_wake_first(struct selector_s *sel)
+{
+    sel_wait_list_t *item;
+
+    item = sel->wait_list.next;
+    if (item->send_sig && item != &sel->wait_list) {
+#ifdef BROKEN_PSELECT
+	item->signalled = true;
+	item->wait_time->tv_sec = 0;
+	item->wait_time->tv_nsec = 0;
+#endif
+	item->send_sig(item->thread_id, item->send_sig_cb_data);
+    }
+}
+
 /* See comment on i_wake_sel_thread() for notes on BROKEN_PSELECT. */
 void
 sel_wake_one(struct selector_s *sel, long thread_id, sel_send_sig_cb killer,
@@ -788,16 +804,24 @@ sel_alloc_timer(struct selector_s     *sel,
 static int
 sel_stop_timer_i(struct selector_s *sel, sel_timer_t *timer)
 {
-    if (timer->val.stopped)
-	return ETIMEDOUT;
+    int rv = 0;
 
+    if (timer->val.stopped)
+	rv = ETIMEDOUT;
+    /*
+     * It should not be possible for the timer to be stopped but in
+     * the heap, but that's happening sometimes.  (The opposite is
+     * possible, a timer can be not stopped but not in the heap; that
+     * is used to signal a timer restart on return from a timer
+     * handler.)  So make sure it's not in the heap.
+     */
     if (timer->val.in_heap) {
 	theap_remove(&sel->timer_heap, timer);
 	timer->val.in_heap = 0;
     }
     timer->val.stopped = 1;
 
-    return 0;
+    return rv;
 }
 
 int
@@ -896,6 +920,7 @@ sel_stop_timer_with_done(sel_timer_t *timer,
     }
     sel_get_monotonic_time(&timer->val.timeout);
     theap_add(&sel->timer_heap, timer);
+    timer->val.in_heap = 1;
 
  out_unlock:
     sel_timer_unlock(sel);
@@ -1039,6 +1064,8 @@ sel_run(sel_runner_t *runner, sel_runner_func_t func, void *cb_data)
 	sel->runner_head = runner;
 	sel->runner_tail = runner;
     }
+    /* Make sure someone is awake to run the runner. */
+    i_sel_wake_first(sel);
     sel_timer_unlock(sel);
     return 0;
 }
@@ -1539,6 +1566,7 @@ sel_free_selector(struct selector_s *sel)
     elem = theap_get_top(&(sel->timer_heap));
     while (elem) {
 	theap_remove(&(sel->timer_heap), elem);
+	elem->val.in_heap = 0;
 	free(elem);
 	elem = theap_get_top(&(sel->timer_heap));
     }
