@@ -503,7 +503,7 @@ sterm_sbreak(struct sergensio *sio, int breakv,
 	     void *cb_data)
 {
     return serconf_set_get(sergensio_get_gensio_data(sio),
-			   GENSIO_IOD_CONTROL_SEND_BREAK, breakv,
+			   GENSIO_IOD_CONTROL_SET_BREAK, breakv,
 			   sterm_xlat_sbreak, done, cb_data);
 }
 
@@ -861,6 +861,7 @@ sterm_sub_open(void *handler_data, struct gensio_iod **riod)
     }
 
     sdata->timer_stopped = false;
+    sdata->iod = NULL; /* If it's a re-open make sure this is clear. */
 
     options = GENSIO_OPEN_OPTION_WRITEABLE;
     if (!sdata->write_only)
@@ -938,14 +939,18 @@ sterm_sub_open(void *handler_data, struct gensio_iod **riod)
     if (!sdata->write_only && !sdata->disablebreak) {
 	err = o->iod_control(sdata->iod, GENSIO_IOD_CONTROL_SET_BREAK,
 			     false, sdata->disablebreak);
-	if (err) {
-	    gensio_log(o, GENSIO_LOG_ERR,
+	if (err)
+	    gensio_log(o, GENSIO_LOG_WARNING,
 		       "serialdev: "
 		       "Setting break failed on %s, "
-		       "try adding the nobreak option",
-		       sdata->devname);
-	    goto out_uucp;
-	}
+		       "try adding the nobreak option: %s",
+		       sdata->devname, gensio_err_to_str(err));
+	/*
+	 * Do not fail on an error here.  There are USB and bluetooth devices that fail
+	 * this because they don't implement it, but it's impossible to tell beforehand
+	 * that they will fail.  This shouldn't be able to fail on a working device, so
+	 * just log and allow it.
+	 */
     }
 
     sterm_lock(sdata);
@@ -980,6 +985,8 @@ sterm_free(void *handler_data)
 {
     struct sterm_data *sdata = handler_data;
 
+    if (sdata->sio)
+	sergensio_data_free(sdata->sio);
     serconf_clear_q(sdata);
     if (sdata->rs485)
 	sdata->o->free(sdata->o, sdata->rs485);
@@ -991,8 +998,6 @@ sterm_free(void *handler_data)
 	sdata->o->free(sdata->o, sdata->devname);
     if (sdata->deferred_op_runner)
 	sdata->o->free_runner(sdata->deferred_op_runner);
-    if (sdata->sio)
-	sergensio_data_free(sdata->sio);
     sdata->o->free(sdata->o, sdata);
 }
 
@@ -1507,13 +1512,7 @@ serialdev_gensio_alloc(const char *devname, const char * const args[],
 	return GE_NOMEM;
     }
 
-    sdata->sio = sergensio_data_alloc(o, io, sergensio_sterm_func, sdata);
-    if (!sdata->sio) {
-	gensio_free(io);
-	return GE_NOMEM;
-    }
-
-    err = gensio_addclass(io, "sergensio", sdata->sio);
+    err = sergensio_addclass(o, io, sergensio_sterm_func, sdata, &sdata->sio);
     if (err) {
 	gensio_free(io);
 	return err;
