@@ -9,9 +9,7 @@
 
 #include <gensio/gensio.h>
 
-extern bool gensio_uucp_locking_enabled;
-
 void uucp_rm_lock(char *devname);
 
-/* returns 0=OK, -1=error (errno will be set), >0=pid of locking process */
-int uucp_mk_lock(char *devname);
+/* Returns gensio errno. */
+int uucp_mk_lock(struct gensio_os_funcs *o, char *devname);
