@@ -5,6 +5,7 @@
  *  SPDX-License-Identifier: LGPL-2.1-only
  */
 
+#include <gensio/gensio.h>
 #include <gensio/gensio_swig.h>
 #include "python_swig_internals.h"
 
@@ -87,13 +88,13 @@ OI_PI_FromStringN(const char *s)
 #define ref_swig_cb(cb, func) gensio_python_ref_swig_cb_i(cb)
 
 static swig_ref
-swig_make_ref_i(void *item, swig_type_info *class)
+swig_make_ref_i(void *item, swig_type_info *classt)
 {
     swig_ref    rv;
     OI_PY_STATE gstate;
 
     gstate = OI_PY_STATE_GET();
-    rv.val = SWIG_NewPointerObj(item, class, SWIG_POINTER_OWN);
+    rv.val = SWIG_NewPointerObj(item, classt, SWIG_POINTER_OWN);
     OI_PY_STATE_PUT(gstate);
     return rv;
 }
@@ -114,22 +115,25 @@ swig_finish_call_rv_gensiods(swig_cb_val *cb, const char *method_name,
 {
     PyObject *o;
     gensiods rv = 0;
+    /* cb can go away in the callback, fetch the class here. */
+    PyObject *t = PyObject_GetAttrString(cb, "__class__");
 
     o = swig_finish_call_rv(cb, method_name, args, optional);
     if (o) {
 	rv = PyLong_AsUnsignedLong(o);
 	if (PyErr_Occurred()) {
-	    PyObject *t = PyObject_GetAttrString(cb, "__class__");
 	    PyObject *c = PyObject_GetAttrString(t, "__name__");
-	    const char *class = OI_PI_AsString(c);
+	    const char *classt = OI_PI_AsString(c);
 
+	    Py_DECREF(c);
 	    PyErr_Format(PyExc_RuntimeError, "gensio callback: "
 			 "Class '%s' method '%s' did not return "
-			 "an integer\n", class, method_name);
+			 "an integer\n", classt, method_name);
 	    wake_curr_waiter();
 	}
 	Py_DECREF(o);
     }
+    Py_DECREF(t);
 
     return rv;
 }
@@ -147,11 +151,11 @@ swig_finish_call_rv_int(swig_cb_val *cb, const char *method_name,
 	if (PyErr_Occurred()) {
 	    PyObject *t = PyObject_GetAttrString(cb, "__class__");
 	    PyObject *c = PyObject_GetAttrString(t, "__name__");
-	    const char *class = OI_PI_AsString(c);
+	    const char *classt = OI_PI_AsString(c);
 
 	    PyErr_Format(PyExc_RuntimeError, "gensio callback: "
 			 "Class '%s' method '%s' did not return "
-			 "an integer\n", class, method_name);
+			 "an integer\n", classt, method_name);
 	    wake_curr_waiter();
 	}
 	Py_DECREF(o);
@@ -192,7 +196,8 @@ void os_funcs_unlock(struct os_funcs_data *odata)
 static void
 os_funcs_ref(struct gensio_os_funcs *o)
 {
-    struct os_funcs_data *odata = o->other_data;
+    struct os_funcs_data *odata =
+	(struct os_funcs_data *) gensio_os_funcs_get_data(o);
 
     os_funcs_lock(odata);
     odata->refcount++;
@@ -211,7 +216,7 @@ alloc_gensio_data(struct gensio_os_funcs *o, swig_cb *handler)
 {
     struct gensio_data *data;
 
-    data = malloc(sizeof(*data));
+    data = (struct gensio_data *) malloc(sizeof(*data));
     if (!data)
 	return NULL;
     data->tmpval = false;
@@ -237,7 +242,8 @@ free_gensio_data(struct gensio_data *data)
 static void
 ref_gensio_data(struct gensio_data *data)
 {
-    struct os_funcs_data *odata = data->o->other_data;
+    struct os_funcs_data *odata = (struct os_funcs_data *)
+	gensio_os_funcs_get_data(data->o);
 
     os_funcs_lock(odata);
     data->refcount++;
@@ -247,7 +253,8 @@ ref_gensio_data(struct gensio_data *data)
 static void
 deref_gensio_data(struct gensio_data *data, struct gensio *io)
 {
-    struct os_funcs_data *odata = data->o->other_data;
+    struct os_funcs_data *odata = (struct os_funcs_data *)
+	gensio_os_funcs_get_data(data->o);
 
     os_funcs_lock(odata);
     data->refcount--;
@@ -264,7 +271,8 @@ static void
 deref_gensio_accepter_data(struct gensio_data *data,
 			   struct gensio_accepter *acc)
 {
-    struct os_funcs_data *odata = data->o->other_data;
+    struct os_funcs_data *odata = (struct os_funcs_data *)
+	gensio_os_funcs_get_data(data->o);
 
     os_funcs_lock(odata);
     data->refcount--;
@@ -278,32 +286,34 @@ deref_gensio_accepter_data(struct gensio_data *data,
 }
 
 static void
-gensio_ref(struct gensio *io)
+gensio_pyref(struct gensio *io)
 {
-    struct gensio_data *data = gensio_get_user_data(io);
+    struct gensio_data *data = (struct gensio_data *) gensio_get_user_data(io);
 
     ref_gensio_data(data);
 }
 
 static void
-sergensio_ref(struct sergensio *sio)
+sergensio_pyref(struct sergensio *sio)
 {
-    struct gensio_data *data = sergensio_get_user_data(sio);
+    struct gensio_data *data =
+	(struct gensio_data *) sergensio_get_user_data(sio);
 
     ref_gensio_data(data);
 }
 
 static void
-gensio_accepter_ref(struct gensio_accepter *acc)
+gensio_accepter_pyref(struct gensio_accepter *acc)
 {
-    struct gensio_data *data = gensio_acc_get_user_data(acc);
+    struct gensio_data *data =
+	(struct gensio_data *) gensio_acc_get_user_data(acc);
 
     ref_gensio_data(data);
 }
 
 static void
 gensio_open_done(struct gensio *io, int err, void *cb_data) {
-    swig_cb_val *cb = cb_data;
+    swig_cb_val *cb = (swig_cb_val *) cb_data;
     swig_ref io_ref;
     PyObject *args, *o;
     OI_PY_STATE gstate;
@@ -311,7 +321,7 @@ gensio_open_done(struct gensio *io, int err, void *cb_data) {
     gstate = OI_PY_STATE_GET();
 
     io_ref = swig_make_ref(io, gensio);
-    gensio_ref(io);
+    gensio_pyref(io);
     args = PyTuple_New(2);
     PyTuple_SET_ITEM(args, 0, io_ref.val);
     if (err) {
@@ -330,7 +340,7 @@ gensio_open_done(struct gensio *io, int err, void *cb_data) {
 
 static void
 gensio_close_done(struct gensio *io, void *cb_data) {
-    swig_cb_val *cb = cb_data;
+    swig_cb_val *cb = (swig_cb_val *) cb_data;
     swig_ref io_ref;
     PyObject *args;
     OI_PY_STATE gstate;
@@ -339,7 +349,7 @@ gensio_close_done(struct gensio *io, void *cb_data) {
 
     io_ref = swig_make_ref(io, gensio);
     args = PyTuple_New(1);
-    gensio_ref(io);
+    gensio_pyref(io);
     PyTuple_SET_ITEM(args, 0, io_ref.val);
 
     swig_finish_call(cb, "close_done", args, false);
@@ -349,9 +359,9 @@ gensio_close_done(struct gensio *io, void *cb_data) {
 }
 
 static void
-sgensio_call(struct gensio *io, long val, char *func)
+sgensio_call(struct gensio *io, long val, const char *func)
 {
-    struct gensio_data *data = gensio_get_user_data(io);
+    struct gensio_data *data = (struct gensio_data *) gensio_get_user_data(io);
     swig_ref io_ref;
     PyObject *args, *o;
     OI_PY_STATE gstate;
@@ -393,7 +403,7 @@ sgensio_signature(struct gensio *io)
      * FIXME - this is wrong, it is for the client side, but this needs
      * to be the server side code that gets a signature.
      */
-    struct gensio_data *data = gensio_get_user_data(io);
+    struct gensio_data *data = (struct gensio_data *) gensio_get_user_data(io);
     swig_ref io_ref;
     PyObject *args;
     OI_PY_STATE gstate;
@@ -417,7 +427,7 @@ sgensio_signature(struct gensio *io)
 static void
 sgensio_sync(struct gensio *io)
 {
-    struct gensio_data *data = gensio_get_user_data(io);
+    struct gensio_data *data = (struct gensio_data *) gensio_get_user_data(io);
     swig_ref io_ref;
     PyObject *args;
     OI_PY_STATE gstate;
@@ -441,7 +451,7 @@ sgensio_sync(struct gensio *io)
 static void
 sgensio_flowcontrol_state(struct gensio *io, bool val)
 {
-    struct gensio_data *data = gensio_get_user_data(io);
+    struct gensio_data *data = (struct gensio_data *) gensio_get_user_data(io);
     swig_ref io_ref;
     PyObject *args, *o;
     OI_PY_STATE gstate;
@@ -548,7 +558,7 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 		   unsigned char *buf, gensiods *buflen,
 		   const char *const *auxdata)
 {
-    struct gensio_data *data = user_data;
+    struct gensio_data *data = (struct gensio_data *) user_data;
     swig_ref io_ref, new_con;
     PyObject *args, *o;
     OI_PY_STATE gstate;
@@ -622,7 +632,8 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 
 	PyTuple_SET_ITEM(args, 2, gensio_py_handle_auxdata(auxdata));
 
-	swig_finish_call(data->handler_val, "new_channel", args, false);
+	rv = swig_finish_call_rv_int(data->handler_val, "new_channel",
+				     args, false);
 	break;
 
     case GENSIO_EVENT_SEND_BREAK:
@@ -745,7 +756,8 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 
 		rv = OI_PI_AsBytesAndSize(o, &p, &len);
 		if (!rv) {
-		    p2 = data->o->zalloc(data->o, len + 1);
+		    p2 = (unsigned char *)
+			gensio_os_funcs_zalloc(data->o, len + 1);
 		    if (!p2) {
 			rv = GE_NOMEM;
 		    } else {
@@ -835,7 +847,7 @@ gensio_child_event(struct gensio *io, void *user_data, int event, int readerr,
 static void
 gensio_acc_shutdown_done(struct gensio_accepter *accepter, void *cb_data)
 {
-    swig_cb_val *cb = cb_data;
+    swig_cb_val *cb = (swig_cb_val *) cb_data;
     swig_ref acc_ref;
     PyObject *args;
     OI_PY_STATE gstate;
@@ -844,7 +856,7 @@ gensio_acc_shutdown_done(struct gensio_accepter *accepter, void *cb_data)
 
     acc_ref = swig_make_ref(accepter, gensio_accepter);
     args = PyTuple_New(1);
-    gensio_accepter_ref(accepter);
+    gensio_accepter_pyref(accepter);
     PyTuple_SET_ITEM(args, 0, acc_ref.val);
 
     swig_finish_call(cb, "shutdown_done", args, false);
@@ -856,7 +868,7 @@ gensio_acc_shutdown_done(struct gensio_accepter *accepter, void *cb_data)
 static void
 gensio_acc_set_acc_cb_done(struct gensio_accepter *accepter, void *cb_data)
 {
-    swig_cb_val *cb = cb_data;
+    swig_cb_val *cb = (swig_cb_val *) cb_data;
     swig_ref acc_ref;
     PyObject *args;
     OI_PY_STATE gstate;
@@ -865,7 +877,7 @@ gensio_acc_set_acc_cb_done(struct gensio_accepter *accepter, void *cb_data)
 
     acc_ref = swig_make_ref(accepter, gensio_accepter);
     args = PyTuple_New(1);
-    gensio_accepter_ref(accepter);
+    gensio_accepter_pyref(accepter);
     PyTuple_SET_ITEM(args, 0, acc_ref.val);
 
     swig_finish_call(cb, "set_accept_callback_done", args, false);
@@ -879,7 +891,8 @@ gensio_acc_io_call_cb(struct gensio_accepter *accepter, struct gensio *io,
 		      const char *func, int opterr, const char *optstr,
 		      bool optional)
 {
-    struct gensio_data *data = gensio_acc_get_user_data(accepter);
+    struct gensio_data *data =
+	(struct gensio_data *) gensio_acc_get_user_data(accepter);
     swig_ref acc_ref, io_ref;
     PyObject *args, *o;
     int rv;
@@ -899,7 +912,7 @@ gensio_acc_io_call_cb(struct gensio_accepter *accepter, struct gensio *io,
     gensio_set_user_data(io, &tmpdata);
 
     acc_ref = swig_make_ref(accepter, gensio_accepter);
-    gensio_accepter_ref(accepter);
+    gensio_accepter_pyref(accepter);
     io_ref = swig_make_ref(io, gensio);
     if (opterr >= 0)
 	args = PyTuple_New(4);
@@ -935,13 +948,13 @@ static int
 gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 		       int event, void *cdata)
 {
-    struct gensio_data *data = user_data;
+    struct gensio_data *data = (struct gensio_data *) user_data;
     swig_ref acc_ref, io_ref;
     PyObject *args, *o;
     OI_PY_STATE gstate;
     struct gensio_data *iodata;
     struct gensio *io;
-    struct gensio_loginfo *i = cdata;
+    struct gensio_loginfo *i = (struct gensio_loginfo *) cdata;
     struct gensio_acc_password_verify_data *pwvfy;
     struct gensio_acc_postcert_verify_data *postvfy;
     char buf[256];
@@ -969,14 +982,14 @@ gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 	return 0;
 
     case GENSIO_ACC_EVENT_NEW_CONNECTION:
-	io = cdata;
+	io = (struct gensio *) cdata;
 	iodata = alloc_gensio_data(data->o, NULL);
-	gensio_set_callback(cdata /*io*/, gensio_child_event, iodata);
+	gensio_set_callback(io, gensio_child_event, iodata);
 
 	gstate = OI_PY_STATE_GET();
 
 	acc_ref = swig_make_ref(accepter, gensio_accepter);
-	gensio_accepter_ref(accepter);
+	gensio_accepter_pyref(accepter);
 	io_ref = swig_make_ref(io, gensio);
 	args = PyTuple_New(2);
 	PyTuple_SET_ITEM(args, 0, acc_ref.val);
@@ -988,12 +1001,12 @@ gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 	return 0;
 
     case GENSIO_ACC_EVENT_AUTH_BEGIN:
-	return gensio_acc_io_call_cb(accepter, cdata, "auth_begin",
-				     -1, NULL, true);
+	return gensio_acc_io_call_cb(accepter, (struct gensio *) cdata,
+				     "auth_begin", -1, NULL, true);
 
     case GENSIO_ACC_EVENT_PRECERT_VERIFY:
-	return gensio_acc_io_call_cb(accepter, cdata, "precert_verify",
-				     -1, NULL, true);
+	return gensio_acc_io_call_cb(accepter, (struct gensio *) cdata,
+				     "precert_verify", -1, NULL, true);
 
     case GENSIO_ACC_EVENT_POSTCERT_VERIFY:
 	postvfy = (struct gensio_acc_postcert_verify_data *) cdata;
@@ -1021,7 +1034,7 @@ gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 	gensio_set_user_data(io, &tmpdata);
 
 	acc_ref = swig_make_ref(accepter, gensio_accepter);
-	gensio_accepter_ref(accepter);
+	gensio_accepter_pyref(accepter);
 	io_ref = swig_make_ref(io, gensio);
 	args = PyTuple_New(2);
 	PyTuple_SET_ITEM(args, 0, acc_ref.val);
@@ -1069,7 +1082,7 @@ gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 	gensio_set_user_data(io, &tmpdata);
 
 	acc_ref = swig_make_ref(accepter, gensio_accepter);
-	gensio_accepter_ref(accepter);
+	gensio_accepter_pyref(accepter);
 	io_ref = swig_make_ref(io, gensio);
 	args = PyTuple_New(2);
 	PyTuple_SET_ITEM(args, 0, acc_ref.val);
@@ -1087,7 +1100,8 @@ gensio_acc_child_event(struct gensio_accepter *accepter, void *user_data,
 
 		rv = OI_PI_AsBytesAndSize(o, &p, &len);
 		if (!rv) {
-		    p2 = data->o->zalloc(data->o, len + 1);
+		    p2 = (unsigned char *)
+			gensio_os_funcs_zalloc(data->o, len + 1);
 		    if (!p2) {
 			rv = GE_NOMEM;
 		    } else {
@@ -1118,7 +1132,8 @@ struct sergensio_cbdata {
 
 #define sergensio_cbdata(name, h) \
 ({							\
-    struct sergensio_cbdata *cbd = malloc(sizeof(*cbd));	\
+    struct sergensio_cbdata *cbd = (struct sergensio_cbdata *) \
+	malloc(sizeof(*cbd));				       \
     if (cbd) {						\
 	cbd->cbname = stringify(name);			\
 	cbd->h_val = ref_swig_cb(h, name);		\
@@ -1136,7 +1151,7 @@ cleanup_sergensio_cbdata(struct sergensio_cbdata *cbd)
 static void
 sergensio_cb(struct sergensio *sio, int err, unsigned int val, void *cb_data)
 {
-    struct sergensio_cbdata *cbd = cb_data;
+    struct sergensio_cbdata *cbd = (struct sergensio_cbdata *) cb_data;
     swig_ref sio_ref;
     PyObject *o, *args;
     OI_PY_STATE gstate;
@@ -1145,7 +1160,7 @@ sergensio_cb(struct sergensio *sio, int err, unsigned int val, void *cb_data)
 
     sio_ref = swig_make_ref(sio, sergensio);
     args = PyTuple_New(3);
-    sergensio_ref(sio);
+    sergensio_pyref(sio);
     PyTuple_SET_ITEM(args, 0, sio_ref.val);
     if (err) {
 	o = OI_PI_FromString(gensio_err_to_str(err));
@@ -1167,7 +1182,7 @@ static void
 sergensio_sig_cb(struct sergensio *sio, int err,
 		 const char *sig, unsigned int len, void *cb_data)
 {
-    swig_cb_val *h_val = cb_data;
+    swig_cb_val *h_val = (swig_cb_val *) cb_data;
     swig_ref sio_ref;
     PyObject *args, *o;
     OI_PY_STATE gstate;
@@ -1176,7 +1191,7 @@ sergensio_sig_cb(struct sergensio *sio, int err,
 
     sio_ref = swig_make_ref(sio, sergensio);
     args = PyTuple_New(3);
-    sergensio_ref(sio);
+    sergensio_pyref(sio);
     PyTuple_SET_ITEM(args, 0, sio_ref.val);
     if (err) {
 	o = OI_PI_FromString(gensio_err_to_str(err));
@@ -1220,7 +1235,7 @@ struct mdns_watch {
 
 static void gensio_mdns_free_done(struct gensio_mdns *mdns, void *userdata)
 {
-    struct mdns *m = userdata;
+    struct mdns *m = (struct mdns *) userdata;
     struct gensio_os_funcs *o = m->o;
     OI_PY_STATE gstate;
 
@@ -1231,22 +1246,22 @@ static void gensio_mdns_free_done(struct gensio_mdns *mdns, void *userdata)
     deref_swig_cb_val(m->done_val);
     OI_PY_STATE_PUT(gstate);
 
-    o->lock(m->lock);
+    gensio_os_funcs_lock(o, m->lock);
     if (m->free_on_close) {
-	o->unlock(m->lock);
-	o->free_lock(m->lock);
-	o->free(o, m);
+	gensio_os_funcs_unlock(o, m->lock);
+	gensio_os_funcs_free_lock(o, m->lock);
+	gensio_os_funcs_zfree(o, m);
 	check_os_funcs_free(o);
     } else {
 	m->mdns = NULL;
-	o->unlock(m->lock);
+	gensio_os_funcs_unlock(o, m->lock);
     }
 }
 
 static void gensio_mdns_remove_watch_done(struct gensio_mdns_watch *watch,
 					  void *userdata)
 {
-    struct mdns_watch *w = userdata;
+    struct mdns_watch *w = (struct mdns_watch *) userdata;
     struct gensio_os_funcs *o = w->o;
     OI_PY_STATE gstate;
 
@@ -1257,28 +1272,28 @@ static void gensio_mdns_remove_watch_done(struct gensio_mdns_watch *watch,
     deref_swig_cb_val(w->done_val);
     OI_PY_STATE_PUT(gstate);
 
-    o->lock(w->lock);
+    gensio_os_funcs_lock(o, w->lock);
     if (w->free_on_close) {
-	o->unlock(w->lock);
-	o->free_lock(w->lock);
+	gensio_os_funcs_unlock(o, w->lock);
+	gensio_os_funcs_free_lock(o, w->lock);
 	deref_swig_cb_val(w->cb_val);
-	o->free(o, w);
+	gensio_os_funcs_zfree(o, w);
 	check_os_funcs_free(o);
     } else {
 	w->watch = NULL;
-	o->unlock(w->lock);
+	gensio_os_funcs_unlock(o, w->lock);
     }
 }
 
 static void gensio_mdns_cb(struct gensio_mdns_watch *watch,
 			   enum gensio_mdns_data_state state,
-			   int interface, int ipdomain,
+			   int ipinterface, int ipdomain,
 			   const char *name, const char *type,
 			   const char *domain, const char *host,
 			   const struct gensio_addr *addr,
-			   const char *txt[], void *userdata)
+			   const char * const *txt, void *userdata)
 {
-    struct mdns_watch *w = userdata;
+    struct mdns_watch *w = (struct mdns_watch *) userdata;
     PyObject *args, *a;
     OI_PY_STATE gstate;
     char *s = NULL;
@@ -1294,7 +1309,7 @@ static void gensio_mdns_cb(struct gensio_mdns_watch *watch,
 
     args = PyTuple_New(9);
     PyTuple_SET_ITEM(args, 0, PyBool_FromLong(state == GENSIO_MDNS_NEW_DATA));
-    PyTuple_SET_ITEM(args, 1, PyInt_FromLong(interface));
+    PyTuple_SET_ITEM(args, 1, PyInt_FromLong(ipinterface));
     PyTuple_SET_ITEM(args, 2, PyInt_FromLong(ipdomain));
     PyTuple_SET_ITEM(args, 3, OI_PI_FromStringN(name));
     PyTuple_SET_ITEM(args, 4, OI_PI_FromStringN(type));
@@ -1303,7 +1318,7 @@ static void gensio_mdns_cb(struct gensio_mdns_watch *watch,
 
     rv = gensio_addr_to_str(addr, NULL, &len, 0);
     if (!rv) {
-	s = malloc(len + 1);
+	s = (char *) malloc(len + 1);
 	rv = gensio_addr_to_str(addr, s, &pos, len + 1);
     }
     if (rv)
@@ -1327,28 +1342,65 @@ static void gensio_mdns_cb(struct gensio_mdns_watch *watch,
     OI_PY_STATE_PUT(gstate);
 }
 
+/*
+ * This cannot return a sequence with None as the first value.  You
+ * can use add_python_seqresult if you need to do that.
+ */
 static PyObject *
 add_python_result(PyObject *result, PyObject *val)
 {
+    PyObject *seq, *o;
+
     if (result == Py_None) {
-	Py_XDECREF(result);
 	result = val;
-    } else {
-	PyObject *seq, *o2;
-
-	if (!PyTuple_Check(result)) {
-	    PyObject *tmpr = result;
-
-	    result = PyTuple_New(1);
-	    PyTuple_SetItem(result, 0, tmpr);
-	}
-	seq = PyTuple_New(1);
-	PyTuple_SetItem(seq, 0, val);
-	o2 = result;
-	result = PySequence_Concat(o2, seq);
-	Py_DECREF(o2);
-	Py_DECREF(seq);
+	Py_DECREF(Py_None);
+	return result;
     }
+
+    if (!PyTuple_Check(result)) {
+	PyObject *tmpr = result;
+
+	result = PyTuple_New(1);
+	PyTuple_SetItem(result, 0, tmpr);
+    }
+
+    seq = PyTuple_New(1);
+    PyTuple_SetItem(seq, 0, val);
+    o = result;
+    result = PySequence_Concat(o, seq);
+    Py_DECREF(o);
+    Py_DECREF(seq);
+    return result;
+}
+
+/*
+ * Always returns a sequence.  Otherwise it's not really very useful,
+ * just return it normally.
+ */
+static PyObject *
+add_python_seqresult(PyObject *result, PyObject *val)
+{
+    PyObject *seq, *o;
+
+    if (result == Py_None) {
+	result = PyTuple_New(1);
+	PyTuple_SetItem(result, 0, val);
+	return result;
+    }
+
+    if (!PyTuple_Check(result)) {
+	PyObject *tmpr = result;
+
+	result = PyTuple_New(1);
+	PyTuple_SetItem(result, 0, tmpr);
+    }
+
+    seq = PyTuple_New(1);
+    PyTuple_SetItem(seq, 0, val);
+    o = result;
+    result = PySequence_Concat(o, seq);
+    Py_DECREF(o);
+    Py_DECREF(seq);
     return result;
 }
 
@@ -1362,7 +1414,7 @@ static bool check_for_err(int err)
     return rv;
 };
 
-static void err_handle(char *name, int rv)
+static void err_handle(const char *name, int rv)
 {
     if (!rv)
 	return;
@@ -1370,7 +1422,7 @@ static void err_handle(char *name, int rv)
 		 gensio_err_to_str(rv));
 }
 
-static void ser_err_handle(char *name, int rv)
+static void ser_err_handle(const char *name, int rv)
 {
     if (!rv)
 	return;
@@ -1378,7 +1430,7 @@ static void ser_err_handle(char *name, int rv)
 		 gensio_err_to_str(rv));
 }
 
-static void cast_error(char *to, char *from)
+static void cast_error(const char *to, const char *from)
 {
     PyErr_Format(PyExc_RuntimeError, "Error casting from %s to %s", from, to);
 }
