@@ -28,17 +28,11 @@
 #include <string.h>
 #include <stdlib.h>
 #include <gensio/gensio.h>
-#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_mdns.h>
 #include <gensio/gensio_selector.h>
 #include <gensio/gensio_osops.h>
 #include <gensio/argvutils.h>
 #include "utils.h"
-#ifdef HAVE_SIGNALFD
-#include <unistd.h>
-#include <signal.h>
-#include <sys/signalfd.h>
-#endif
 
 static const char *progname;
 static int debug;
@@ -65,14 +59,14 @@ static void mdns_freed(struct gensio_mdns *m, void *userdata)
 {
     struct freed_data *f = userdata;
 
-    f->o->wake(f->closewaiter);
+    gensio_os_funcs_wake(f->o, f->closewaiter);
 }
 
 static void mdns_watch_freed(struct gensio_mdns_watch *w, void *userdata)
 {
     struct freed_data *f = userdata;
 
-    f->o->wake(f->closewaiter);
+    gensio_os_funcs_wake(f->o, f->closewaiter);
 }
 
 static const char *
@@ -99,17 +93,17 @@ mdns_info_found(struct gensio_mdns_watch *w,
 		const char *name, const char *type,
 		const char *domain, const char *host,
 		const struct gensio_addr *addr,
-		const char *txt[], void *userdata)
+		const char * const *txt, void *userdata)
 {
     char strbuf[250];
-    const char **s;
+    const char * const *s;
     int rv;
     struct freed_data *f = userdata;
 
     if (state == GENSIO_MDNS_ALL_FOR_NOW) {
 	printf("All-for-now:\n");
 	if (close_on_done)
-	    f->o->wake(f->closewaiter);
+	    gensio_os_funcs_wake(f->o, f->closewaiter);
 	return;
     }
 
@@ -128,27 +122,14 @@ mdns_info_found(struct gensio_mdns_watch *w,
     }
 }
 
-#ifdef HAVE_SIGNALFD
-static ssize_t dummy; /* Eliminate warnings. */
 static void
-sigfd_read(struct gensio_iod *iod, void *cb_data)
+term_handler(void *handler_data)
 {
-    struct freed_data *f = cb_data;
-    struct signalfd_siginfo i;
+    struct freed_data *f = handler_data;
 
-    dummy = iod->f->read(iod, &i, sizeof(i), NULL);
-    f->o->wake(f->closewaiter);
+    gensio_os_funcs_wake(f->o, f->closewaiter);
 }
 
-static void
-sigfd_cleared(struct gensio_iod *iod, void *cb_data)
-{
-    struct freed_data *f = cb_data;
-
-    f->o->wake(f->closewaiter);
-}
-#endif
-
 static void
 help(int err)
 {
@@ -188,12 +169,6 @@ main(int argc, char *argv[])
     struct gensio_mdns_service *service = NULL;
     struct freed_data fdata;
     int rv, arg, err;
-#ifdef HAVE_SIGNALFD
-    int  sigfd = -1;
-    struct gensio_iod *sig_iod = NULL;
-    sigset_t sigmask;
-    bool sigfd_set = false;
-#endif
     const char *name = NULL, *type = NULL, *domain = NULL, *host = NULL;
     int interface = -1, nettype = GENSIO_NETTYPE_UNSPEC, port = -1;
     const char *nettype_str = NULL;
@@ -201,6 +176,7 @@ main(int argc, char *argv[])
     const char **txt = NULL;
     gensiods txtargc = 0, txtargs = 0;
     bool do_service = false;
+    struct gensio_os_proc_data *proc_data;
 
     progname = argv[0];
 
@@ -208,9 +184,9 @@ main(int argc, char *argv[])
     if (rv) {
 	fprintf(stderr, "Could not allocate OS handler: %s\n",
 		gensio_err_to_str(rv));
-	goto out_err;
+	return 1;
     }
-    o->vlog = do_vlog;
+    gensio_os_funcs_set_vlog(o, do_vlog);
 
     for (arg = 1; arg < argc; arg++) {
 	if (argv[arg][0] != '-')
@@ -267,7 +243,15 @@ main(int argc, char *argv[])
 	    return 1;
     }
 
-    closewaiter = o->alloc_waiter(o);
+    rv = gensio_os_proc_setup(o, &proc_data);
+    if (rv) {
+	fprintf(stderr, "Could not setup process data: %s\n",
+		gensio_err_to_str(rv));
+	gensio_os_funcs_free(o);
+	return 1;
+    }
+
+    closewaiter = gensio_os_funcs_alloc_waiter(o);
     if (!closewaiter) {
 	rv = GE_NOMEM;
 	fprintf(stderr, "Could not allocate close waiter\n");
@@ -277,34 +261,12 @@ main(int argc, char *argv[])
     fdata.o = o;
     fdata.closewaiter = closewaiter;
 
-#ifdef HAVE_SIGNALFD
-    if (!close_on_done || do_service) {
-	sigemptyset(&sigmask);
-	sigaddset(&sigmask, SIGINT);
-	sigprocmask(SIG_BLOCK, &sigmask, NULL);
-	rv = signalfd(-1, &sigmask, 0);
-	if (rv < 0) {
-	    perror("signalfd");
-	    goto out_err;
-	}
-	sigfd = rv;
-	rv = o->add_iod(o, GENSIO_IOD_SIGNAL, sigfd, &sig_iod);
-	if (rv) {
-	    fprintf(stderr, "Can't add signalfd iod: %s\n",
-		    gensio_err_to_str(rv));
-	    goto out_err;
-	}
-	rv = o->set_fd_handlers(sig_iod, &fdata, sigfd_read, NULL, NULL,
-				sigfd_cleared);
-	if (rv) {
-	    fprintf(stderr, "Can't set sigfd handler: %s\n",
-		    gensio_err_to_str(rv));
-	    goto out_err;
-	}
-	sigfd_set = true;
-	o->set_read_handler(sig_iod, true);
+    rv = gensio_os_proc_register_term_handler(proc_data, term_handler, &fdata);
+    if (rv) {
+	fprintf(stderr, "Can't register term handler: %s\n",
+		gensio_err_to_str(rv));
+	goto out_err;
     }
-#endif
 
     rv = gensio_alloc_mdns(o, &mdns);
     if (rv) {
@@ -352,7 +314,7 @@ main(int argc, char *argv[])
 	}
     }
 
-    o->wait(closewaiter, 1, NULL);
+    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
 
  out_err:
     if (service) {
@@ -367,7 +329,7 @@ main(int argc, char *argv[])
 	    fprintf(stderr, "Could not free mdns watch: %s\n",
 		    gensio_err_to_str(rv));
 	else
-	    o->wait(closewaiter, 1, NULL);
+	    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
     }
 
     if (mdns) {
@@ -376,32 +338,22 @@ main(int argc, char *argv[])
 	    fprintf(stderr, "Could not free mdns handler: %s\n",
 		    gensio_err_to_str(rv));
 	else
-	    o->wait(closewaiter, 1, NULL);
+	    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
     }
 
-#ifdef HAVE_SIGNALFD
-    if (sigfd_set) {
-	o->clear_fd_handlers(sig_iod);
-	o->wait(closewaiter, 1, NULL);
-    }
-    if (sig_iod)
-	o->release_iod(sig_iod);
-    if (sigfd != -1)
-	close(sigfd);
-#endif
-
     if (txt)
 	gensio_argv_free(o, txt);
 
-    if (closewaiter)
-	o->free_waiter(closewaiter);
+    if (o && closewaiter)
+	gensio_os_funcs_free_waiter(o, closewaiter);
     if (o) {
 	gensio_time endwait = { 0, 0 };
 
-	while (o->service(o, &endwait) != GE_TIMEDOUT)
+	while (gensio_os_funcs_service(o, &endwait) != GE_TIMEDOUT)
 	    ;
 	gensio_cleanup_mem(o);
-	o->free_funcs(o);
+	gensio_os_funcs_free(o);
+	gensio_os_proc_cleanup(proc_data);
     }
     gensio_osfunc_exit(!!rv);
 }
