@@ -172,6 +172,7 @@ struct gtconn_info {
     struct gtinfo *g;
     struct gensio *user_io;
     struct gensio *io;
+    struct gensio *close_io;
     const char *ios;
     bool close_done;
 };
@@ -187,7 +188,7 @@ vreport_err(struct gtinfo *g, const char *fmt, va_list ap)
 #endif
     vfprintf(stderr, fmt, ap);
     fprintf(stderr, "\n");
-    fflush(stdout);
+    fflush(stderr);
 }
 
 static void
@@ -246,8 +247,8 @@ check_finish(struct ioinfo *ioinfo)
 	gensio_free(io);
     }
 
-    o->free(o, gtconn);
-    o->free(o, ogtconn);
+    gensio_os_funcs_zfree(o, gtconn);
+    gensio_os_funcs_zfree(o, ogtconn);
 
     free_ioinfo(ioinfo);
     free_ioinfo(oioinfo);
@@ -255,7 +256,7 @@ check_finish(struct ioinfo *ioinfo)
     free_ser_ioinfo(osubdata);
 
     if (!g->server_mode || (g->in_shutdown && gensio_list_empty(&g->io_list)))
-	g->o->wake(g->waiter);
+	gensio_os_funcs_wake(g->o, g->waiter);
 }
 
 static void
@@ -267,7 +268,7 @@ i_io_closed(struct gensio *io, void *close_data)
     if (ioinfo_io(ioinfo) == NULL)
 	gensio_free(io);
     gtconn->close_done = true;
-    gtconn->io = NULL;
+    gtconn->close_io = NULL;
     check_finish(ioinfo);
 }
 
@@ -278,9 +279,9 @@ io_closed(struct gensio *io, void *close_data)
     struct gtconn_info *gtconn = ioinfo_userdata(ioinfo);
     struct gtinfo *g = gtconn->g;
 
-    g->o->lock(g->lock);
+    gensio_os_funcs_lock(g->o, g->lock);
     i_io_closed(io, close_data);
-    g->o->unlock(g->lock);
+    gensio_os_funcs_unlock(g->o, g->lock);
 }
 
 static void
@@ -294,10 +295,11 @@ i_gshutdown(struct ioinfo *ioinfo, bool user_req)
 
     if (gtconn->io) {
 	ioinfo_set_not_ready(ioinfo);
-	err = gensio_close(gtconn->io, io_closed, NULL);
-	if (err)
-	    i_io_closed(gtconn->io, NULL);
+	gtconn->close_io = gtconn->io;
 	gtconn->io = NULL;
+	err = gensio_close(gtconn->close_io, io_closed, NULL);
+	if (err)
+	    i_io_closed(gtconn->close_io, NULL);
     }
     if (ogtconn->io) {
 	/*
@@ -305,10 +307,11 @@ i_gshutdown(struct ioinfo *ioinfo, bool user_req)
 	 * for oomtest to work it has to get that failure.  So let it
 	 * report the error if it happens.
 	 */
-	err = gensio_close(ogtconn->io, io_closed, NULL);
-	if (err)
-	    i_io_closed(ogtconn->io, NULL);
+	ogtconn->close_io = ogtconn->io;
 	ogtconn->io = NULL;
+	err = gensio_close(ogtconn->close_io, io_closed, NULL);
+	if (err)
+	    i_io_closed(ogtconn->close_io, NULL);
     }
     if (!g->err && !user_req)
 	g->err = GE_IOERR;
@@ -320,9 +323,9 @@ gshutdown(struct ioinfo *ioinfo, bool user_req)
     struct gtconn_info *gtconn = ioinfo_userdata(ioinfo);
     struct gtinfo *g = gtconn->g;
 
-    g->o->lock(g->lock);
+    gensio_os_funcs_lock(g->o, g->lock);
     i_gshutdown(ioinfo, user_req);
-    g->o->unlock(g->lock);
+    gensio_os_funcs_unlock(g->o, g->lock);
 }
 
 static void
@@ -493,12 +496,12 @@ add_io(struct gtinfo *g, struct gensio *io, bool open_finished)
     struct ioinfo *ioinfo1 = NULL, *ioinfo2 = NULL;
     struct gtconn_info *gtconn1 = NULL, *gtconn2 = NULL;
 
-    gtconn1 = o->zalloc(o, sizeof(*gtconn1));
+    gtconn1 = gensio_os_funcs_zalloc(o, sizeof(*gtconn1));
     if (!gtconn1) {
 	report_err(g, "Could not allocate gtconn 1");
 	goto out_err;
     }
-    gtconn2 = o->zalloc(o, sizeof(*gtconn2));
+    gtconn2 = gensio_os_funcs_zalloc(o, sizeof(*gtconn2));
     if (!gtconn2) {
 	report_err(g, "Could not allocate gtconn 2");
 	goto out_err;
@@ -566,15 +569,15 @@ add_io(struct gtinfo *g, struct gensio *io, bool open_finished)
     if (subdata2)
 	free_ser_ioinfo(subdata2);
     if (gtconn1)
-	o->free(o, gtconn1);
+	gensio_os_funcs_zfree(o, gtconn1);
     if (gtconn2)
-	o->free(o, gtconn2);
+	gensio_os_funcs_zfree(o, gtconn2);
     if (ioinfo1)
 	free_ioinfo(ioinfo1);
     if (ioinfo2)
 	free_ioinfo(ioinfo2);
     if (!g->server_mode)
-	g->o->wake(g->waiter);
+	gensio_os_funcs_wake(g->o, g->waiter);
     return err;
 }
 
@@ -597,14 +600,15 @@ i_handle_term(struct gtinfo *g)
 
 	closed_one = true;
 	if (gtconn->io) {
-	    err = gensio_close(gtconn->io, io_closed, NULL);
-	    if (err)
-		i_io_closed(gtconn->io, NULL);
+	    gtconn->close_io = gtconn->io;
 	    gtconn->io = NULL;
+	    err = gensio_close(gtconn->close_io, io_closed, NULL);
+	    if (err)
+		i_io_closed(gtconn->close_io, NULL);
 	}
     }
     if (!closed_one)
-	g->o->wake(g->waiter);
+	gensio_os_funcs_wake(g->o, g->waiter);
 }
 
 static void
@@ -612,9 +616,9 @@ handle_term(void *info)
 {
     struct gtinfo *g = info;
 
-    g->o->lock(g->lock);
+    gensio_os_funcs_lock(g->o, g->lock);
     i_handle_term(g);
-    g->o->unlock(g->lock);
+    gensio_os_funcs_unlock(g->o, g->lock);
 }
 
 static int
@@ -630,7 +634,7 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
 	vreport_err(g, li->str, li->args);
 
 	g->err = 1;
-	g->o->wake(g->waiter);
+	gensio_os_funcs_wake(g->o, g->waiter);
 	return 0;
     }
 
@@ -640,7 +644,7 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
 	struct ioinfo *oioinfo;
 	struct gtconn_info *ogtconn;
 
-	g->o->lock(g->lock);
+	gensio_os_funcs_lock(g->o, g->lock);
 	if (g->in_shutdown) {
 	    gensio_free(io);
 	} else if (g->server_mode || gensio_list_empty(&g->io_list)) {
@@ -666,14 +670,14 @@ io_acc_event(struct gensio_accepter *accepter, void *user_data,
 		goto out_unlock;
 	    }
 	} else {
-	    gensio_free(data);
+	    gensio_free(io);
 	}
 	if (!g->server_mode && g->acc) {
 	    gensio_acc_free(g->acc);
 	    g->acc = NULL;
 	}
     out_unlock:
-	g->o->unlock(g->lock);
+	gensio_os_funcs_unlock(g->o, g->lock);
 
 	return 0;
     }
@@ -718,12 +722,13 @@ help(int err)
     printf("  -e, --escchar - Set the local terminal escape character.\n"
 	   "    Set to -1 to disable the escape character\n"
 	   "    Default is ^\\ for tty stdin and disabled for non-tty stdin\n");
+    printf("  --version - Print the version number and exit.\n");
     printf("  -h, --help - This help\n");
     gensio_osfunc_exit(err);
 }
 
 static void
-do_vlog(struct gensio_os_funcs *f, enum gensio_log_levels level,
+do_vlog(struct gensio_os_funcs *o, enum gensio_log_levels level,
 	const char *log, va_list args)
 {
     char buf[200];
@@ -732,7 +737,7 @@ do_vlog(struct gensio_os_funcs *f, enum gensio_log_levels level,
 	return;
     vsnprintf(buf, sizeof(buf), log, args);
 
-    report_err(f->other_data, "gensio %s log: %s",
+    report_err(gensio_os_funcs_get_data(o), "gensio %s log: %s",
 	       gensio_log_level_to_str(level), buf);
 }
 
@@ -748,7 +753,7 @@ gensio_loop(void *info)
 {
     struct gensio_loop_info *li = info;
 
-    li->o->wait(li->loopwaiter, 1, NULL);
+    gensio_os_funcs_wait(li->o, li->loopwaiter, 1, NULL);
 }
 
 #ifndef _WIN32
@@ -846,6 +851,9 @@ main(int argc, char *argv[])
 	    debug++;
 	    if (debug > 1)
 		gensio_set_log_mask(GENSIO_LOG_MASK_ALL);
+	} else if ((rv = cmparg(argc, argv, &arg, NULL, "--version", NULL))) {
+	    printf("Version %s\n", gensio_version_string);
+	    exit(0);
 	} else if ((rv = cmparg(argc, argv, &arg, "-h", "--help", NULL)))
 	    help(0);
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--dummyrand",
@@ -896,14 +904,14 @@ main(int argc, char *argv[])
 
     if (use_glib) {
 #ifndef HAVE_GLIB
-	fprintf(stderr, "glib specified, but glib OS handler not avaiable.\n");
+	fprintf(stderr, "glib specified, but glib OS handler not available.\n");
 	exit(1);
 #else
 	rv = gensio_glib_funcs_alloc(&g.o);
 #endif
     } else if (use_tcl) {
 #ifndef HAVE_TCL
-	fprintf(stderr, "tcl specified, but tcl OS handler not avaiable.\n");
+	fprintf(stderr, "tcl specified, but tcl OS handler not available.\n");
 	exit(1);
 #else
 	if (num_extra_threads > 0)
@@ -920,17 +928,17 @@ main(int argc, char *argv[])
 		gensio_err_to_str(rv));
 	goto out_err;
     }
-    g.o->other_data = &g;
-    g.o->vlog = do_vlog;
+    gensio_os_funcs_set_data(g.o, &g);
+    gensio_os_funcs_set_vlog(g.o, do_vlog);
 
-    g.waiter = g.o->alloc_waiter(g.o);
+    g.waiter = gensio_os_funcs_alloc_waiter(g.o);
     if (!g.waiter) {
 	rv = GE_NOMEM;
 	fprintf(stderr, "Could not allocate OS waiter\n");
 	goto out_err;
     }
 
-    g.lock = g.o->alloc_lock(g.o);
+    g.lock = gensio_os_funcs_alloc_lock(g.o);
     if (!g.lock) {
 	rv = GE_NOMEM;
 	fprintf(stderr, "Could not allocate OS lock\n");
@@ -946,14 +954,15 @@ main(int argc, char *argv[])
 
     rv = GE_NOMEM;
     if (num_extra_threads > 0) {
-	loopinfo = g.o->zalloc(g.o, sizeof(*loopinfo) * num_extra_threads);
+	loopinfo = gensio_os_funcs_zalloc(g.o,
+				      sizeof(*loopinfo) * num_extra_threads);
 	if (!loopinfo)
 	    goto out_err;
     }
 
     for (i = 0; i < num_extra_threads; i++) {
 	loopinfo[i].o = g.o;
-	loopinfo[i].loopwaiter = g.o->alloc_waiter(g.o);
+	loopinfo[i].loopwaiter = gensio_os_funcs_alloc_waiter(g.o);
 	if (!loopinfo[i].loopwaiter) {
 	    fprintf(stderr, "Could not allocate loop waiter\n");
 	    goto out_err;
@@ -989,12 +998,12 @@ main(int argc, char *argv[])
 	if (rv)
 	    goto out_err;
     } else {
-	g.o->lock(g.lock);
+	gensio_os_funcs_lock(g.o, g.lock);
 	rv = add_io(&g, io, false);
 	if (rv) {
 	    gensio_free(io);
 	    io = NULL;
-	    g.o->unlock(g.lock);
+	    gensio_os_funcs_unlock(g.o, g.lock);
 	    goto out_err;
 	}
 	rv = gensio_open(io, io_open_paddr, &g);
@@ -1005,7 +1014,7 @@ main(int argc, char *argv[])
 		    gensio_err_to_str(rv));
 	    i_gshutdown(ioinfo, false);
 	}
-	g.o->unlock(g.lock);
+	gensio_os_funcs_unlock(g.o, g.lock);
 	io = NULL;
     }
 
@@ -1013,33 +1022,33 @@ main(int argc, char *argv[])
     if (rv)
 	handle_term(&g);
 
-    g.o->wait(g.waiter, 1, NULL);
+    gensio_os_funcs_wait(g.o, g.waiter, 1, NULL);
 
  out_err:
     if (io)
 	gensio_free(io);
     if (g.lock)
-	g.o->lock(g.lock);
+	gensio_os_funcs_lock(g.o, g.lock);
     if (g.acc) {
 	gensio_acc_free(g.acc);
 	g.acc = NULL;
     }
     if (g.lock)
-	g.o->unlock(g.lock);
+	gensio_os_funcs_unlock(g.o, g.lock);
 
     if (!rv && g.err)
 	rv = g.err;
 
     for (i = 0; loopinfo && i < num_extra_threads; i++) {
 	if (loopinfo[i].loopth) {
-	    g.o->wake(loopinfo[i].loopwaiter);
+	    gensio_os_funcs_wake(g.o, loopinfo[i].loopwaiter);
 	    gensio_os_wait_thread(loopinfo[i].loopth);
 	}
 	if (loopinfo[i].loopwaiter)
-	    g.o->free_waiter(loopinfo[i].loopwaiter);
+	    gensio_os_funcs_free_waiter(g.o, loopinfo[i].loopwaiter);
     }
     if (loopinfo)
-	g.o->free(g.o, loopinfo);
+	gensio_os_funcs_zfree(g.o, loopinfo);
 
     /*
      * We wait until there are no gensios left pending.  You can get
@@ -1050,7 +1059,7 @@ main(int argc, char *argv[])
      */
     if (gensio_num_alloced() == 0)
 	endwait.secs = 0; /* Just run events until we are out. */
-    while (g.o && g.o->service(g.o, &endwait) != GE_TIMEDOUT) {
+    while (g.o && gensio_os_funcs_service(g.o, &endwait) != GE_TIMEDOUT) {
 	if (gensio_num_alloced() == 0) {
 	    /* Waiting for no gensios left, then run events til we are out. */
 	    endwait.secs = 0;
@@ -1058,14 +1067,14 @@ main(int argc, char *argv[])
 	}
     }
     if (g.waiter)
-	g.o->free_waiter(g.waiter);
+	gensio_os_funcs_free_waiter(g.o, g.waiter);
     if (g.lock)
-	g.o->free_lock(g.lock);
+	gensio_os_funcs_free_lock(g.o, g.lock);
     if (proc_data)
 	gensio_os_proc_cleanup(proc_data);
     if (g.o) {
 	gensio_cleanup_mem(g.o);
-	g.o->free_funcs(g.o);
+	gensio_os_funcs_free(g.o);
     }
 
 #ifndef _WIN32
