@@ -93,6 +93,7 @@ help(const char *progname)
     P("        either rsa or ec.  The default is rsa\n");
     P("  --force, -f - Don't ask questions, just do the operation.  This\n");
     P("        may overwrite data without asking.\n");
+    P("  --version - Print the version number and exit.\n");
     P("\n");
     P("Commands are:\n");
     P("  setup\n");
@@ -136,7 +137,7 @@ help(const char *progname)
     P("    Put the local certificate for the given host onto the remote\n");
     P("    host so it can be used for login.  It uses old credentials\n");
     P("    (credentials with .1 appended to the name, per keygen) if\n");
-    P("    they are there.  This is useful if you have upated your\n");
+    P("    they are there.  This is useful if you have updated your\n");
     P("    certificate and need to send a new one to some remote hosts.\n");
     P("    It finds the certificate name as described in the keygen\n");
     P("    command.  If old credentials exist, it will use those to\n");
@@ -465,7 +466,7 @@ hash_file(const char *dir, const char *name, void *cbdata)
 	fprintf(stderr, "Out of memory allocating %s%c%s", dir, DIRSEP, name);
 	return ITERATE_DIR_ERR;
     }
-    
+
     cert = load_cert(s);
     if (!cert)
 	goto out_err;
@@ -1176,6 +1177,10 @@ main(int argc, char **argv)
 	    force = true;
 	    continue;
 	}
+	if (cmparg(argc, argv, &i, NULL, "--version", NULL)) {
+	    printf("Version %s\n", gensio_version_string);
+	    exit(0);
+	}
 	if (cmparg(argc, argv, &i, "-h", "--help", NULL)) {
 	    help(argv[0]);
 	    continue;
