@@ -60,12 +60,14 @@ struct gdata {
     bool got_oob;
 };
 
+struct gensio_os_proc_data *proc_data;
+
 static void
 gshutdown(struct ioinfo *ioinfo, bool user_req)
 {
     struct gdata *ginfo = ioinfo_userdata(ioinfo);
 
-    ginfo->o->wake(ginfo->waiter);
+    gensio_os_funcs_wake(ginfo->o, ginfo->waiter);
 }
 
 static void
@@ -100,7 +102,9 @@ static int gevent(struct ioinfo *ioinfo, struct gensio *io, int event,
     return 0;
 }
 
-static void winch_ready(struct gensio_iod *iod, void *cb_data);
+static void winch_ready(int x_chrs, int y_chrs,
+			int x_bits, int y_bits,
+			void *handler_data);
 
 /*
  * We wait until the other end has signalled us to say that it is
@@ -111,13 +115,37 @@ static void
 goobdata(struct ioinfo *ioinfo, unsigned char *buf, gensiods *buflen)
 {
     struct gdata *ginfo = ioinfo_userdata(ioinfo);
+    struct ioinfo *oioinfo = ioinfo_otherioinfo(ioinfo);
+    struct gdata *oginfo = ioinfo_userdata(oioinfo);
 
     if (ginfo->got_oob)
 	return;
 
     ginfo->got_oob = true;
-    if (ginfo->interactive)
-	winch_ready(NULL, ioinfo);
+
+    /*
+     * Wait for the other end of the connection to be fully up before
+     * we send the window size across to avoid screwing up login
+     * handling.
+     */
+    if (oginfo->interactive) {
+	struct gensio *io = ioinfo_io(oioinfo);
+	struct gensio_iod *iod;
+	gensiods len = sizeof(iod);
+	int err;
+
+	/*
+	 * Which iod is passed in the data as an index, but the iod is
+	 * returned in the same data.  It looks a little strange to do
+	 * this, but that's how it works.
+	 */
+	memcpy(&iod, "0", 2);
+	err = gensio_control(io, GENSIO_CONTROL_DEPTH_FIRST, true,
+			     GENSIO_CONTROL_IOD, (char *) &iod, &len);
+	if (!err)
+	    gensio_os_proc_register_winsize_handler(proc_data, iod,
+						    winch_ready, ioinfo);
+    }
 }
 
 static struct ioinfo_user_handlers guh = {
@@ -230,14 +258,14 @@ increase_size(struct gensio_os_funcs *o, char **rval, gensiods *size)
     gensiods newsize = *size + 10;
     char *val;
 
-    val = o->zalloc(o, newsize);
+    val = gensio_os_funcs_zalloc(o, newsize);
     if (!val) {
 	fprintf(stderr, "Out of memory allocating 2fa data\n");
 	return GE_NOMEM;
     }
     if (*rval) {
 	memcpy(val, *rval, *size);
-	o->free(o, *rval);
+	gensio_os_funcs_zfree(o, *rval);
     }
     *rval = val;
     *size = newsize;
@@ -287,7 +315,7 @@ get2fa(struct gdata *ginfo, char **rval, gensiods *len, const char *prompt)
 
  out_err:
     if (val)
-	o->free(o, val);
+	gensio_os_funcs_zfree(o, val);
     gensio_clear_sync(tty);
     gensio_close_s(tty);
     gensio_free(tty);
@@ -301,7 +329,7 @@ io_close(struct gensio *io, void *close_data)
     struct gdata *ginfo = ioinfo_userdata(ioinfo);
     struct gensio_waiter *closewaiter = close_data;
 
-    ginfo->o->wake(closewaiter);
+    gensio_os_funcs_wake(ginfo->o, closewaiter);
 }
 
 static const char *progname;
@@ -363,6 +391,7 @@ help(int err)
 	   "    done from the local machine\n");
     printf("  -4 - Do IPv4 only.\n");
     printf("  -6 - Do IPv6 only.\n");
+    printf("  --version - Print the version number and exit.\n");
     printf("  -h, --help - This help\n");
     exit(err);
 }
@@ -724,7 +753,7 @@ lookup_certinfo(struct gensio_os_funcs *o,
     free(tmp);
     if (!*rcertspec) {
     out_err1:
-	o->free(o, *rCAspec);
+	gensio_os_funcs_zfree(o, *rCAspec);
 	*rCAspec = NULL;
 	goto out_err;
     }
@@ -735,9 +764,9 @@ lookup_certinfo(struct gensio_os_funcs *o,
     free(tmp);
     if (!*rkeyspec) {
     out_err2:
-	o->free(o, *rcertspec);
+	gensio_os_funcs_zfree(o, *rcertspec);
 	*rcertspec = NULL;
-	o->free(o, *rCAspec);
+	gensio_os_funcs_zfree(o, *rCAspec);
 	*rCAspec = NULL;
 	goto out_err;
     }
@@ -1195,38 +1224,25 @@ auth_event(struct gensio *io, void *user_data, int event, int ierr,
     }
 }
 
-#if HAVE_DECL_SIGWINCH
-#include <sys/types.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <sys/ioctl.h>
-
 static void winch_sent(void *cb_data);
 
-static int winch_pipe[2];
-static struct gensio_iod *winch_iod;
 static unsigned char winch_buf[11];
 static struct ioinfo_oob winch_oob = { .buf = winch_buf };
 static bool winch_oob_sending;
 static bool winch_oob_pending;
+static int winch_x_chrs, winch_y_chrs, winch_x_bits, winch_y_bits;
 
 static void
-send_winch(struct ioinfo *ioinfo)
+send_winch(struct ioinfo *ioinfo,
+	   int x_chrs, int y_chrs, int x_bits, int y_bits)
 {
-    struct winsize win;
-    int rv;
-
-    rv = ioctl(0, TIOCGWINSZ, &win);
-    if (rv == -1)
-	return;
-
     /* See goobdata() comment for message format. */
     winch_oob.buf[0] = 'w';
     gensio_u16_to_buf(winch_oob.buf + 1, 8); /* Number of following bytes. */
-    gensio_u16_to_buf(winch_oob.buf + 3, win.ws_row);
-    gensio_u16_to_buf(winch_oob.buf + 5, win.ws_col);
-    gensio_u16_to_buf(winch_oob.buf + 7, win.ws_xpixel);
-    gensio_u16_to_buf(winch_oob.buf + 9, win.ws_ypixel);
+    gensio_u16_to_buf(winch_oob.buf + 3, y_chrs);
+    gensio_u16_to_buf(winch_oob.buf + 5, x_chrs);
+    gensio_u16_to_buf(winch_oob.buf + 7, x_bits);
+    gensio_u16_to_buf(winch_oob.buf + 9, y_bits);
     winch_oob.len = 11;
     winch_oob.cb_data = ioinfo;
     winch_oob.send_done = winch_sent;
@@ -1242,115 +1258,28 @@ winch_sent(void *cb_data)
     winch_oob_sending = false;
     if (winch_oob_pending) {
 	winch_oob_pending = false;
-	send_winch(ioinfo);
+	send_winch(ioinfo,
+		   winch_x_chrs, winch_y_chrs, winch_x_bits, winch_y_bits);
     }
 }
 
 static void
-winch_ready(struct gensio_iod *iod, void *cb_data)
+winch_ready(int x_chrs, int y_chrs, int x_bits, int y_bits,
+	    void *handler_data)
 {
-    struct ioinfo *ioinfo = cb_data;
-    char dummy;
-    int rv = 1;
+    struct ioinfo *ioinfo = handler_data;
 
-    /* Clear out the pipe. */
-    while (rv == 1)
-	rv = read(winch_pipe[0], &dummy, 1);
-    /* errno should be EAGAIN here. */
-
-    if (!isatty(0))
-	return;
-
-    if (winch_oob_sending)
+    if (winch_oob_sending) {
+	winch_x_chrs = x_chrs;
+	winch_y_chrs = y_chrs;
+	winch_x_bits = x_bits;
+	winch_y_bits = y_bits;
 	winch_oob_pending = true;
-    else
-	send_winch(ioinfo);
-}
-
-static void
-handle_sigwinch(int signum)
-{
-    int rv;
-
-    rv = write(winch_pipe[1], "w", 1);
-    if (rv != 1) {
-	/* What can be done here? */
+    } else {
+	send_winch(ioinfo, x_chrs, y_chrs, x_bits, y_bits);
     }
 }
 
-static int
-setup_window_change(void)
-{
-    struct sigaction sigact;
-
-    if (pipe(winch_pipe) == -1) {
-	perror("Unable to allocate SIGWINCH pipe");
-	return 1;
-    }
-    if (fcntl(winch_pipe[0], F_SETFL, O_NONBLOCK) == -1) {
-	perror("Unable to set nonblock on SIGWINCH pipe[0]");
-	return 1;
-    }
-    if (fcntl(winch_pipe[1], F_SETFL, O_NONBLOCK) == -1) {
-	perror("Unable to set nonblock on SIGWINCH pipe[1]");
-	return 1;
-    }
-
-    memset(&sigact, 0, sizeof(sigact));
-    sigact.sa_handler = handle_sigwinch;
-    if (sigaction(SIGWINCH, &sigact, NULL)) {
-	perror("Unable to setup SIGWINCH");
-	return 1;
-    }
-
-    return 0;
-}
-
-static int
-setup_window_change2(struct gensio_os_funcs *o, struct ioinfo *ioinfo)
-{
-    int rv;
-
-    rv = o->add_iod(o, GENSIO_IOD_PIPE, winch_pipe[0], &winch_iod);
-    if (rv) {
-	fprintf(stderr, "Could not add SIGWINCH fd iod: %s\n",
-		gensio_err_to_str(rv));
-	return 1;
-    }
-
-    rv = o->set_fd_handlers(winch_iod, ioinfo, winch_ready,
-			    NULL, NULL, NULL);
-    if (rv) {
-	fprintf(stderr, "Could not set SIGWINCH fd handler: %s\n",
-		gensio_err_to_str(rv));
-	return 1;
-    }
-    o->set_read_handler(winch_iod, true);
-
-    return 0;
-}
-
-#else
-
-static void
-winch_ready(struct gensio_iod *iod, void *cb_data)
-{
-}
-
-static int
-setup_window_change(void)
-{
-    return 0;
-}
-
-static int
-setup_window_change2(struct gensio_os_funcs *o, struct ioinfo *ioinfo)
-{
-    return 0;
-}
-
-#endif /* HAVE_DECL_SIGWINCH */
-
 static void
 pr_localport(const char *fmt, va_list ap)
 {
@@ -1603,7 +1532,7 @@ mdns_cb(struct gensio_mdns_watch *w,
 	int iface, int ipdomain,
 	const char *name, const char *type,
 	const char *domain, const char *host,
-	const struct gensio_addr *addr, const char *txt[],
+	const struct gensio_addr *addr, const char * const *txt,
 	void *userdata)
 {
     struct mdns_cb_data *cb_data = userdata;
@@ -1643,10 +1572,10 @@ mdns_cb(struct gensio_mdns_watch *w,
     cb_data->err = gensio_addr_to_str(addr, NULL, &addrstrlen, 0);
     if (cb_data->err)
 	goto out_wake;
-    addrstr = o->zalloc(o, addrstrlen + 1);
+    addrstr = gensio_os_funcs_zalloc(o, addrstrlen + 1);
     cb_data->err = gensio_addr_to_str(addr, addrstr, &pos, addrstrlen + 1);
     if (cb_data->err) {
-	o->free(o, addrstr);
+	gensio_os_funcs_zfree(o, addrstr);
 	goto out_wake;
     }
 
@@ -1667,9 +1596,9 @@ mdns_cb(struct gensio_mdns_watch *w,
 
  out_wake:
     if (addrstr)
-	o->free(o, addrstr);
+	gensio_os_funcs_zfree(o, addrstr);
     cb_data->done = true;
-    o->wake(cb_data->wait);
+    gensio_os_funcs_wake(o, cb_data->wait);
 }
 
 static void
@@ -1677,7 +1606,7 @@ mdns_freed(struct gensio_mdns *m, void *userdata)
 {
     struct mdns_cb_data *cb_data = userdata;
 
-    cb_data->o->wake(cb_data->wait);
+    gensio_os_funcs_wake(cb_data->o, cb_data->wait);
 }
 
 static int
@@ -1695,7 +1624,7 @@ lookup_mdns_transport(struct gensio_os_funcs *o, const char *name,
     memset(&cb_data, 0, sizeof(cb_data));
     cb_data.err = GE_NOTFOUND;
     cb_data.o = o;
-    cb_data.wait = o->alloc_waiter(o);
+    cb_data.wait = gensio_os_funcs_alloc_waiter(o);
     if (!cb_data.wait) {
 	fprintf(stderr, "Unable to allocate wait: out of memory\n");
 	return 1;
@@ -1708,7 +1637,7 @@ lookup_mdns_transport(struct gensio_os_funcs *o, const char *name,
 
     err = gensio_alloc_mdns(o, &mdns);
     if (err) {
-	o->free_waiter(cb_data.wait);
+	gensio_os_funcs_free_waiter(o, cb_data.wait);
 	fprintf(stderr, "Unable to allocate mdns data: %s\n",
 		gensio_err_to_str(err));
 	return 1;
@@ -1717,19 +1646,19 @@ lookup_mdns_transport(struct gensio_os_funcs *o, const char *name,
     err = gensio_mdns_add_watch(mdns, -1, nettype, name, type, NULL, NULL,
 				mdns_cb, &cb_data, &mdns_watch);
     if (err) {
-	o->free_waiter(cb_data.wait);
+	gensio_os_funcs_free_waiter(o, cb_data.wait);
 	gensio_free_mdns(mdns, NULL, NULL);
 	fprintf(stderr, "Unable to add mdns watch: %s\n",
 		gensio_err_to_str(err));
 	return 1;
     }
 
-    o->wait(cb_data.wait, 1, &timeout);
+    gensio_os_funcs_wait(o, cb_data.wait, 1, &timeout);
 
     gensio_mdns_remove_watch(mdns_watch, NULL, NULL);
     gensio_free_mdns(mdns, mdns_freed, &cb_data);
 
-    o->wait(cb_data.wait, 1, NULL);
+    gensio_os_funcs_wait(o, cb_data.wait, 1, NULL);
 
     if (cb_data.err) {
 	fprintf(stderr, "Error looking up %s with mdns: %s\n",
@@ -1779,14 +1708,13 @@ main(int argc, char *argv[])
     const char *addr, *cstr;
     const char *iptype = ""; /* Try both IPv4 and IPv6 by default. */
     const char *mdns_type = NULL;
-    struct gensio_os_proc_data *proc_data;
     const char *val_2fa = "", *pfx_2fa = "";
 
     localport_err = pr_localport;
 
     memset(&userdata1, 0, sizeof(userdata1));
     memset(&userdata2, 0, sizeof(userdata2));
-    userdata2.interactive = true;
+    userdata1.interactive = true;
 
     err = gensio_default_os_hnd(0, &o);
     if (err) {
@@ -1794,19 +1722,16 @@ main(int argc, char *argv[])
 		gensio_err_to_str(err));
 	return 1;
     }
-    o->vlog = do_vlog;
+    gensio_os_funcs_set_vlog(o, do_vlog);
 
     err = gensio_os_proc_setup(o, &proc_data);
     if (err) {
 	fprintf(stderr, "Could not setup process data: %s\n",
 		gensio_err_to_str(err));
-	o->free_funcs(o);
+	gensio_os_funcs_free(o);
 	return 1;
     }
 
-    if (setup_window_change())
-	return 1;
-
     progname = argv[0];
 
     if (can_do_raw()) {
@@ -1814,7 +1739,7 @@ main(int argc, char *argv[])
 	userdata1.ios = io1_default_tty;
     } else {
 	userdata1.ios = io1_default_notty;
-	userdata2.interactive = false;
+	userdata1.interactive = false;
     }
 
     for (arg = 1; arg < argc; arg++) {
@@ -1922,6 +1847,9 @@ main(int argc, char *argv[])
 	    debug++;
 	    if (debug > 1)
 		gensio_set_log_mask(GENSIO_LOG_MASK_ALL);
+	} else if ((rv = cmparg(argc, argv, &arg, NULL, "--version", NULL))) {
+	    printf("Version %s\n", gensio_version_string);
+	    exit(0);
 	} else if ((err = cmparg(argc, argv, &arg, "-h", "--help", NULL))) {
 	    help(0);
 	} else {
@@ -2002,12 +1930,16 @@ main(int argc, char *argv[])
 	service[len++] = '\0';
 
 	userdata1.ios = io1_default_notty;
-	userdata2.interactive = false;
+	userdata1.interactive = false;
 	service_len = len;
     } else {
 	char *termvar = getenv("TERM");
 	unsigned int len = 0;
 
+#ifdef _WIN32
+	if (!termvar)
+	    termvar = "ansi";
+#endif
 	if (termvar) {
 	    len = 6 + 5 + strlen(termvar) + 2;
 	    service = malloc(len);
@@ -2043,14 +1975,14 @@ main(int argc, char *argv[])
     userdata1.o = o;
     userdata2.o = o;
 
-    userdata1.waiter = o->alloc_waiter(o);
+    userdata1.waiter = gensio_os_funcs_alloc_waiter(o);
     if (!userdata1.waiter) {
 	fprintf(stderr, "Could not allocate OS waiter\n");
 	return 1;
     }
     userdata2.waiter = userdata1.waiter;
 
-    closewaiter = o->alloc_waiter(o);
+    closewaiter = gensio_os_funcs_alloc_waiter(o);
     if (!closewaiter) {
 	fprintf(stderr, "Could not allocate close waiter\n");
 	return 1;
@@ -2136,6 +2068,10 @@ main(int argc, char *argv[])
     aux_data.flags = htonl(aux_data.flags);
 
     if (use_mux) {
+	len = 1;
+	gensio_control(userdata2.io,
+		       0, false, GENSIO_CONTROL_ENABLE_OOB, "1", &len);
+
 	len = 4;
 	err = gensio_control(userdata2.io, 1 + use_telnet, GENSIO_CONTROL_SET,
 			     GENSIO_CONTROL_SERVICE, "mux", &len);
@@ -2173,7 +2109,7 @@ main(int argc, char *argv[])
 	return 1;
     }
 
-    if (userdata2.interactive) {
+    if (userdata1.interactive) {
 	err = gensio_control(userdata2.io, GENSIO_CONTROL_DEPTH_ALL,
 			     GENSIO_CONTROL_SET,
 			     GENSIO_CONTROL_NODELAY, "1", NULL);
@@ -2202,7 +2138,7 @@ main(int argc, char *argv[])
     }
 
     if (mdns_transport)
-	o->free(o, (char *) transport);
+	gensio_os_funcs_zfree(o, (char *) transport);
 
     userdata1.can_close = true;
     err = gensio_open_s(userdata1.io);
@@ -2217,13 +2153,10 @@ main(int argc, char *argv[])
     ioinfo_set_ready(ioinfo1, userdata1.io);
     ioinfo_set_ready(ioinfo2, userdata2.io);
 
-    if (setup_window_change2(o, ioinfo2))
-	return 1;
-
     start_local_ports(userdata2.io);
     start_remote_ports(ioinfo2);
 
-    o->wait(userdata1.waiter, 1, NULL);
+    gensio_os_funcs_wait(o, userdata1.waiter, 1, NULL);
 
  closeit:
     free(service);
@@ -2247,22 +2180,22 @@ main(int argc, char *argv[])
     }
 
     if (closecount > 0)
-	o->wait(closewaiter, closecount, NULL);
+	gensio_os_funcs_wait(o, closewaiter, closecount, NULL);
 
     if (CAspec)
-	o->free(o, CAspec);
+	gensio_os_funcs_zfree(o, CAspec);
     if (certspec)
-	o->free(o, certspec);
+	gensio_os_funcs_zfree(o, certspec);
     if (keyspec)
-	o->free(o, keyspec);
+	gensio_os_funcs_zfree(o, keyspec);
 
     gensio_free(userdata1.io);
     gensio_free(userdata2.io);
 
     if (userdata2.ios)
 	free(userdata2.ios);
-    o->free_waiter(closewaiter);
-    o->free_waiter(userdata1.waiter);
+    gensio_os_funcs_free_waiter(o, closewaiter);
+    gensio_os_funcs_free_waiter(o, userdata1.waiter);
 
     free_ioinfo(ioinfo1);
     free_ioinfo(ioinfo2);
@@ -2270,7 +2203,7 @@ main(int argc, char *argv[])
     free_ser_ioinfo(subdata2);
 
     gensio_os_proc_cleanup(proc_data);
-    o->free_funcs(o);
+    gensio_os_funcs_free(o);
     if (keyname)
 	free(keyname);
     if (certname)
