@@ -275,7 +275,7 @@ closecount_decr(struct gdata *ginfo)
     assert(ginfo->closecount > 0);
     ginfo->closecount--;
     if (ginfo->closecount == 0)
-	ginfo->o->wake(ginfo->waiter);
+	gensio_os_funcs_wake(ginfo->o, ginfo->waiter);
 }
 
 static void
@@ -353,11 +353,11 @@ gshutdown(struct ioinfo *ioinfo, bool user_req)
     struct gdata *ginfo = pcinfo->ginfo;
 
     if (user_req) {
-	ginfo->o->wake(ginfo->waiter);
+	gensio_os_funcs_wake(ginfo->o, ginfo->waiter);
     } else {
 	close_con_info(pcinfo);
 	if (ginfo->closecount == 0)
-	    ginfo->o->wake(ginfo->waiter);
+	    gensio_os_funcs_wake(ginfo->o, ginfo->waiter);
     }
 }
 
@@ -1073,12 +1073,19 @@ mux_event(struct gensio *io, void *user_data, int event, int ierr,
 	  unsigned char *buf, gensiods *buflen,
 	  const char *const *auxdata)
 {
+    gensiods len;
+
     switch (event) {
     case GENSIO_EVENT_READ:
     case GENSIO_EVENT_WRITE_READY:
 	abort();
 
     case GENSIO_EVENT_NEW_CHANNEL:
+	/* Enable oob data from the new channel. */
+	len = 1;
+	gensio_control((struct gensio *) buf,
+		       0, false, GENSIO_CONTROL_ENABLE_OOB, "1", &len);
+
 	new_rem_io((struct gensio *) buf, user_data);
 	return 0;
     }
@@ -1091,6 +1098,7 @@ open_mux(struct gensio *io, struct gdata *ginfo, const char *service)
 {
     struct gensio_os_funcs *o = ginfo->o;
     struct gensio *mux_io;
+    gensiods len;
     int err;
     /*
      * The buffer sizes are carefully chosen here to mesh with ssl and
@@ -1113,6 +1121,10 @@ open_mux(struct gensio *io, struct gdata *ginfo, const char *service)
 	exit(1);
     }
 
+    /* Enable OOB data from the mux. */
+    len = 1;
+    gensio_control(mux_io, 0, false, GENSIO_CONTROL_ENABLE_OOB, "1", &len);
+
     err = gensio_open_nochild_s(mux_io);
     if (err) {
 	syslog(LOG_ERR, "mux open failed: %s", gensio_err_to_str(err));
@@ -1139,7 +1151,7 @@ handle_new(struct gensio_runner *r, void *cb_data)
     bool interactive = false;
     unsigned int i;
 
-    o->free_runner(r);
+    gensio_os_funcs_free_runner(o, r);
 
     err = ssl_gensio_alloc(net_io, ssl_args, o, NULL, NULL, &ssl_io);
     if (err) {
@@ -1367,7 +1379,7 @@ acc_event(struct gensio_accepter *accepter, void *user_data,
 	 * so parent doesn't own us.  We have to tell the os handler,
 	 * too that we forked, or epoll() misbehaves.
 	 */
-	err = o->handle_fork(o);
+	err = gensio_os_funcs_handle_fork(o);
 	if (err) {
 	    syslog(LOG_ERR, "Could not fork gensio handler: %s",
 		   gensio_err_to_str(err));
@@ -1405,12 +1417,12 @@ acc_event(struct gensio_accepter *accepter, void *user_data,
 
 	/* Since handle_new does blocking calls, can't do it here. */
 	gensio_set_user_data(io, ginfo); /* Just temporarily. */
-	r = o->alloc_runner(o, handle_new, io);
+	r = gensio_os_funcs_alloc_runner(o, handle_new, io);
 	if (!r) {
 	    syslog(LOG_ERR, "Could not allocate runner");
 	    exit(1);
 	}
-	err = o->run(r);
+	err = gensio_os_funcs_run(o, r);
 	if (err) {
 	    syslog(LOG_ERR, "Could not run runner: %s",
 		   gensio_err_to_str(err));
@@ -1458,6 +1470,7 @@ help(int err)
     printf("     use the name as the PAM program name and run the PAM auth\n");
     printf("     after the certificate auth succeeds.  For 2-factor auth.\n");
     printf("  -P, --pidfile <file> - Create the given pidfile.\n");
+    printf("  --version - Print the version number and exit.\n");
     printf("  -h, --help - This help\n");
     exit(err);
 }
@@ -1566,6 +1579,9 @@ main(int argc, char *argv[])
 	    debug++;
 	    if (debug > 1)
 		gensio_set_log_mask(GENSIO_LOG_MASK_ALL);
+	} else if ((rv = cmparg(argc, argv, &arg, NULL, "--version", NULL))) {
+	    printf("Version %s\n", gensio_version_string);
+	    exit(0);
 	} else if ((rv = cmparg(argc, argv, &arg, "-h", "--help", NULL)))
 	    help(0);
 	else {
@@ -1595,13 +1611,13 @@ main(int argc, char *argv[])
 		gensio_err_to_str(rv));
 	return 1;
     }
-    o->vlog = do_vlog;
+    gensio_os_funcs_set_vlog(o, do_vlog);
 
     rv = gensio_os_proc_setup(o, &proc_data);
     if (rv) {
 	fprintf(stderr, "Could not setup process data: %s\n",
 		gensio_err_to_str(rv));
-	o->free_funcs(o);
+	gensio_os_funcs_free(o);
 	return 1;
     }
 
@@ -1618,7 +1634,7 @@ main(int argc, char *argv[])
 	return 1;
     }
 
-    ginfo.waiter = o->alloc_waiter(o);
+    ginfo.waiter = gensio_os_funcs_alloc_waiter(o);
     if (!ginfo.waiter) {
 	fprintf(stderr, "Could not allocate OS waiter\n");
 	return 1;
@@ -1729,7 +1745,7 @@ main(int argc, char *argv[])
 		exit(1);
 	    }
 	}
-	o->handle_fork(o);
+	gensio_os_funcs_handle_fork(o);
 
 	/* Close all my standard I/O. */
 	if (chdir("/") < 0) {
@@ -1743,7 +1759,7 @@ main(int argc, char *argv[])
 	make_pidfile();
     }
 
-    o->wait(ginfo.waiter, 1, NULL);
+    gensio_os_funcs_wait(o, ginfo.waiter, 1, NULL);
 
     if (tcp_acc) {
 	ginfo.closecount++;
@@ -1778,7 +1794,7 @@ main(int argc, char *argv[])
     close_cons(&ginfo);
 
     if (ginfo.closecount > 0)
-	o->wait(ginfo.waiter, 1, NULL);
+	gensio_os_funcs_wait(o, ginfo.waiter, 1, NULL);
 
     if (tcp_acc)
 	gensio_acc_free(tcp_acc);
@@ -1790,10 +1806,10 @@ main(int argc, char *argv[])
     free(ginfo.key);
     free(ginfo.cert);
 
-    o->free_waiter(ginfo.waiter);
+    gensio_os_funcs_free_waiter(o, ginfo.waiter);
 
     gensio_os_proc_cleanup(proc_data);
-    o->free_funcs(o);
+    gensio_os_funcs_free(o);
 
     if (passwd) {
 	memset(passwd, 0, strlen(passwd));
