@@ -194,6 +194,9 @@ local_port_new_con(struct local_portinfo *pi, struct gensio *io)
 	goto out_err;
     }
 
+    len = 1;
+    gensio_control(pc->io2, 0, false, GENSIO_CONTROL_ENABLE_OOB, "1", &len);
+
     len = strlen(pi->service_str);
     err = gensio_control(pc->io2, 0, GENSIO_CONTROL_SET, GENSIO_CONTROL_SERVICE,
 			 pi->service_str, &len);
